package org.kldp.ebmsh.bi.generator;

import java.io.BufferedReader;
import java.io.InputStreamReader;

import org.kldp.ebmsh.bi.generator.context.ActionHandlerProcesserContext;
import org.kldp.ebmsh.bi.generator.context.Ejb2MDBContext;
import org.kldp.ebmsh.bi.generator.context.Ejb2EjbJarContext;
import org.kldp.ebmsh.bi.generator.context.EjbJarContext;
import org.kldp.ebmsh.bi.generator.context.Jeus5ServerDescriptionContext;
import org.kldp.ebmsh.bi.generator.context.MDBContext;
import org.kldp.ebmsh.bi.generator.context.ServerDescriptionContext;
import org.kldp.ebmsh.bi.generator.source.ActionHandlerProcesserGenerator;
import org.kldp.ebmsh.bi.generator.source.MDBSourceGenerator;
import org.kldp.ebmsh.bi.generator.source.Ejb2MDBSourceGenerator;
import org.kldp.ebmsh.bi.generator.source.Ejb2EjbJarGenerator;
import org.kldp.ebmsh.bi.generator.source.EjbJarGenerator;
import org.kldp.ebmsh.bi.generator.source.Jeus5ServerDescriptionGenerator;
import org.kldp.ebmsh.bi.generator.source.ServerDescriptionGenerator;
import org.kldp.ebmsh.bi.generator.template.TemplateEngine;
import org.kldp.ebmsh.bi.generator.template.VelocityTemplateEngine;

import com.softmeca.util.io.FileIoUtil;
import com.softmeca.util.log.LogUtil;

public class Generator {
	public static void Error(String errorMessage) {
		System.out.println();
		System.out.println("----- Error -----");
		
		System.out.println(errorMessage);
		
		System.out.println("----- Error -----");
		System.out.println();
		
		System.exit(-1);
	}
	
	public static void main(String[] args) {
		try {
			BufferedReader stdin = new BufferedReader(new InputStreamReader(System.in));
			
			System.out.print("Ejb  [ : 2.0] : ");
			String ejbVersion = stdin.readLine();
			System.out.print(" Ÿ [ : jeus5.0] : ");
			String serverType = stdin.readLine();
			System.out.print(" QueueConnectionFactory ̸ [Jeus ⺻ QueueConnectionFactory: queueConnectionFactory] : ");
			String queueConnectionFactoryName = stdin.readLine();
			System.out.print(" Action ̸ : ");
			String actionName = stdin.readLine();
			String outputDirectory = FileIoUtil.fileSeparatorAddOrNone("output") + actionName;
			
			stdin.close();
			
			System.out.println();
			
			System.out.println("----- Է  -----");
			System.out.println("Ejb  : " + ejbVersion);
			System.out.println(" Ÿ : " + serverType);
			System.out.println(" QueueConnectionFactory ̸ : " + queueConnectionFactoryName);
			System.out.println(" Action ̸ : " + actionName);
			System.out.println("----- Է  -----");
			
			System.out.println();
			
			/* Generator init */
			TemplateEngine templateEngine = null;
			EjbJarGenerator ejbJarGenerator = null;
			MDBSourceGenerator mdbGenerator = null;
			ServerDescriptionGenerator serverDescriptionGenerator = null;			
			ActionHandlerProcesserGenerator actionHandlerProcesserGenerator = null;
			
			/* Context init */
			EjbJarContext ejbJarContext = null;
			MDBContext mdbContext = null;
			ServerDescriptionContext serverDescriptionContext = null;
			ActionHandlerProcesserContext actionHandlerProcesserContext = null;
			
			System.out.println("----- ʱȭ -----");
			/* Template engine init */
			System.out.println("Template  ʱȭ մϴ.");			
			templateEngine = new VelocityTemplateEngine();
			
			/* EjbJarGenerator, EjbMDBSourceGenerator init */
			System.out.println("EjbJarGenerator ʱȭ մϴ.");
			if( ejbVersion.equals("2.0") ) {
				/* EjbJar init */
				ejbJarGenerator = new Ejb2EjbJarGenerator(templateEngine);
				
				Ejb2EjbJarContext ejb2EjbJarContext = new Ejb2EjbJarContext();
				ejb2EjbJarContext.setClassName(actionName);
				ejb2EjbJarContext.setEjbName(actionName);
				
				ejbJarContext = ejb2EjbJarContext;
				/* EjbJar init */
			}
			else {
				StringBuffer errorMessage = new StringBuffer();
				
				errorMessage.append(" ʴ ejb version Դϴ.\n");
				errorMessage.append("Է  : " + ejbVersion + "   : 2.0");
				
				Error(errorMessage.toString());
			}
			
			
			/* MDBSourceGenerator init */
			System.out.println("MDBSourceGenerator ʱȭ մϴ.");
			if( ejbVersion.equals("2.0") ) {
				/* MDBSource init */
				mdbGenerator = new Ejb2MDBSourceGenerator(templateEngine);
				
				Ejb2MDBContext ejb2MDBContext = new Ejb2MDBContext();
				
				ejb2MDBContext.setClassName(actionName);
				ejb2MDBContext.setProcesserClassName(actionName + "Processer");
				
				mdbContext = ejb2MDBContext;
				/* MDBSource init */
			}
			else {
				StringBuffer errorMessage = new StringBuffer();
				
				errorMessage.append(" ʴ ejb version Դϴ.\n");
				errorMessage.append("Է  : " + ejbVersion + "   : 2.0");
				
				Error(errorMessage.toString());
			}
			
			
			/* ServerDescriptionGenerator init */
			System.out.println("ServerDescriptionGenerator ʱȭ մϴ.");
			if( serverType.equals("jeus5.0") ) {
				serverDescriptionGenerator = new Jeus5ServerDescriptionGenerator(templateEngine);
				
				Jeus5ServerDescriptionContext jeus5ServerDescriptionContext = new Jeus5ServerDescriptionContext();
				
				jeus5ServerDescriptionContext.setEjbName(actionName);
				jeus5ServerDescriptionContext.setQueueConnectionFactoryName(queueConnectionFactoryName);
				jeus5ServerDescriptionContext.setDestination(actionName);
				
				serverDescriptionContext = jeus5ServerDescriptionContext;
			}
			else {
				StringBuffer errorMessage = new StringBuffer();
				
				errorMessage.append(" ʴ  Ÿ Դϴ.\n");
				errorMessage.append("Է  Ÿ : " + serverType + "   Ÿ : jeus5.0");
				
				Error(errorMessage.toString());
			}
			
			/* Action handler processer init */
			System.out.println("ActionHandlerProcesserGenerator  ʱȭ մϴ.");
			actionHandlerProcesserGenerator = new ActionHandlerProcesserGenerator(templateEngine);			
			actionHandlerProcesserContext = new ActionHandlerProcesserContext();			
			actionHandlerProcesserContext.setProcesserClassName(actionName + "Processer");
			System.out.println("----- ʱȭ -----");
			
			System.out.println();
			
			/* Run */
			System.out.println("-----  -----");
			System.out.println("EjbJarGenerator մϴ.");
			ejbJarGenerator.generate(ejbJarContext, outputDirectory);
			System.out.println("MdbGenerator մϴ.");
			mdbGenerator.generate(mdbContext, outputDirectory);
			System.out.println("ServerDescriptionGenerator մϴ.");
			serverDescriptionGenerator.generate(serverDescriptionContext, outputDirectory);
			System.out.println("ActionHandlerProcesserGenerator մϴ.");
			actionHandlerProcesserGenerator.generate(actionHandlerProcesserContext, outputDirectory);
			System.out.println("-----  -----");
			
			System.out.println();
			
			System.out.println(" 丮 : " + outputDirectory);
			System.out.println(" Ϸ");
		}
		catch(Throwable t) {
			Error(LogUtil.stackTraceToString(t));
		}
	}
}
