package org.kldp.ebmsh.bi.processer;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.Date;

import javax.jms.*;

import org.apache.commons.logging.*;
import org.apache.xerces.parsers.DOMParser;
import org.kldp.ebmsh.bi.environment.EbbiConstants;
import org.kldp.ebmsh.bi.persistant.BiDatabaseManager;
import org.kldp.ebmsh.bi.persistant.BiMessageStatus;
import org.kldp.ebmsh.bi.persistant.BiPersistantException;
import org.kldp.ebmsh.bi.persistant.DocumentIdInfo;
import org.kldp.ebmsh.bi.queue.*;
import org.kldp.ebmsh.bi.tracker.Tracker;
import org.kldp.util.j2ee.resource.QueueConnectionFactoryServiceLocator;
import org.kldp.util.j2ee.resource.QueueServiceLocator;
import org.kldp.util.log.LogUtil;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;

import com.sun.org.apache.xpath.internal.XPathAPI;

/**
 * BsiProcesser
 * 
 * @author Sirius
 */
public abstract class BiProcesser {
	private static Log log = LogFactory.getLog(BiProcesser.class);
	
	public void process(Message message) {
		QueueConnectionFactory queueConnectionFactory = null;
		Queue applicationResultQueue = null;
		BiDatabaseManager bsiDatabaseManager = null;
		BiUserProcesserResult bsiUserProcesserResult = null;
		ToApplicationMessage toApplicationMessage = null;
		
		log.info("JMS ޽ ToApplicationMessage ȯմϴ.");
		
		// JMSMessage ToApplicationMessage ȯ
		toApplicationMessage = (ToApplicationMessage)new ToApplicationMessageConverter().convertToQueueMessage(message);
		
		try {
			Tracker.tracking(toApplicationMessage.getMessageId(), new Date(), "Business logic  մϴ", "");
			
			//   ʱȭ
			queueConnectionFactory = QueueConnectionFactoryServiceLocator.getInstance().getQueueConnectionFactory(EbbiConstants.queueConnectionFactoryName);
			applicationResultQueue = QueueServiceLocator.getInstance().getQueue(EbbiConstants.queueJNDIHeaderName + EbbiConstants.applicationResultQueueName);
			bsiDatabaseManager = new BiDatabaseManager();
			
			log.info("޽ ¸ Receive մϴ.");
			// Application Layer Status Receive · Է
			bsiDatabaseManager.saveMessageStatus(toApplicationMessage.getMessageId(), BiMessageStatus.Receive, "");
						
			// DocumentId  DB ɴϴ.
			Tracker.tracking(toApplicationMessage.getMessageId(), new Date(), " DocumentId ɴϴ", "");
			
			log.info("DB DocumentId  ɴϴ.");
			DocumentIdInfo documentIdInfo = bsiDatabaseManager.getDocumentIdInfoByAction(toApplicationMessage.getAction());
			
			//  DocumentId ɴϴ.
			if( documentIdInfo != null ) {
				int payloadIndex = documentIdInfo.getPayloadIndex();
				String xPath = documentIdInfo.getXPath();
				
				log.info("DocumentId  -> PayloadIndex: " + payloadIndex + "\tXPath: " + xPath);
				
				byte[] payload = toApplicationMessage.getPayloads()[payloadIndex - 1];
				
				InputStream inputStream = new ByteArrayInputStream(payload);
				
				DOMParser parser = new DOMParser();
				parser.parse(new InputSource(inputStream));
				
				Document document = parser.getDocument();
	
				String documentId = XPathAPI.selectSingleNode(document.getFirstChild(), xPath).getFirstChild().getNodeValue();
				
				log.info("DB DocumentId MessageId mapping  Էմϴ.");
				bsiDatabaseManager.saveMessageIdAndDocumentId(toApplicationMessage.getMessageId(), documentId);
			}
			else 
				log.info("DB DocumentId  ϴ.");
			
			log.info("User  processer մϴ.");
			Tracker.tracking(toApplicationMessage.getMessageId(), new Date(), "UserProcess  մϴ", "");
			
			// User process 
			bsiUserProcesserResult = this.userProcesser(toApplicationMessage);
			
			// Application Layer Status Success · Է
			log.info("޽ ¸ Sucess մϴ.");
			bsiDatabaseManager.saveMessageStatus(toApplicationMessage.getMessageId(), BiMessageStatus.Success, "");
		}
		catch(Throwable e) {
			log.fatal(LogUtil.stackTraceToString(e));
			
			Tracker.tracking(toApplicationMessage.getMessageId(), new Date(), "Business logic ó  ߻߽ϴ", e.toString());
			
			// Application Layer Status Fail · Է
			log.info("޽ ¸ Fail մϴ.");
			
			bsiDatabaseManager.saveMessageStatus(toApplicationMessage.getMessageId(), BiMessageStatus.Fail, e.getMessage());
		}
		finally {
			ApplicationResultMessage resultMessage = new ApplicationResultMessage();
			
			try {
				if( bsiUserProcesserResult == null ) {
					// ApplicationResultQueue  ޽  
					resultMessage.setUid(toApplicationMessage.getUid());
					resultMessage.setNullMessage(true);
					resultMessage.setBlock(toApplicationMessage.getBlock());
					resultMessage.setRefToMessageId(toApplicationMessage.getMessageId());
				}
				else {
					// ApplicationResultQueue  ޽ ִ 
					resultMessage.setUid(toApplicationMessage.getUid());
					resultMessage.setNullMessage(false);
					resultMessage.setBlock(toApplicationMessage.getBlock());
					resultMessage.setCpaId(toApplicationMessage.getCpaId());
					resultMessage.setFrom(toApplicationMessage.getTo());
					resultMessage.setTo(toApplicationMessage.getFrom());
					resultMessage.setRefToMessageId(toApplicationMessage.getMessageId());
					resultMessage.setConversationId(toApplicationMessage.getConversationId());
					resultMessage.setService(bsiUserProcesserResult.getService());
					resultMessage.setAction(bsiUserProcesserResult.getAction());
					resultMessage.setPayloadCount(bsiUserProcesserResult.getPayloadCount());
					resultMessage.setPayloads(bsiUserProcesserResult.getPayloads());
				}
				
				log.info("ApplicationResult Queue ޽ մϴ.");
				
				Tracker.tracking(toApplicationMessage.getMessageId(), new Date(), "ApplicationResult Queue ޽ մϴ", "");
				
				QueueHelper queueHelper = new QueueHelper(queueConnectionFactory, applicationResultQueue);				
				queueHelper.sendMessage(resultMessage, new ApplicationResultMessageConverter());
			}
			catch(RuntimeException e) {
				log.fatal(LogUtil.stackTraceToString(e));
				
				Tracker.tracking(toApplicationMessage.getMessageId(), new Date(), "ApplicationResult Queue ޽  ߽ϴ", e.getMessage());
				
				// Application Layer Status Fail · Է
				bsiDatabaseManager.saveMessageStatus(toApplicationMessage.getMessageId(), BiMessageStatus.Fail, e.getMessage());
			}

			Tracker.tracking(toApplicationMessage.getMessageId(), new Date(), "Business logic  մϴ", "");
		}
	}

	
	/**
	 * Version     - v1.0
	 * Create time - May 17, 2007
	 * Update time - May 17, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param toApplicationMessage
	 * @return
	 * @throws Exception
	 * 
	 * User ϴ Ͻ   ޼
	 */
	public abstract BiUserProcesserResult userProcesser(ToApplicationMessage toApplicationMessage) throws BiUserProcesserException;	
}