package org.kldp.ebmsh.bi.queue;

import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.QueueSession;

public class ApplicationResultMessageConverter implements MessageConverter {
	public Message convertToJMSMessage(QueueSession queueSession, QueueMessage queueMessage) throws BiQueueMessageException {
		MapMessage mapMessage = null;
		ApplicationResultMessage message = (ApplicationResultMessage)queueMessage;
		
		validateMessage(message);
		
		try {
			mapMessage = queueSession.createMapMessage();
			
			mapMessage.setLong("uid", message.getUid());
			mapMessage.setBoolean("nullMessage", message.isNullMessage());
			mapMessage.setString("refToMessageId", message.getRefToMessageId());
			mapMessage.setString("cpaId", message.getCpaId());
			mapMessage.setString("from", message.getFrom());
			mapMessage.setString("to", message.getTo());
			mapMessage.setString("action", message.getAction());
			mapMessage.setString("conversationId", message.getConversationId());
			mapMessage.setString("service", message.getService());
			mapMessage.setBoolean("block", message.getBlock());
			
			if( message.getPayloadCount() != 0 ) {
				mapMessage.setInt("payloadCount", message.getPayloadCount());
				
				for(int i = 0; i < message.getPayloadCount(); i++) 
					mapMessage.setBytes("payload" + i, message.getPayloads()[i]);
			}
			else 
				mapMessage.setInt("payloadCount", 0);
		}
		catch(JMSException e) {
			throw new BiQueueMessageException(0, 0, e);
		}
		
		return mapMessage;
	}

	public QueueMessage convertToQueueMessage(Message message) throws BiQueueMessageException {
		MapMessage mapMessage= null;
		ApplicationResultMessage applicationMessage = null;
		
		// message validation
		this.validateMessage(message);
		
		try {
			// ޽ casting
			mapMessage = (MapMessage)message;
			applicationMessage = new ApplicationResultMessage();
			
			applicationMessage.setUid(mapMessage.getLong("uid"));
			applicationMessage.setNullMessage(mapMessage.getBoolean("nullMessage"));
			applicationMessage.setRefToMessageId(mapMessage.getString("refToMessageId"));
			applicationMessage.setCpaId(mapMessage.getString("cpaId"));
			applicationMessage.setFrom(mapMessage.getString("from"));
			applicationMessage.setTo(mapMessage.getString("to"));
			applicationMessage.setAction(mapMessage.getString("action"));
			applicationMessage.setConversationId(mapMessage.getString("conversationId"));
			applicationMessage.setService(mapMessage.getString("service"));
			applicationMessage.setBlock(mapMessage.getBoolean("block"));
			
			int payloadCount = mapMessage.getInt("payloadCount");
			
			if( payloadCount != 0  ) {
				byte[][] payloads = new byte[payloadCount][0];
				
				for(int i = 0; i < payloadCount; i++) 
					payloads[i] = mapMessage.getBytes("payload" + i);
				
				applicationMessage.setPayloadCount(payloadCount);
				applicationMessage.setPayloads(payloads);
			}
			else 
				applicationMessage.setPayloadCount(0);
		}
		catch(JMSException e) {
			throw new BiQueueMessageException(0, 0, e);
		}
		
		return applicationMessage;
	}

	public void validateMessage(Message message) throws BiQueueMessageException {
		try {
			if( message == null )
				throw new IllegalArgumentException("Message nullԴϴ.");
			
			if( !(message instanceof MapMessage) )
				throw new IllegalArgumentException("Message Ÿ MapMessage ƴմϴ.");
			
			MapMessage _mapMessage = (MapMessage)message;
			
			if( !_mapMessage.itemExists("uid") )
				throw new IllegalArgumentException("UID nullԴϴ.");
			
			if( !_mapMessage.itemExists("nullMessage") )
				throw new IllegalArgumentException("NullMessage nullԴϴ.");
			
			if( !_mapMessage.itemExists("cpaId") )
				throw new IllegalArgumentException("CpaId nullԴϴ.");
			
			if( !_mapMessage.itemExists("from") )
				throw new IllegalArgumentException("From nullԴϴ.");
			
			if( !_mapMessage.itemExists("to") ) 
				throw new IllegalArgumentException("To nullԴϴ.");
			
			if( !_mapMessage.itemExists("service") )
				throw new IllegalArgumentException("Service nullԴϴ.");
			
			if( !_mapMessage.itemExists("conversationId") )
				throw new IllegalArgumentException("ConversationId nullԴϴ.");
			
			if( !_mapMessage.itemExists("action") )
				throw new IllegalArgumentException("Action nullԴϴ.");
			
			if( !_mapMessage.itemExists("refToMessageId") )
				throw new IllegalArgumentException("RefToMessageId nullԴϴ.");
	
			if( !_mapMessage.itemExists("block") )
				throw new IllegalArgumentException("Block nullԴϴ.");
			
			if( !_mapMessage.itemExists("payloadCount") )
				throw new IllegalArgumentException("PayloadCount nullԴϴ.");
				
			int payloadCount = _mapMessage.getInt("payloadCount");
			
			if( payloadCount != 0 ) {
				byte[] tmpPayload = _mapMessage.getBytes("payload" + (payloadCount - 1));
				
				if( tmpPayload == null ) {
					throw new IllegalArgumentException("÷ι  ġ ʽϴ.");
				}
			}
		}
		catch(IllegalArgumentException e) {
			throw new BiQueueMessageException(1, 0, e);
		}
		catch(JMSException e) {
			throw new BiQueueMessageException(1, 0, e);
		}
	}

	public void validateMessage(QueueMessage queueMessage) throws BiQueueMessageException {
		try {
			if( queueMessage == null )
				throw new IllegalArgumentException("ApplicationResultMessage nullԴϴ.");
		}
		catch(IllegalArgumentException e) {
			throw new BiQueueException(1, 0, e);
		}
	}
}