package org.kldp.ebmsh.bi.queue;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.kldp.util.log.LogUtil;



/**
 * QueueHelper 
 * 
 * Ư JMS Queue ޽   ϴ Helper Ŭ
 * 
 * @author Sirius
 */
public class QueueHelper {
	protected static Log log = LogFactory.getLog(QueueHelper.class);
	
	protected Queue queue;
	protected QueueConnectionFactory queueConnectionFactory;
	
	public QueueHelper(QueueConnectionFactory queueConnectionFactory, Queue queue) {
		this.queue = queue;
		this.queueConnectionFactory = queueConnectionFactory;
	}
	
	public void sendMessage(QueueMessage queueMessage, MessageConverter messageConverter) throws BiQueueException {
		QueueConnection queueConnection = null;
		QueueSession session = null;
		QueueSender queueSender = null;
		
		try {
			queueConnection = queueConnectionFactory.createQueueConnection();
			session = queueConnection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
			queueSender = session.createSender(queue);
			
			// QueueMessage javax.jms.Message ȯѴ.
			Message message = messageConverter.convertToJMSMessage(session, queueMessage);
			
			// Queue ޽ Ѵ.
			queueSender.send(message);
		}
		catch(JMSException e) {
			throw new BiQueueException(0, 0, e);
		}
		finally {
			if( queueSender != null )
				try {
					queueSender.close();
				}
				catch(JMSException e) {
					log.fatal(LogUtil.stackTraceToString(e));
				}
				
			
			if( session != null )
				try {
					session.close();
				}
				catch(JMSException e) {
					log.fatal(LogUtil.stackTraceToString(e));
				}
			
			if( queueConnection != null )
				try {
					queueConnection.close();
				}
				catch(JMSException e) {
					log.fatal(LogUtil.stackTraceToString(e));
				}
		}
	}
	
	public void sendMessage(Message jmsMessage) throws BiQueueException {
		QueueConnection queueConnection = null;
		QueueSession session = null;
		QueueSender queueSender = null;
		
		try {
			queueConnection = queueConnectionFactory.createQueueConnection();
			session = queueConnection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
			queueSender = session.createSender(queue);
			
			// Queue ޽ Ѵ.
			queueSender.send(jmsMessage);
		}
		catch(JMSException e) {
			throw new BiQueueException(0, 0, e);
		}
		finally {
			if( queueConnection != null ) {
				try {
					queueConnection.close();
				}
				catch(JMSException e) {
					log.fatal(LogUtil.stackTraceToString(e));
				}
			}
		}
	}
}