package org.kldp.ebbp.dao;

import java.util.List;
import java.util.Set;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.LockMode;
import org.hibernate.Query;
import org.hibernate.Transaction;
import org.hibernate.criterion.Example;
import org.kldp.ebbp.instance.Bpinstance;


/**
 * Data access object (DAO) for domain model class Bpinstance.
 * @see com.softmeca.ebbp.hibernate.Bpinstance
 * @author MyEclipse - Hibernate Tools
 */
public class BpinstanceDAO extends BaseHibernateDAO {

    private static final Log log = LogFactory.getLog(BpinstanceDAO.class);

	//property constants
	public static final String BP_ID = "bpId";

    
    public void save(Bpinstance transientInstance) {
        log.debug("saving Bpinstance instance");
        
        Transaction tx = getSession().beginTransaction();
        
        try {
            getSession().save(transientInstance);
            
            tx.commit();
            
            log.debug("save successful");
        } catch (RuntimeException re) {
        	tx.rollback();
        	
            log.error("save failed", re);
            throw re;
        }
        
        getSession().close();
    }
    
    public void saveOrUpdate(Bpinstance bpinstance) {
    	Transaction tx = getSession().beginTransaction();
        
        try {
            getSession().saveOrUpdate(bpinstance);
            
            tx.commit();
            
            log.debug("save or update successful");
        } catch (RuntimeException re) {
        	tx.rollback();
        	
            log.error("save or update failed", re);
            throw re;
        }
        
        getSession().close();
    }
    
	public void delete(Bpinstance persistentInstance) {
        log.debug("deleting Bpinstance instance");
        Transaction tx = getSession().beginTransaction();
        
        try {
            getSession().delete(persistentInstance);
            
            tx.commit();
            
            log.debug("delete successful");
        } catch (RuntimeException re) {
        	tx.rollback();
        	
            log.error("delete failed", re);
            throw re;
        }
        
        getSession().close();
    }
    
    public Bpinstance findById( java.lang.String id) {
        log.debug("getting Bpinstance instance with id: " + id);
        try {
            Bpinstance instance = (Bpinstance) getSession()
                    .get("com.softmeca.ebbp.instance.Bpinstance", id);
            
            getSession().close();
            
            return instance;
        } catch (RuntimeException re) {
            log.error("get failed", re);
            throw re;
        }
        
        
    }
    
    public List<Bpinstance> findAll() {
    	try {
    		String queryString = "from Bpinstance as model";
    		Query queryObject = getSession().createQuery(queryString);
    		
    		getSession().close();
    		
    		return queryObject.list();
    	}
    	catch(RuntimeException re) {
    		log.error("get failed", re);
            throw re;
    	}
    }
    
    
    public List findByExample(Bpinstance instance) {
        log.debug("finding Bpinstance instance by example");
        try {
            List results = getSession()
                    .createCriteria("com.softmeca.ebbp.instance.Bpinstance")
                    .add(Example.create(instance))
            .list();
            log.debug("find by example successful, result size: " + results.size());
            return results;
        } catch (RuntimeException re) {
            log.error("find by example failed", re);
            throw re;
        }
    }    
    
    public List findByProperty(String propertyName, Object value) {
      log.debug("finding Bpinstance instance with property: " + propertyName
            + ", value: " + value);
      try {
         String queryString = "from Bpinstance as model where model." 
         						+ propertyName + "= ?";
         Query queryObject = getSession().createQuery(queryString);
		 queryObject.setParameter(0, value);
		 return queryObject.list();
      } catch (RuntimeException re) {
         log.error("find by property name failed", re);
         throw re;
      }
	}

	public List findByBpId(Object bpId) {
		return findByProperty(BP_ID, bpId);
	}
	
    public Bpinstance merge(Bpinstance detachedInstance) {
        log.debug("merging Bpinstance instance");
        try {
            Bpinstance result = (Bpinstance) getSession()
                    .merge(detachedInstance);
            log.debug("merge successful");
            return result;
        } catch (RuntimeException re) {
            log.error("merge failed", re);
            throw re;
        }
    }

    public void attachDirty(Bpinstance instance) {
        log.debug("attaching dirty Bpinstance instance");
        try {
            getSession().saveOrUpdate(instance);
            log.debug("attach successful");
        } catch (RuntimeException re) {
            log.error("attach failed", re);
            throw re;
        }
    }
    
    public void attachClean(Bpinstance instance) {
        log.debug("attaching clean Bpinstance instance");
        try {
            getSession().lock(instance, LockMode.NONE);
            log.debug("attach successful");
        } catch (RuntimeException re) {
            log.error("attach failed", re);
            throw re;
        }
    }
    
    
}