package org.kldp.ebbp.processspecification.collaboration;

import java.util.ArrayList;

import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;
import org.kldp.ebbp.processspecification.role.Role;
import org.kldp.ebbp.processspecification.state.Decision;
import org.kldp.ebbp.processspecification.state.Failure;
import org.kldp.ebbp.processspecification.state.Start;
import org.kldp.ebbp.processspecification.state.State;
import org.kldp.ebbp.processspecification.state.ToLink;
import org.kldp.ebbp.processspecification.transactionactivity.BusinessTransactionActivity;
import org.kldp.ebbp.processspecification.transactionactivity.TransactionActivity;
import org.oasisOpen.docs.ebxmlBp.ebbp20.BusinessCollaborationType;
import org.oasisOpen.docs.ebxmlBp.ebbp20.BusinessTransactionActivityDocument;
import org.oasisOpen.docs.ebxmlBp.ebbp20.BusinessTransactionType;
import org.oasisOpen.docs.ebxmlBp.ebbp20.CollaborationActivityDocument;
import org.oasisOpen.docs.ebxmlBp.ebbp20.CollaborationActivityType;
import org.oasisOpen.docs.ebxmlBp.ebbp20.CompletionType;
import org.oasisOpen.docs.ebxmlBp.ebbp20.DecisionDocument;
import org.oasisOpen.docs.ebxmlBp.ebbp20.DecisionType;
import org.oasisOpen.docs.ebxmlBp.ebbp20.FailureDocument;
import org.oasisOpen.docs.ebxmlBp.ebbp20.RoleType;
import org.oasisOpen.docs.ebxmlBp.ebbp20.StartType;
import org.oasisOpen.docs.ebxmlBp.ebbp20.TimeToPerformDocument;
import org.oasisOpen.docs.ebxmlBp.ebbp20.ComplexBusinessTransactionActivityDocument.ComplexBusinessTransactionActivity;
import org.oasisOpen.docs.ebxmlBp.ebbp20.FromLinkDocument.FromLink;
import org.oasisOpen.docs.ebxmlBp.ebbp20.impl.BusinessTransactionTypeImpl;
import org.oasisOpen.docs.ebxmlBp.ebbp20.impl.DecisionTypeImpl;


public class BusinessCollaboration extends Collaboration {
	private BusinessCollaborationType businessCollaborationType = null;
	
	public BusinessCollaboration(BusinessCollaborationType businessCollaborationDocumentType) {
		this.businessCollaborationType = businessCollaborationDocumentType;
	}
	
	
	@Override
	public TimeToPerform getTimeToPerform() {
		TimeToPerformDocument.TimeToPerform timeToPerform = businessCollaborationType.getTimeToPerform();
		
		return new TimeToPerform(timeToPerform);
	}
	
	@Override
	public State getStateByNameID(String nameID) {
		String ebbpNamespace = "declare namespace ebbp='http://docs.oasis-open.org/ebxml-bp/ebbp-2.0';";
		
		String pathExpression = "$this/ebbp:*[@nameID='" + nameID  + "']";
		
		XmlObject[] results = businessCollaborationType.selectPath(ebbpNamespace + pathExpression);
		
		if( results[0] instanceof org.oasisOpen.docs.ebxmlBp.ebbp20.BusinessTransactionActivityDocument.BusinessTransactionActivity ) {
			return new BusinessTransactionActivity( (org.oasisOpen.docs.ebxmlBp.ebbp20.BusinessTransactionActivityDocument.BusinessTransactionActivity)results[0]);
		}
		
		else if( results[0] instanceof DecisionType) {
			return new Decision( (DecisionType)results[0] );
		}
		
		else if( results[0] instanceof CompletionType ) {
			CompletionType completionType = (CompletionType)results[0];
			
			return new Failure( (CompletionType)results[0] );
		}
		
		//System.out.println( "nameID ˻ : " + results[0].getClass() );
		
		return null;
	}


	@Override
	public Role getRoleByNameID(String roleNameID) {
		RoleType[] roleTypeArray = businessCollaborationType.getRoleArray();
		
		for(int i = 0; i < roleTypeArray.length; i++) {
			if( roleTypeArray[i].getNameID().equals(roleNameID) )
				return new Role(roleTypeArray[i]);
		}
		
		return null;
	}


	@Override
	public Start getStartState() {
		return new Start( businessCollaborationType.getStart() );
	}
}