package org.kldp.ebbp.processspecification.transactionactivity;

import java.io.Reader;
import java.util.ArrayList;

import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;
import org.kldp.ebbp.processspecification.state.StateInput;
import org.kldp.ebbp.processspecification.state.StateResult;
import org.kldp.ebbp.processspecification.transaction.CommercialTransaction;
import org.kldp.ebbp.processspecification.transaction.Transaction;
import org.oasisOpen.docs.ebxmlBp.ebbp20.BusinessTransactionActivityDocument;
import org.oasisOpen.docs.ebxmlBp.ebbp20.BusinessTransactionHeadDocument;
import org.oasisOpen.docs.ebxmlBp.ebbp20.BusinessTransactionType;
import org.oasisOpen.docs.ebxmlBp.ebbp20.CommercialTransactionDocument;
import org.oasisOpen.docs.ebxmlBp.ebbp20.DecisionType;
import org.oasisOpen.docs.ebxmlBp.ebbp20.FromLinkDocument.FromLink;
import org.oasisOpen.docs.ebxmlBp.ebbp20.impl.BusinessTransactionTypeImpl;
import org.oasisOpen.docs.ebxmlBp.ebbp20.impl.CommercialTransactionDocumentImpl;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;


public class BusinessTransactionActivity extends TransactionActivity  {
	private ArrayList<Performs> performs = null;
	private BusinessTransactionActivityDocument.BusinessTransactionActivity businessTransactionActivity = null;
	
	public BusinessTransactionActivity(BusinessTransactionActivityDocument.BusinessTransactionActivity businessTransactionActivity) {		
		performs = new ArrayList<Performs>();
		
		this.businessTransactionActivity = businessTransactionActivity;
	}
	
	public Transaction getRefBusinessTransaction() {
		String businessTransactionRef = businessTransactionActivity.getBusinessTransactionRef();
		
		String ebbpNamespace = "declare namespace ebbp='http://docs.oasis-open.org/ebxml-bp/ebbp-2.0';";

		String pathExpression = "//ebbp:CommercialTransaction[@nameID='" + businessTransactionRef + "']";
		
		XmlObject[] results = businessTransactionActivity.selectPath(ebbpNamespace + pathExpression);
		
		BusinessTransactionType btti = (BusinessTransactionTypeImpl) results[0];
		
		String nodeName = btti.getDomNode().getNodeName();
		
		if( nodeName.equals("CommercialTransaction") ) 
			return new CommercialTransaction(btti);
		else
			return null;
	}
	
	public String getNameID() {
		return businessTransactionActivity.getNameID();
	}
	
	public StateResult perform(StateInput stateInput) {
		StateResult stateResult = new StateResult();
		
		// BTA code 		
		System.out.println("BTA Perform.");
		
		// Next state find
		String ebbpNamespace = "declare namespace ebbp='http://docs.oasis-open.org/ebxml-bp/ebbp-2.0';";
		
		System.out.println("nameID: " + getNameID());
		
		String pathExpression = "$this/../ebbp:*/ebbp:FromLink[@fromBusinessStateRef='" + getNameID() + "']/..";
		
		XmlObject[] results = businessTransactionActivity.selectPath(ebbpNamespace + pathExpression);
		
		NamedNodeMap nnm = results[0].getDomNode().getAttributes();
		
		for(int i = 0; i < nnm.getLength(); i++) {
			if( nnm.item(i).getNodeName().equals("nameID") ) {
				stateResult.setNextStateNameID(nnm.item(i).getNodeValue());
			}
		}
		
		return stateResult;
	}

}