package org.kldp.ebmsh.queue;

import java.util.Date;

import javax.ejb.EJBException;
import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.xml.soap.SOAPException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.kldp.ebmsh.application.BlockingApplicationProcesserManager;
import org.kldp.ebmsh.message.packaging.EbxmlMessage;
import org.kldp.ebmsh.message.persistant.MessageStatusConstraints;
import org.kldp.ebmsh.message.persistant.PersistantStorage;
import org.kldp.ebmsh.message.transport.EbxmlMessageSender;
import org.kldp.ebmsh.message.util.MessageBuilderUtil;
import org.kldp.ebmsh.message.util.MessageUtil;
import org.kldp.ebmsh.queue.ApplicationResultMessage;
import org.kldp.ebmsh.queue.ApplicationResultMessageConverter;
import org.kldp.ebmsh.tracker.Tracker;
import org.kldp.util.log.LogUtil;


/**
 * ApplicationResultQueueHandler
 * 
 * ø̼ ̾ ó ġ   ޽   ϴ ť ڵ鸵ϴ Ŭ
 * 
 * @author Sirius
 */
public class ApplicationResultQueueHandler implements MessageDrivenBean, MessageListener {
	private final static Log log = LogFactory.getLog(ApplicationResultQueueHandler.class);
	private PersistantStorage storage = null;
	private MessageDrivenContext ctx;
	
	public void ejbCreate() throws EJBException {
		storage = new PersistantStorage();
	}

	public void ejbRemove() throws EJBException {
	}

	public void setMessageDrivenContext(MessageDrivenContext ctx) throws EJBException {
		this.ctx = ctx;
	}

	public void onMessage(Message message) {
		String refToMessageId = null;
		
		try {
			// ޽ 
			log.info("JMS ޽ ApplicationResultQueueMessage ȯմϴ.");
			
			ApplicationResultMessage applicationResultMessage = (ApplicationResultMessage)new ApplicationResultMessageConverter().convertToQueueMessage(message);
			
			refToMessageId = applicationResultMessage.getRefToMessageId();
			
			Tracker.tracking(refToMessageId, new Date(), "Business logic  ó մϴ.", "");
			
			//   ޽ Ұ
			if( !applicationResultMessage.isNullMessage() ) {
				// Sync  SyncApplicationProcesserManager ˸.
				if( applicationResultMessage.getBlock() ) {
					// Application Layer   ApplicationProcesserManager ˷ش.
					BlockingApplicationProcesserManager.getInstance().onMessage(applicationResultMessage);
				}
				// ASync  SendQueue   ޽ ʹ.
				else {
					Tracker.tracking(refToMessageId, new Date(), "  ebXML ޽ մϴ.", "");
					log.info("  ebXML ޽ մϴ.");
					
					String cpaId = applicationResultMessage.getCpaId();
					String from = applicationResultMessage.getFrom();
					String to = applicationResultMessage.getTo();
					String action = applicationResultMessage.getAction();
					String conversationId = applicationResultMessage.getConversationId();
					String service = applicationResultMessage.getService();
					int payloadCount = applicationResultMessage.getPayloadCount();
					byte[][] payloads = applicationResultMessage.getPayloads();
					
					EbxmlMessage resultEbxmlMessage = null;
					
					if( payloadCount == 0 )
						resultEbxmlMessage = MessageBuilderUtil.buildCustomActionMessage(from, to, cpaId, conversationId, action, service);
					else
						resultEbxmlMessage = MessageBuilderUtil.buildCustomActionMessageWithPayloads(from, to, cpaId, conversationId, action, service, payloads);
					
					resultEbxmlMessage.getEbxmlHeader().getMessageHeader().getMessageData().addRefToMessageId().setValue(refToMessageId);
									
					log.info(" ޽  մϴ!.");
					
					new EbxmlMessageSender().sendMessage(resultEbxmlMessage);
				}
			}
			
			log.info("޽ ¸ Success մϴ.");
			
			// ޽  
			storage.saveMessageStatus(refToMessageId, MessageStatusConstraints.Success, "");
			
			Tracker.tracking(refToMessageId, new Date(), "Business logic  ó մϴ.", "");
		}
		catch(SOAPException e) {
			log.fatal(LogUtil.stackTraceToString(e));
			
			Tracker.tracking(refToMessageId, new Date(), "Business logic  ó .", e.getMessage());
			
			// ޽  
			storage.saveMessageStatus(refToMessageId, MessageStatusConstraints.Fail, "");
		}
		catch(RuntimeException e) {
			log.fatal(LogUtil.stackTraceToString(e));
			
			Tracker.tracking(refToMessageId, new Date(), "Business logic  ó .", e.getMessage());
			
			// ޽  
			storage.saveMessageStatus(refToMessageId, MessageStatusConstraints.Fail, "");
			
			throw e;
		}
	}
}