package org.kldp.ebmsh.application;

import java.io.IOException;
import java.io.InputStream;

import org.kldp.ebmsh.config.EbmsConstants;
import org.kldp.ebmsh.message.packaging.EbmsEbxmlMessageException;
import org.kldp.ebmsh.message.packaging.EbxmlMessage;
import org.kldp.ebmsh.queue.EbmsQueueException;
import org.kldp.ebmsh.queue.QueueHelper;
import org.kldp.ebmsh.queue.ToApplicationMessage;
import org.kldp.ebmsh.queue.ToApplicationMessageConverter;
import org.kldp.ebmsh.queue.ToApplicationQueueManager;



/**
 * BlockingApplicationProcesser
 * 
 * Blocking ø̼ μ
 * 
 * Blocking  ¿ ø̼ ̾(Ͻ ) óѴ.
 * 
 * @author Sirius
 */
public class BlockingApplicationProcesser {
	private EbxmlMessage resultEbxmlMessage;
	
	public EbxmlMessage process(EbxmlMessage _ebxmlMessage) throws EbmsApplicationProcesserException {
		try {
			long uid = System.currentTimeMillis();
			
			ToApplicationMessage applicationMessage = new ToApplicationMessage();
			applicationMessage.setUid(uid);
			applicationMessage.setMessageId(_ebxmlMessage.getMessageId());
			applicationMessage.setCpaId(_ebxmlMessage.getCPAId());
			applicationMessage.setFrom(_ebxmlMessage.getFrom());
			applicationMessage.setTo(_ebxmlMessage.getTo());
			applicationMessage.setConversationId(_ebxmlMessage.getConversationId());
			applicationMessage.setAction(_ebxmlMessage.getAction());
			applicationMessage.setService(_ebxmlMessage.getService());
			applicationMessage.setTimestamp(_ebxmlMessage.getTimestamp());
			applicationMessage.setBlock(false);
			
			int payloadCount = _ebxmlMessage.countAttachments();
					
			if( payloadCount != 0 ) {
				byte[][] payloads = new byte[payloadCount][0];
			
				try {
					for(int i = 0; i < payloadCount; i++) {
						InputStream inputStream = _ebxmlMessage.getPayload(i+1).getInputStream();

						StringBuffer sb = new StringBuffer();

						byte[] tmp = new byte[inputStream.available()];
						
						inputStream.read(tmp);

						sb.append(new String(tmp).toCharArray());

						payloads[i] = sb.toString().getBytes();
					}
				}
				catch(IOException e) {
					throw new EbmsEbxmlMessageException(1, 0, "Payload  ϴ.", e);
				}
				
				applicationMessage.setPayloadCount(payloadCount);
				applicationMessage.setPayloads(payloads);
			}
			else {
				applicationMessage.setPayloadCount(0);
			}
			
			
			// ReceiveQueue 
			QueueHelper queueHelper = ToApplicationQueueManager.getInstance().getToApplicationQueueHelper(EbmsConstants.queueJNDIHeaderName + _ebxmlMessage.getAction());
			queueHelper.sendMessage(applicationMessage, new ToApplicationMessageConverter());
			
			// ApplicationProcesserManager  ( Queue ϰ   Ѵ. )
			BlockingApplicationProcesserManager.getInstance().registerBlockingApplicationProcesser(uid, this);
			
			//  ⸦ ٸ
			this.wait();
			
			return resultEbxmlMessage;
		}
		catch(InterruptedException e) {
			throw new EbmsApplicationProcesserException(0, 0, e);
		}
		catch(EbmsEbxmlMessageException e) {
			throw new EbmsApplicationProcesserException(0, 0, e);
		}
		catch(EbmsQueueException e) {
			throw new EbmsApplicationProcesserException(0, 0, e);
		}
	}
	
	public void processNotify(EbxmlMessage ebxmlMessage) {
		resultEbxmlMessage = ebxmlMessage;
	}
}