package org.kldp.ebmsh.application;

import java.util.Hashtable;

import javax.xml.soap.SOAPException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.kldp.ebmsh.message.packaging.EbmsEbxmlMessageException;
import org.kldp.ebmsh.message.packaging.EbxmlMessage;
import org.kldp.ebmsh.message.util.MessageBuilderUtil;
import org.kldp.ebmsh.message.util.MessageUtil;
import org.kldp.ebmsh.queue.ApplicationResultMessage;
import org.kldp.util.log.LogUtil;


/**
 * Blocking application processer manager 
 * 
 * @author Sirius
 */
public class BlockingApplicationProcesserManager {
	private final static Log log = LogFactory.getLog(BlockingApplicationProcesserManager.class);
	private final static BlockingApplicationProcesserManager instance = new BlockingApplicationProcesserManager();
	private Hashtable applicationProcessers = null;
	
	
	/**
	 * Singleton 
	 */
	private BlockingApplicationProcesserManager() {
		applicationProcessers = new Hashtable();
	}
	
	public static BlockingApplicationProcesserManager getInstance() {
		return instance;
	}
	
	
	/**
	 * Blocking application processer ߰ϴ ޼
	 * 
	 * 
	 * @param _uid
	 * @param _applicationProcesser
	 */
	public void registerBlockingApplicationProcesser(long _uid, BlockingApplicationProcesser _applicationProcesser) throws EbmsApplicationProcesserException {
		applicationProcessers.put(new Long(_uid), _applicationProcesser);
	}
	
	
	/**
	 * Blocking application processer ϴ ޼
	 * 
	 * @param _uid
	 * @throws EbmsApplicationProcesserException
	 */
	public void removeBlockingApplicationProcesser(long _uid) throws EbmsApplicationProcesserException {
		applicationProcessers.remove(new Long(_uid));
	}
	
	
	/**
	 * Blocking application layer ó Ϸ ȣѴ.
	 * 
	 * @param applicationResultMessage
	 */
	public void onMessage(ApplicationResultMessage applicationResultMessage) throws EbmsApplicationProcesserException {
		try {
			EbxmlMessage responseEbxmlMessage = null;
			long uid = applicationResultMessage.getUid();
			
			//  ٸ ʴ ޽ ° 
			if( !applicationProcessers.containsKey(new Long(uid)) ) {
				log.fatal(" ٸ ʴ ޽ ߽ϴ. UID -> " + uid);
				
				return ;
			}
			
			// ٸ μ 
			BlockingApplicationProcesser applicationProcesser = (BlockingApplicationProcesser)applicationProcessers.get(new Long(uid));
			
			// ٸ μ Ʈ 
			removeBlockingApplicationProcesser(uid);
			
			try {
				//   EbxmlMessage 
				if( applicationResultMessage.getPayloadCount() == 0 ) {
					responseEbxmlMessage = MessageBuilderUtil.buildCustomActionMessage(applicationResultMessage.getFrom(), applicationResultMessage.getTo(), applicationResultMessage.getCpaId(), applicationResultMessage.getConversationId(), applicationResultMessage.getAction(), applicationResultMessage.getService());
					
					String refToMessageId = applicationResultMessage.getRefToMessageId();
					
					responseEbxmlMessage.getEbxmlHeader().getMessageHeader().getMessageData().addRefToMessageId().setValue(refToMessageId);
					
					responseEbxmlMessage.saveChanges();
				}
				else { 
					responseEbxmlMessage = MessageBuilderUtil.buildCustomActionMessageWithPayloads(applicationResultMessage.getFrom(), applicationResultMessage.getTo(), applicationResultMessage.getCpaId(), applicationResultMessage.getConversationId(), applicationResultMessage.getAction(), applicationResultMessage.getService(), applicationResultMessage.getPayloads());
					
					String refToMessageId = applicationResultMessage.getRefToMessageId();
					
					responseEbxmlMessage.getEbxmlHeader().getMessageHeader().getMessageData().addRefToMessageId().setValue(refToMessageId);
					
					responseEbxmlMessage.saveChanges();
				}
			}
			catch(EbmsEbxmlMessageException e) {
				//   EbxmlMessage  
				log.fatal("  EbxmlMessage   ϴ.\n" + LogUtil.stackTraceToString(e));
				
				responseEbxmlMessage = null;
			}
			
			// Application ó   ˷
			applicationProcesser.processNotify(responseEbxmlMessage);
		}
		catch(SOAPException e) {
			throw new EbmsApplicationProcesserException(1, 0, e);
		}
	}
}