package org.kldp.ebmsh.cpa;

import java.io.ByteArrayInputStream;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.oasisOpen.committees.ebxmlCppa.schema.cppCpa20.CertificateDocument;

public class Certificate {
	CertificateDocument.Certificate certificate = null;
	
	public Certificate(CertificateDocument.Certificate _certificate) {
		this.certificate = _certificate;
	}
	
	public String getCertId() throws EbmsCpaException {
		try {
			return certificate.getCertId();
		}
		catch(Exception e) {
			throw new EbmsCpaException(0, 0, e);
		}
	}
	
	public String[] getKeyNames() throws EbmsCpaException {
		try {
			return certificate.getKeyInfo().getKeyNameArray();
		}
		catch(Exception e) {
			throw new EbmsCpaException(0, 0, e);
		}
	}
	
	public X509Certificate[] getX509Certificate() throws EbmsCpaException {
		try {
			X509Certificate[] x509Certificate = null;
			
			if( certificate.getKeyInfo().getX509DataArray().length != 0) {
				x509Certificate = new X509Certificate[certificate.getKeyInfo().getX509DataArray().length];
				
				for(int i = 0; i < certificate.getKeyInfo().getX509DataArray().length; i++) { 					
					ByteArrayInputStream inStream = new ByteArrayInputStream(certificate.getKeyInfo().getX509DataArray(i).getX509CertificateArray(i));
					CertificateFactory cf = CertificateFactory.getInstance("X.509");
					x509Certificate[i] = (X509Certificate)cf.generateCertificate(inStream);
				}
			}
			
			return x509Certificate;
		}
		catch(Exception e) {
			throw new EbmsCpaException(0, 0, e);
		}
	}
	
	public void setCertId(String _certId) throws EbmsCpaException {
		try {
			certificate.setCertId(_certId);
		}
		catch(Exception e) {
			throw new EbmsCpaException(0, 0, e);
		}
	}
	
	public void setKeyName(int _seq, String _keyName) throws EbmsCpaException {
		try {
			certificate.getKeyInfo().setKeyNameArray(_seq, _keyName);
		}
		catch(Exception e) {
			throw new EbmsCpaException(0, 0, e);
		}
	}
	
	public void setX509Certificate(int _seq, String _certificate) throws EbmsCpaException {		
		try {
			org.w3.x2000.x09.xmldsig.X509DataType dataType = org.w3.x2000.x09.xmldsig.X509DataType.Factory.newInstance();
			
			dataType.setX509CertificateArray(_seq, _certificate.getBytes());
			
			certificate.addNewKeyInfo();
			
			certificate.getKeyInfo().setX509DataArray(_seq, dataType);
		}
		catch(Exception e) {
			throw new EbmsCpaException(0, 0, e);
		}
	}
}