package org.kldp.ebmsh.cpa;

import java.io.File;
import java.io.Serializable;
import java.util.Calendar;

import org.apache.xmlbeans.XmlObject;
import org.kldp.ebmsh.config.EbmsConstants;
import org.oasisOpen.committees.ebxmlCppa.schema.cppCpa20.CollaborationProtocolAgreementDocument;
import org.oasisOpen.committees.ebxmlCppa.schema.cppCpa20.ConstituentDocument;
import org.oasisOpen.committees.ebxmlCppa.schema.cppCpa20.NamespaceSupportedDocument;
import org.oasisOpen.committees.ebxmlCppa.schema.cppCpa20.PackagingDocument;
import org.oasisOpen.committees.ebxmlCppa.schema.cppCpa20.PartyInfoDocument;
import org.oasisOpen.committees.ebxmlCppa.schema.cppCpa20.SimplePartDocument;
import org.oasisOpen.committees.ebxmlCppa.schema.cppCpa20.StatusDocument;
import org.oasisOpen.committees.ebxmlCppa.schema.cppCpa20.StatusValueType;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;



/**
 * CPA 
 * 
 * @author Sirius
 */
public class Cpa {
	private CollaborationProtocolAgreementDocument cpaDoc = null;
	private CollaborationProtocolAgreementDocument.CollaborationProtocolAgreement cpa = null;

	public Cpa() {
		cpaDoc = CollaborationProtocolAgreementDocument.Factory.newInstance();
		cpa = cpaDoc.addNewCollaborationProtocolAgreement();
	}

	public Cpa(CollaborationProtocolAgreementDocument cpaDoc, CollaborationProtocolAgreementDocument.CollaborationProtocolAgreement cpa) {
		this.cpaDoc = cpaDoc;
		this.cpa = cpa;
	}
	
	public void save() throws EbmsCpaException {
		try {
			cpaDoc.save(new File(EbmsConstants.cpaPath + this.getCpaId() + ".xml"));
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}
	
	public void setValue(String pathExpression, String newValue) throws EbmsCpaException {
		try {
			String tpNamespace = "declare namespace tp='http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd';";
			String dsNamespace = "declare namespace ds='http://www.w3.org/2000/09/xmldsig#';";
			String xlinkNamespace = "declare namespace xlink='http://www.w3.org/1999/xlink';";

			XmlObject[] xmlObject = cpa.selectPath(tpNamespace + dsNamespace + xlinkNamespace + pathExpression);

			xmlObject[0].getDomNode().setNodeValue(newValue);
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}

	public String getValue(String pathExpression) throws EbmsCpaException {
		try {
			String tpNamespace = "declare namespace tp='http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd';";
			String dsNamespace = "declare namespace ds='http://www.w3.org/2000/09/xmldsig#';";
			String xlinkNamespace = "declare namespace xlink='http://www.w3.org/1999/xlink';";

			XmlObject[] xmlObject = cpa.selectPath(tpNamespace + dsNamespace + xlinkNamespace + pathExpression);

			return xmlObject[0].getDomNode().getNodeValue();
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}

	public String getCpaToString() throws EbmsCpaException {
		try {
			return cpaDoc.toString();
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}

	/**
	 * CollaborationProtocolAgreement CpaId ȯ
	 * 
	 * @return CpaId
	 */	
	public String getCpaId() throws EbmsCpaException {
		try {
			return cpa.getCpaid();
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}

	/**
	 * CollaborationProtocolAgreement Status ߰
	 * 
	 * @param _status		Status Ʈ 
	 * 
	 * @return  Status
	 */
	public StatusDocument.Status addStatus(StatusValueType.Enum _status) throws EbmsCpaException {
		try {
			StatusDocument.Status status = cpa.addNewStatus();

			status.setValue(_status);

			return status;
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}

	/**
	 * CollaborationProtocolAgreement Status 
	 * 
	 * @param _status		Status Ʈ 
	 * 
	 * @return void
	 */
	public void setStatus(StatusValueType.Enum _status) throws EbmsCpaException {
		try {
			cpa.getStatus().setValue(_status);
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}

	/**
	 * CollaborationProtocolAgreement Status ȯ
	 *  
	 * @return Status Ʈ 
	 */
	public StatusValueType.Enum getStatus() throws EbmsCpaException {
		try {
			return cpa.getStatus().getValue();
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}



	/**
	 * CollaborationProtocolAgreement Start ߰
	 * 
	 * @param _start	 ¥
	 * 
	 * @return  Start
	 */
	public Calendar addStart(Calendar _start) throws EbmsCpaException {
		try {
			cpa.setStart(_start);

			return cpa.getStart();
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}

	/**
	 * CollaborationProtocolAgreement Start 
	 * 
	 * @param _start	 ¥
	 * 
	 * @return void
	 */
	public void setStart(Calendar start) throws EbmsCpaException {
		try {
			cpa.setStart(start);
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}

	/**
	 * CollaborationProtocolAgreement Start ȯ
	 *  
	 * @return Start Ʈ 
	 */
	public Calendar getStart() throws EbmsCpaException {
		try {
			return cpa.getStart();
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}


	/**
	 * CollaborationProtocolAgreement End ߰
	 * 
	 * @param _end	 ¥
	 * 
	 * @return  End
	 */
	public Calendar addEnd(Calendar _end) throws EbmsCpaException {
		try {
			cpa.setEnd(_end);

			return cpa.getEnd();
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}	

	/**
	 * CollaborationProtocolAgreement End 
	 * 
	 * @param _end	 ¥
	 * 
	 * @return void
	 */
	public void setEnd(Calendar end) throws EbmsCpaException {
		try {
			cpa.setEnd(end);
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}

	/**
	 * CollaborationProtocolAgreement End ȯ
	 *  
	 * @return Status Ʈ 
	 */
	public Calendar getEnd() throws EbmsCpaException {
		try {
			return cpa.getEnd();
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}


	/**
	 * CollaborationProtocolAgreement PartyInfo ߰
	 * 
	 * @param _partyName			PartyInfo Attribute 'partyName'
	 * @param _defaultMshChannelId	PartyInfo Attribute 'defaultMshChannelId'
	 * @param _defaultMshPackageId	PartyInfo Attribute 'defaultMshPackageId'
	 * 
	 * @return  PartyInfo
	 */
	public PartyInfo addPartyInfo(String _partyName, String _defaultMshChannelId, String _defaultMshPackageId) throws EbmsCpaException {
		try {
			PartyInfoDocument.PartyInfo partyInfo = cpa.addNewPartyInfo();

			partyInfo.setPartyName(_partyName);
			partyInfo.setDefaultMshChannelId(_defaultMshChannelId);
			partyInfo.setDefaultMshPackageId(_defaultMshPackageId);

			PartyInfo _partyInfo = new PartyInfo(partyInfo);

			return _partyInfo;
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}

	/**
	 * CollaborationProtocolAgreement PartyInfo 
	 * 
	 *   ״ ϰ   ڿ 'null' Է
	 *  
	 * @param _oldPartyName			 partyName
	 * @param _newPartyName			ο partyName
	 * @param _defaultMshChannelId	PartyInfo Attribute 'defaultMshChannelId'
	 * @param _defaultMshPackageId	PartyInfo Attribute 'defaultMshPackageId'
	 * 
	 * @return void
	 */
	public void setPartyInfo(String _oldPartyName, String _newPartyName, String _defaultMshChannelId, String _defaultMshPackageId) throws EbmsCpaException {
		try {
			PartyInfoDocument.PartyInfo[] partyInfos = cpa.getPartyInfoArray();

			for(int i = 0; i < partyInfos.length; i++) {
				if( partyInfos[i].getPartyName().equals(_oldPartyName) ) {
					if(_newPartyName != null)
						partyInfos[i].setPartyName(_newPartyName);

					if(_defaultMshChannelId != null)
						partyInfos[i].setDefaultMshChannelId(_defaultMshChannelId);

					if(_defaultMshPackageId != null)
						partyInfos[i].setDefaultMshPackageId(_defaultMshPackageId);

					break;
				}
			}
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}


	/**
	 * CollaborationProtocolAgreement PartyInfo ȯ
	 *  
	 * @return CPA   PartyInfo
	 */
	public PartyInfo[] getPartyInfos() throws EbmsCpaException {
		try {
			PartyInfo[] partyInfoArray = null;

			PartyInfoDocument.PartyInfo[] partyInfo = cpa.getPartyInfoArray();

			if(partyInfo != null) {
				partyInfoArray = new PartyInfo[partyInfo.length];

				for(int i = 0; i < partyInfoArray.length; i++) 
					partyInfoArray[i] = new PartyInfo(partyInfo[i]);
			}

			return partyInfoArray;
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}

	/**
	 * CollaborationProtocolAgreement PartyInfo ȯ
	 * 
	 * @param _partyName  partyName
	 *  
	 * @return PartyInfo
	 */
	public PartyInfo getPartyInfo(String _partyName) throws EbmsCpaException {
		try {
			PartyInfo partyInfo = null;

			String tpNamespace = "declare namespace tp='http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd';";

			String pathExpression = "$this/tp:PartyInfo[@tp:partyName='" + _partyName + "']";

			XmlObject[] results = cpa.selectPath(tpNamespace + pathExpression);

			if(results.length != 0) 
				partyInfo = new PartyInfo((PartyInfoDocument.PartyInfo)results[0]);

			return partyInfo;
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}


	/**
	 * CPA SimplePart ߰
	 * _nameSpace 'null' Է  NamespaceSupported element  ʴ´.
	 * 
	 *  @param	_id			Simplepart attribute 'id'
	 *  @param	_mimeType	Simplepart attribute 'mimetype'
	 *  @param	_nameSpace 	NamespaceSupported value
	 *  
	 *  @return  SimplePart
	 *  */	
	public SimplePartDocument.SimplePart addSimplePart(String _id, String _mimetype, String _nameSpace) throws EbmsCpaException {
		try {
			SimplePartDocument.SimplePart simplePart = cpa.addNewSimplePart();

			simplePart.setId(_id);
			simplePart.setMimetype(_mimetype);

			if(_nameSpace != null) {
				NamespaceSupportedDocument.NamespaceSupported nameSpaceSupported = simplePart.addNewNamespaceSupported();

				nameSpaceSupported.setLocation("http://www.oasis-open.org/committees/ebxml-msg/schema/msg-header-2_0.xsd");
				nameSpaceSupported.setVersion("2.0");

				nameSpaceSupported.setStringValue(_nameSpace);
			}

			return simplePart;
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}

	/**
	 * CPA SimplePart 
	 *   ״ ϰ   ڿ 'null' Է
	 * 
	 *  @param	_oldId		 Simplepart attribute 'id'
	 *  @param	_newId		ο Simplepart attribute 'id'
	 *  @param	_mimeType	Simplepart attribute 'mimetype'
	 *  @param	_nameSpace 	NamespaceSupported value
	 *  
	 *  @return void
	 *  */	
	public void setSimplePart(String _oldId, String _newId, String _mimeType, String _nameSpace) throws EbmsCpaException {
		try {
			SimplePartDocument.SimplePart[] simplePart = cpa.getSimplePartArray();

			for(int i = 0; i < simplePart.length; i++) {
				if(simplePart[i].getId().equals(_oldId)) {
					if(_newId != null) 
						this.setAttribute(_oldId, _newId);

					if(_mimeType != null)
						simplePart[i].setMimetype(_mimeType);

					if(_nameSpace != null) {
						NamespaceSupportedDocument.NamespaceSupported namespaceSupported = simplePart[i].getNamespaceSupportedArray(0);
						namespaceSupported.setStringValue(_nameSpace);
					}	
				}
			}
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}

	/**
	 * CPA SimplePart Attribute Id ȯ
	 *  
	 * @return  SimplePart id 
	 */
	public String[] getSimplePartIds() throws EbmsCpaException {
		try {
			String[] simplePartId = null;
			SimplePartDocument.SimplePart[] simplePartArray = null;

			simplePartArray = cpa.getSimplePartArray();

			if(simplePartArray != null) {
				simplePartId = new String[simplePartArray.length];

				for(int i = 0; i < simplePartArray.length; i++) 
					simplePartId[i] = simplePartArray[i].getId();
			}

			return simplePartId;
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}


	/**
	 * CPA Packaging element ߰
	 * ( Packaging element - CompositeList - Encapsulation ߰ )
	 * 
	 *  @param	_packagingId						Packaging Id
	 *  @param	_encapsulationId					Encapsulation Id
	 *  @param	_constituentMessageHeaderIdRef 	MessageHeader IdRef
	 *  @param	_constituentPayLoadIdRef 		Payload IdRef
	 *  
	 *  @return  Packaging
	 */	
	public PackagingDocument.Packaging addPackagingwithEncapsulation(String _packagingId, String _encapsulationId, String _constituentMessageHeaderIdRef, String _constituentPayloadIdRef) throws EbmsCpaException {
		try {
			PackagingDocument.Packaging packaging = cpa.addNewPackaging();

			packaging.setId(_packagingId);

			PackagingDocument.Packaging.ProcessingCapabilities processingCapabilities = packaging.addNewProcessingCapabilities();
			processingCapabilities.setGenerate(true);
			processingCapabilities.setParse(true);

			PackagingDocument.Packaging.CompositeList compositeList = packaging.addNewCompositeList();

			PackagingDocument.Packaging.CompositeList.Encapsulation encapsulation = compositeList.addNewEncapsulation();
			encapsulation.setId(_encapsulationId);
			encapsulation.setMimetype("multipart/related");
			encapsulation.setMimeparameters("type=&quot;text/xml&quot;");		

			ConstituentDocument.Constituent constituentMessageHeader = encapsulation.addNewConstituent();
			constituentMessageHeader.setIdref(_constituentMessageHeaderIdRef);

			ConstituentDocument.Constituent constituentPayload = encapsulation.addNewConstituent();
			constituentPayload.setIdref(_constituentPayloadIdRef);
			constituentPayload.setMinOccurs(new java.math.BigInteger("0"));
			constituentPayload.setMaxOccurs(new java.math.BigInteger("99"));

			return packaging;
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}

	/**
	 * CPA Packaging element ߰
	 * ( Packaging element - CompositeList - Composite ߰ )
	 * 
	 *  @param	_packagingId						Packaging Id
	 *  @param	_compositeId						CompositeId Id
	 *  @param	_constituentMessageHeaderIdRef 	MessageHeader IdRef
	 *  @param	_constituentPayLoadIdRef 		Payload IdRef
	 *  
	 *  @return  Packaging
	 *  */	
	public PackagingDocument.Packaging addPackagingwithComposite(String _packagingId, String _compositeId, String _constituentMessageHeaderIdRef, String _constituentPayloadIdRef) throws EbmsCpaException {
		try {
			PackagingDocument.Packaging packaging = cpa.addNewPackaging();

			packaging.setId(_packagingId);

			PackagingDocument.Packaging.ProcessingCapabilities processingCapabilities = packaging.addNewProcessingCapabilities();
			processingCapabilities.setGenerate(true);
			processingCapabilities.setParse(true);

			PackagingDocument.Packaging.CompositeList compositeList = packaging.addNewCompositeList();

			PackagingDocument.Packaging.CompositeList.Composite composite = compositeList.addNewComposite();
			composite.setId(_compositeId);
			composite.setMimetype("multipart/related");
			composite.setMimeparameters("type=&quot;text/xml&quot;");		

			ConstituentDocument.Constituent constituentMessageHeader = composite.addNewConstituent();
			constituentMessageHeader.setIdref(_constituentMessageHeaderIdRef);

			ConstituentDocument.Constituent constituentPayload = composite.addNewConstituent();
			constituentPayload.setIdref(_constituentPayloadIdRef);
			constituentPayload.setMinOccurs(new java.math.BigInteger("0"));
			constituentPayload.setMaxOccurs(new java.math.BigInteger("99"));

			return packaging;
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}


	/**
	 * CPA Packaging element 
	 * Packaging Attribute Id 
	 * 
	 *  @param	_oldPackagingId					 Packaging Id
	 *  @param	_newPackagingId					ο packaging Id
	 *  
	 *  @return void
	 */	
	public void setPackaging(String _oldPackagingId, String _newPackagingId) throws EbmsCpaException {
		try {
			PackagingDocument.Packaging[] packagings = cpa.getPackagingArray();

			for(int i = 0; i < packagings.length; i++) {
				if( packagings[i].getId().equals(_oldPackagingId) ) {
					this.setAttribute(_oldPackagingId, _newPackagingId);

					break;
				}
			}
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}

	/**
	 * Packaging  ִ Composite  
	 * 
	 *   ״ ϰ   ڿ 'null' Է
	 * 
	 *  @param	_packagingId					Composite  ִ Packaging Id
	 *  @param	_oldCompositeId				 Composite Id
	 *  @param	_newCompositeId				ο Composite Id
	 *  @param	_messageHeaderIdRef			ù° Composite  MessageHeaderIdRef
	 *  @param	_payLoadIdRef				ι° Composite  PayLoadIdRef
	 *  
	 *  @return void
	 *  */	
	public void setCompositeUnderPackaging(String _packagingId, String _oldCompositeId, String _newCompositeId, String _messageHeaderIdRef, String _payLoadIdRef) throws EbmsCpaException {
		try {
			PackagingDocument.Packaging[] packagings = cpa.getPackagingArray();

			for(int i = 0; i < packagings.length; i++) {
				if( packagings[i].getId().equals(_packagingId) ) {
					if(_newCompositeId != null)
						this.setAttribute(_oldCompositeId, _newCompositeId);

					if(_messageHeaderIdRef != null) 
						packagings[i].getCompositeListArray(0).getCompositeArray(0).getConstituentArray(0).setIdref(_packagingId);

					if(_payLoadIdRef != null) 
						packagings[i].getCompositeListArray(0).getCompositeArray(1).getConstituentArray(1).setIdref(_payLoadIdRef);

					break;
				}
			}
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}

	/**
	 * Packaging  ִ Encapsulation  
	 * 
	 *   ״ ϰ   ڿ 'null' Է
	 * 
	 *  @param	_packagingId				Composite  ִ Packaging Id
	 *  @param	_oldEncapsulationId			 Encapsulation Id
	 *  @param	_newEncapsulationId			ο Encapsulation Id
	 *  @param	_messageHeaderIdRef			ù° Composite  MessageHeaderIdRef
	 *  @param	_payLoadIdRef				ι° Composite  PayLoadIdRef
	 *  
	 *  @return void
	 *  */	
	public void setEncapsulationUnderPackaging(String _packagingId, String _oldEncapsulationId, String _newEncapsulationId, String _messageHeaderIdRef, String _payLoadIdRef) throws EbmsCpaException {

		try {
			PackagingDocument.Packaging[] packagings = cpa.getPackagingArray();

			for(int i = 0; i < packagings.length; i++) {
				if( packagings[i].getId().equals(_packagingId) ) {
					if(_newEncapsulationId != null)
						this.setAttribute(_oldEncapsulationId, _newEncapsulationId);

					if(_messageHeaderIdRef != "") 					
						packagings[i].getCompositeListArray(0).getEncapsulationArray(0).getConstituent().setIdref(_messageHeaderIdRef);

					if(_payLoadIdRef != "") 
						packagings[i].getCompositeListArray(0).getEncapsulationArray(1).getConstituent().setIdref(_payLoadIdRef);

					break;
				}
			}
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}	

	/**
	 * Packaging  ִ Encapsulation  ȯ
	 *  
	 * @return Packaging Id ȯ
	 */
	public java.lang.String[] getPackagingIds() throws EbmsCpaException {
		try {
			String[] packgingId = null;

			PackagingDocument.Packaging[] packging = cpa.getPackagingArray();

			if(packging != null) {
				packgingId = new String[packging.length];

				for(int i = 0; i < packging.length; i++) 
					packgingId[i] = packging[i].getId();
			}

			return packgingId;
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}	

	/**
	 * CPA ϴ ϳ attribute ο attribute ȯ
	 * 
	 * @param _oldValue  attribute 
	 * @param _newValue ο attribute 
	 *  
	 * @return void
	 */
	protected void setAttribute(String _oldValue, String _newValue) throws EbmsCpaException {
		try {
			String tpNamespace = "declare namespace tp='http://www.oasis-open.org/committees/ebxml-cppa/schema/cpp-cpa-2_0.xsd';";

			String pathExpression = "//*[@*='" + _oldValue + "']";

			XmlObject[] results = cpaDoc.selectPath(tpNamespace + pathExpression);

			for(int i = 0; i < results.length; i++) {
				Node node = results[i].getDomNode();

				NamedNodeMap nnm = node.getAttributes();

				for(int j = 0; j < nnm.getLength(); j++) {
					Node _node = nnm.item(j);

					if(_node.getNodeValue().equals(_oldValue)) 
						_node.setNodeValue(_newValue);
				}
			}
		}
		catch(Throwable t) {
			throw new EbmsCpaException(0, 0, t);
		}
	}
}