//Source file: c:\\code\\com\\softmeca\\ebmsh\\message\\packaging\\impl\\ErrorMessage.java

package org.kldp.ebmsh.message.packaging.impl;

import java.util.ArrayList;
import java.util.Iterator;

import javax.xml.soap.SOAPException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.kldp.ebmsh.message.packaging.Description;
import org.kldp.ebmsh.message.packaging.EbxmlHeader;
import org.kldp.ebmsh.message.packaging.EbxmlMessage;
import org.kldp.ebmsh.message.packaging.EbxmlMessageFactory;
import org.kldp.ebmsh.message.packaging.ErrorCode;
import org.kldp.ebmsh.message.packaging.ErrorElement;
import org.kldp.ebmsh.message.packaging.ErrorList;
import org.kldp.util.log.LogUtil;



/**
@author pio
@version 0.0
@since 2003.1.1
Ebxml ErrorMessage
 */
public class ErrorMessage extends EbxmlMessageImpl 
{

	/**
   @since 2003.1.1
   @roseuid 400B868B01D0
	 */

	private static Log log = LogFactory.getLog( ErrorMessage.class); 

	public ErrorMessage() 
	{

	}

	public ErrorMessage(EbxmlMessage message,EbxmlMessageFactory factory) throws SOAPException 
	{

		super(message,factory);
		
		initMessageHeader();
	}

	/**
   @since 2003.1.1
   error list ɴϴ.
   @return java.util.Iterator
   @roseuid 400B86380126
	 */


	public Iterator getErrors()
	{
		try{

			EbxmlHeaderImpl ebh =(EbxmlHeaderImpl) this.getEbxmlHeader();

			return ebh.getChildElements("ErrorList");
		}catch(SOAPException e)
		{

			log.fatal( LogUtil.stackTraceToString(e)) ;
			return null;
		}



	}


	public ErrorList getErrorList() 
	{
		try{

			EbxmlHeader ebh = this.getEbxmlHeader();
			return ebh.getErrorList();
		}catch(SOAPException e)
		{

			log.fatal( LogUtil.stackTraceToString(e)) ;
			return null;
		}

	}

	/**
   @since 2003.1.1
   Error ä ֽϴ
   @param errorCode
   @param errorString
   @roseuid 400B86C1005B
	 */

	public void setError(String errorCode, String severity,String location,String description) 
	{

		try{

			EbxmlHeader ebh = this.getEbxmlHeader() ;

			log.info( "set error " +  location  + " " +description);

			if(!ebh.hasMessageHeader())
			{
				log.info("initMessageHeader()   ȣؼ MessageHeader ʱȭ ּ.");
			}


			ErrorList el = ebh.getErrorList();

			ErrorElement err = el.addErrorElement();
			err.setErrorCode(errorCode);
			err.setServerity(severity);
			err.setLocation(location);

			Description d = err.addDescription();
			d.setValue(description); 

		}catch(SOAPException e)
		{

			log.fatal( LogUtil.stackTraceToString(e)) ;	
		}
	}
	/* codeContext  Բ ... test case 58°  䱸ϴ  */

	public void setError(String codeContext, String errorCode, String severity,String location,String description) 
	{

		try{


			EbxmlHeader ebh = this.getEbxmlHeader() ;

			log.info( "set error with context " +  location  +" " +  description + " " + codeContext);

			if(!ebh.hasMessageHeader())
			{
				log.info("initMessageHeader()   ȣؼ MessageHeader ʱȭ ּ.");
			}

			ErrorList el = ebh.getErrorList();


			el.setHighestSeverity(ErrorCode.Error);

			ErrorElement err = el.addErrorElement();
			err.setCodeContext(codeContext);
			err.setErrorCode(errorCode);
			err.setServerity(severity);
			err.setLocation(location);

			Description d = err.addDescription();
			d.setValue(description); 

		}catch(SOAPException e)
		{

			log.fatal( LogUtil.stackTraceToString(e)) ;	
		}
	}
	/**
   @since 2003.1.1
   EbxmlMessageImpl  ̺ؼ ڽ ޽ش ʱȭŲ.
   @roseuid 400B917001E4
	 */
	//TODO initMessageHeader() 
	public void initMessageHeader() 
	{

		try{

			EbxmlHeader ebh = this.getEbxmlHeader() ;


			//¿  ּ óѴ. 

			if(!ebh.hasMessageHeader())
			{
				log.info("initMessageHeader()   ȣؼ MessageHeader ʱȭ ּ.");
			}
			ebh.addErrorList();

		}catch(SOAPException e)
		{

			log.fatal( LogUtil.stackTraceToString(e)) ;	
		}

	}

	/**
   @since 2003.1.1
   DeliveryFailure    ƴ ǴѴ.
   @return boolean
   @roseuid 401489CD02CA
	 */
	public boolean isDeliveryFailure() 
	{
		boolean deliveryFailure =false; 

		Iterator i = getErrors();
		while(i.hasNext())
		{
			ErrorList el = (ErrorList)i.next() ;

			try
			{
				Iterator ii = el.getErrorElements();
				while(ii.hasNext())
				{
					ErrorElement err = (ErrorElement)ii.next() ;
					if(	err.getErrorCode().compareTo( ErrorCode.DeliveryFailure) ==0)
					{
						deliveryFailure = true; 
					}

				}
			}catch(SOAPException e)
			{

				log.fatal( LogUtil.stackTraceToString(e)) ;
			}

		}
		return deliveryFailure;
	}

	/**
	  @since 2003.1.1
	    ڵ带  ArrorList ´.
	  @return java.util.ArrayList
	  @throws javax.xml.soap.SOAPException
	  @roseuid 40187038026C
	 */
	public ArrayList getErrorsToArrayList() throws SOAPException 
	{
		ArrayList al = new ArrayList();

		Iterator i = getErrors();
		while(i.hasNext())
		{
			ErrorList el = (ErrorList)i.next() ;

			try
			{
				Iterator ii = el.getErrorElements();
				while(ii.hasNext())
				{
					ErrorElement err = (ErrorElement)ii.next() ;
					al.add(err);

				}
			}catch(SOAPException e)
			{

				log.fatal( LogUtil.stackTraceToString(e)) ;
			}

		}
		return al;
	}
}
