package org.kldp.ebmsh.message.packaging.validation;



import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.kldp.ebmsh.EbmsException;
import org.kldp.ebmsh.message.packaging.EbxmlMessage;
import org.kldp.ebmsh.message.packaging.impl.EbxmlMessageFactoryImpl;
import org.kldp.ebmsh.message.packaging.impl.ErrorMessage;
import org.kldp.ebmsh.message.util.MessageBuilderUtil;
import org.kldp.util.log.LogUtil;


/** 
 * Exception class that can generate ebXML error messages from the information 
 * it has been given.
 * 
 * @author Frankie Lam
 * @version $Revision: 1.1 $
 */
public class EbmsEbxmlValidationException extends EbmsException {


	private static Log log = LogFactory.getLog( EbmsEbxmlValidationException.class);

    private String _errorCode;
    private String _errorString;
    private String _severity;
    private String _location;
    private String _reftomsgid;
  
  

    /** 
     * Constructs a <code>EbxmlValidationException</code> object given its 
     * error code, severity level, error string and the ebXML message being
     * referred.
     * 
     * @param errorCode     One of the error codes defined in ebMS 4.2.3.4.1.
     * @param severity      Severity level of the error, which should be either 
     *                      SEVERITY_WARNING or SEVERITY_ERROR
     * @param errorString   String describing the error.
     * @param location      Location of the message containing the error.
     */
    public EbmsEbxmlValidationException(String errorCode, String severity,
        String errorString, String location) {
       
        this._errorCode = errorCode;
        this._errorString = errorString;
        this._severity = severity;
        this._location = location;
    }
    
    public void setrefToMsgId(String ref)
    {
    	
    	this._reftomsgid = ref;
    }


    /** 
     * Get <code>EbxmlMessage</code> error message from the information given
     * int this object.
     * 
     * @return <code>EbxmlMessage</code> containing the error information.
     */
    public EbxmlMessage getEbxmlMessage(EbxmlMessage msg) {
    	try {
    		ErrorMessage errMsg = MessageBuilderUtil.buildMessageErrorMessage(msg);
    		
    		//TODO ⼭ uri  Ȯϰ  ־ ϴµ  ߻  URI .
    		errMsg.setError("urn:oasis:names:tc:ebxml-msg:service:errors",this._errorCode,this._severity,this._location ,this._errorString);
    		
    		return errMsg;
    	}
    	catch(Exception e) {
    		log.fatal(LogUtil.stackTraceToString(e)) ;
    		
    		return null;
    	}
    	
    }
    
    public String _getErrorCode()
    {	
    	return this._errorCode;
    }

    public String getErrorString() {
    	return this._errorString;
    }
    
    public String getLocation() {
    	return this._location;
    }
}
