package org.kldp.ebmsh.message.persistant;

import java.sql.Connection;
import java.sql.SQLException;

import javax.xml.soap.SOAPException;

import org.apache.commons.dbutils.QueryRunner;
import org.apache.commons.dbutils.ResultSetHandler;
import org.apache.commons.dbutils.handlers.BeanHandler;
import org.kldp.ebmsh.config.EbmsConstants;
import org.kldp.ebmsh.message.packaging.EbxmlMessage;
import org.kldp.util.io.FileIoUtil;


/**
 * Bizbox_AckMessage DAO
 * 
 * @author Sirius
 * @since 2007-03-30
 */
public class AckMessageDAO {
	/**
	 * RefToMessageId AckMessage 
	 * 
	 * @param refToMessageId
	 * @return
	 * @throws EbmsPersistantException
	 */
	public AckMessage getAckMessageByRefToMessageId(String refToMessageId) throws EbmsPersistantException {
		try {
			AckMessage ackMessage = null;
			
			DbTemplate dbTemplate = new DbTemplate();
			
			final String queryStr = "select * from " + EbmsConstants.bizboxDatabaseHeaderName + "AckMessage where RefToMessageId=?";
			
			final Object[] parameters = new Object[1];
			parameters[0] = refToMessageId; // [0] MessageId
			
			ackMessage = (AckMessage)dbTemplate.executeQuery(
					new DbWorker() {
						
						public Object work(Connection _conn) throws EbmsPersistantException {
							QueryRunner runner = new QueryRunner();
							ResultSetHandler resultSetHandler = new BeanHandler(AckMessage.class);
							
							AckMessage ackMessage = null;
							
							try {
								ackMessage = (AckMessage)runner.query(_conn, queryStr, parameters, resultSetHandler);
							}
							catch(SQLException e) {
								throw new EbmsPersistantException(0, e.getErrorCode(), e);
							}
							
							return ackMessage;
						}
						
					});
			
			return ackMessage;
		}
		catch(EbmsPersistantException e) {
			throw e;
		}
	}
	
	/**
	 * AckMessage  
	 * 
	 * @param ackMessage
	 * @throws EbmsPersistantException
	 */
	public void insertAckMessage(EbxmlMessage ackMessage) throws EbmsPersistantException {
		try {
			String messageId = ackMessage.getMessageId();
			String refToMessageId = ackMessage.getEbxmlHeader().getAcknowledgment().getRefToMessageId().getValue();
			
			String ackFilePath = FileIoUtil.fileSeparatorAddOrNone("outgoing") + FileIoUtil.fileSeparatorAddOrNone("ackmessage") + messageId;
			
			final String queryStr = "insert into " + EbmsConstants.bizboxDatabaseHeaderName + "AckMessage(MessageId, RefToMessageId, ACKFileName) values(?, ?, ?)";
			
			final Object[] parameters = new Object[3];
			parameters[0] = messageId; // [0] MessageId
			parameters[1] = refToMessageId; // [1] RefToMessageId
			parameters[2] = ackFilePath; // [2] ACKFileName
			
			DbTemplate dbTemplate = new DbTemplate();
			dbTemplate.executeQuery(
					new DbWorker() {
						public Object work(Connection _conn) throws EbmsPersistantException {
							QueryRunner runner = new QueryRunner();
							
							try {
								runner.update(_conn, queryStr, parameters);
							} catch (SQLException e) {
								throw new EbmsPersistantException(1, e.getErrorCode(), e);
							}
							
							return null;
						}
					}
				);
		}
		catch(SOAPException e) {
			throw new EbmsPersistantException(1, 0, e);
		}
		catch(EbmsPersistantException e) {
			throw e;
		}
	}
	
	/**
	 * AckMessage  Ʈմϴ.
	 * 
	 * @param ebxmlMessage
	 * @param receiveTime
	 * @throws EbmsPersistantException
	 */
	public void updateAckMessage(EbxmlMessage ackMessage) throws EbmsPersistantException {
		try {
			String messageId = ackMessage.getMessageId();
			String refToMessageId = ackMessage.getEbxmlHeader().getAcknowledgment().getRefToMessageId().getValue();
			
			String ackFilePath = FileIoUtil.fileSeparatorAddOrNone("outgoing") + FileIoUtil.fileSeparatorAddOrNone("ackmessage") + messageId;
			
			final String queryStr = "update " + EbmsConstants.bizboxDatabaseHeaderName + "AckMessage set RefToMessageId=?, ACKFileName=? where MessageId=?";

			final Object[] parameters = new Object[3];
			parameters[0] = refToMessageId; // [0] ReceiveTime
			parameters[1] = ackFilePath; // [1] MSGFilePath
			parameters[2] = messageId; // [2] MessageId

			DbTemplate dbTemplate = new DbTemplate();
			dbTemplate.executeQuery(
					new DbWorker() {
						public Object work(Connection _conn) throws EbmsPersistantException {
							QueryRunner runner = new QueryRunner();

							try {
								runner.update(_conn, queryStr, parameters);
							} catch (SQLException e) {
								throw new EbmsPersistantException(1, e.getErrorCode(), e);
							}

							return null;
						}
					}
			);
		}
		catch(SOAPException e) {
			throw new EbmsPersistantException(1, 0, e);
		}
		catch(EbmsPersistantException e) {
			throw e;
		}
	}
}
