package org.kldp.ebmsh.message.persistant;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Timestamp;

import org.apache.commons.dbutils.QueryRunner;
import org.apache.commons.dbutils.ResultSetHandler;
import org.apache.commons.dbutils.handlers.BeanHandler;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.kldp.ebmsh.config.EbmsConstants;
import org.kldp.ebmsh.message.packaging.EbxmlMessage;


/**
 * " + Environment.bizboxDatabaseName + "IncomingMessage DAO
 * 
 * @author Sirius
 * @since 2007-03-30
 */
public class IncomingMessageDAO {
	private final static Log log = LogFactory.getLog(IncomingMessageDAO.class);

	/**
	 * InComingMessage  մϴ.
	 * 
	 * @param ebxmlMessage
	 * @param receiveTime
	 * @throws EbmsPersistantException
	 */
	public void insertIncomingMessage(EbxmlMessage ebxmlMessage, long receiveTime, String messageFilePath) throws EbmsPersistantException {
		log.info("DB IncomingMessage   մϴ");
		
		try {
			String messageId = ebxmlMessage.getMessageId();
			Timestamp receiveTimeTimestamp = new Timestamp(receiveTime);

			final String queryStr = "insert into " + EbmsConstants.bizboxDatabaseHeaderName + "InComingMessage(MessageId, ReceiveTime, MSGFilePath) values(?, ?, ?)";

			final Object[] parameters = new Object[3];
			parameters[0] = messageId; // [0] MessageId
			parameters[1] = receiveTimeTimestamp; // [1] ReceiveTime
			parameters[2] = messageFilePath;

			DbTemplate dbTemplate = new DbTemplate();
			dbTemplate.executeQuery(
					new DbWorker() {
						public Object work(Connection _conn) throws EbmsPersistantException {
							QueryRunner runner = new QueryRunner();

							try {
								runner.update(_conn, queryStr, parameters);
							} catch (SQLException e) {
								throw new EbmsPersistantException(1, e.getErrorCode(), e);
							}

							return null;
						}
					}
			);
		}
		catch(EbmsPersistantException e) {
			throw e;
		}
	}
	
	/**
	 * InComingMessage  Ʈմϴ.
	 * 
	 * @param ebxmlMessage
	 * @param receiveTime
	 * @throws EbmsPersistantException
	 */
	public void updateIncomingMessage(EbxmlMessage ebxmlMessage, long receiveTime, String messageFilePath) throws EbmsPersistantException {
		try {
			String messageId = ebxmlMessage.getMessageId();
			Timestamp receiveTimeTimestamp = new Timestamp(receiveTime);

			final String queryStr = "update " + EbmsConstants.bizboxDatabaseHeaderName + "InComingMessage set ReceiveTime=?, MSGFilePath=? where MessageId=?";

			final Object[] parameters = new Object[3];
			parameters[0] = receiveTimeTimestamp; // [0] ReceiveTime
			parameters[1] = messageFilePath; // [1] MSGFilePath
			parameters[2] = messageId; // [2] MessageId

			DbTemplate dbTemplate = new DbTemplate();
			dbTemplate.executeQuery(
					new DbWorker() {
						public Object work(Connection _conn) throws EbmsPersistantException {
							QueryRunner runner = new QueryRunner();

							try {
								runner.update(_conn, queryStr, parameters);
							} catch (SQLException e) {
								throw new EbmsPersistantException(1, e.getErrorCode(), e);
							}

							return null;
						}
					}
			);
		}
		catch(EbmsPersistantException e) {
			throw e;
		}
	}
	
	
	/**
	 * MessageId شϴ InConmingMessage 
	 * 
	 * @param startTime
	 * @param endTime
	 * @return
	 * @throws EbmsPersistantException
	 */
	IncomingMessage getInComingMessage(String messageId) throws EbmsPersistantException {
		try {
			IncomingMessage inComingMessage = null;
			
			DbTemplate dbTemplate = new DbTemplate();
			
			final String queryStr = "select * from " + EbmsConstants.bizboxDatabaseHeaderName + "InComingMessage where MessageId=?";
			
			final Object[] params = new Object[] { messageId };
			
			inComingMessage = (IncomingMessage)dbTemplate.executeQuery(
					new DbWorker() {
						
						public Object work(Connection _conn) throws EbmsPersistantException {
							QueryRunner runner = new QueryRunner();
							ResultSetHandler resultSetHandler = new BeanHandler(IncomingMessage.class);
							
							IncomingMessage queryResult = null;
							
							try {
								queryResult = (IncomingMessage)runner.query(_conn, queryStr, params, resultSetHandler);
							}
							catch(SQLException e) {
								throw new EbmsPersistantException(0, e.getErrorCode(), e);
							}
							
							return queryResult;
						}
						
					});
			
			return inComingMessage;
		}
		catch(EbmsPersistantException e) {
			throw e;
		}
	}
}