package org.kldp.ebmsh.message.persistant;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Timestamp;

import org.apache.commons.dbutils.QueryRunner;
import org.apache.commons.dbutils.ResultSetHandler;
import org.apache.commons.dbutils.handlers.BeanHandler;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.kldp.ebmsh.config.EbmsConstants;
import org.kldp.ebmsh.message.packaging.EbxmlMessage;
import org.kldp.ebmsh.message.util.MessageUtil;


/**
 * " + Environment.bizboxDatabaseName + "MessageStore DAO
 * 
 * @author Sirius
 * @since 2007-03-30
 */
public class MessageStoreDAO {
	private final static Log log = LogFactory.getLog(MessageStoreDAO.class);
	
	/**
	 * ޽  մϴ.
	 * 
	 * @param ebxmlMessage
	 * @throws EbmsPersistantException
	 */
	public void insertMessageStore(EbxmlMessage ebxmlMessage) throws EbmsPersistantException {
		try {
			final String queryStr = "insert into " + EbmsConstants.bizboxDatabaseHeaderName + "MessageStore(MessageId, CpaId, FromPartyId, ToPartyId, ConversationId, Service, Action, TimeStamp) values(?, ?, ?, ?, ?, ?, ?, ?)";
			
			String messageId = ebxmlMessage.getMessageId();
			String cpaId = ebxmlMessage.getCPAId();
			String from = ebxmlMessage.getFrom();
			String to = ebxmlMessage.getTo();
			String conversationId = ebxmlMessage.getConversationId();
			String service = ebxmlMessage.getService();
			String action = ebxmlMessage.getAction();
			
			java.util.Date messageDate = MessageUtil.utcToDate( ebxmlMessage.getTimestamp() );
			Timestamp timestamp = new Timestamp( messageDate.getTime() );
			
			final Object[] parameters = new Object[8];
			parameters[0] = messageId; // [0] MessageId
			parameters[1] = cpaId; // [1] CpaId
			parameters[2] = from; // [2] FromPartyId
			parameters[3] = to; // [3] ToPartyId
			parameters[4] = conversationId; // [4] ConversationId
			parameters[5] = service; // [5] Service
			parameters[6] = action; // [6] Action
			parameters[7] = timestamp; // [7] Timestamp
			
			
			DbTemplate dbTemplate = new DbTemplate();
			dbTemplate.executeQuery(
					new DbWorker() {
						public Object work(Connection _conn) throws EbmsPersistantException {
							QueryRunner runner = new QueryRunner();
							
							try {
								runner.update(_conn, queryStr, parameters);
							} catch (SQLException e) {
								throw new EbmsPersistantException(1, e.getErrorCode(), e);
							}
							
							return null;
						}
					}
				);
		}
		catch(EbmsPersistantException e) {
			throw e;
		}
	}
	
	
	/**
	 * ޽  Ʈմϴ.
	 * 
	 * @param ebxmlMessage
	 * @throws EbmsPersistantException
	 */
	public void updateMessageStore(EbxmlMessage ebxmlMessage) throws EbmsPersistantException {
		try {
			final String queryStr = "update " + EbmsConstants.bizboxDatabaseHeaderName + "MessageStore set CpaId=?, FromPartyId=?, ToPartyId=?, ConversationId=?, Service=?, Action=?, TimeStamp=? where MessageId=?";
			
			String messageId = ebxmlMessage.getMessageId();
			String cpaId = ebxmlMessage.getCPAId();
			String from = ebxmlMessage.getFrom();
			String to = ebxmlMessage.getTo();
			String conversationId = ebxmlMessage.getConversationId();
			String service = ebxmlMessage.getService();
			String action = ebxmlMessage.getAction();
			
			java.util.Date messageDate = MessageUtil.utcToDate( ebxmlMessage.getTimestamp() );
			Timestamp timestamp = new Timestamp( messageDate.getTime() );
			
			final Object[] parameters = new Object[8];
			parameters[0] = cpaId; // [0] CpaId
			parameters[1] = from; // [1] FromPartyId
			parameters[2] = to; // [2] ToPartyId
			parameters[3] = conversationId; // [3] ConversationId
			parameters[4] = service; // [4] Service
			parameters[5] = action; // [5] Action
			parameters[6] = timestamp; // [6] Timestamp
			parameters[7] = messageId; // [7] MessageId
			
			
			DbTemplate dbTemplate = new DbTemplate();
			dbTemplate.executeQuery(
					new DbWorker() {
						public Object work(Connection _conn) throws EbmsPersistantException {
							QueryRunner runner = new QueryRunner();
							
							try {
								runner.update(_conn, queryStr, parameters);
							} catch (SQLException e) {
								throw new EbmsPersistantException(1, e.getErrorCode(), e);
							}
							
							return null;
						}
					}
				);
		}
		catch(EbmsPersistantException e) {
			throw e;
		}
	}
	
	
	/**
	 * messageId شϴ ޽  ˻մϴ.
	 * 
	 * @param messageId
	 * @return
	 * @throws EbmsPersistantException
	 */
	public MessageStore getMessageStoreMessage(String messageId) throws EbmsPersistantException {
		try {
			MessageStore messageStore = null;
			
			DbTemplate dbTemplate = new DbTemplate();
			
			final String queryStr = "select * from " + EbmsConstants.bizboxDatabaseHeaderName + "Messagestore where MessageId=?";
			
			final Object[] parameters = new Object[1];
			parameters[0] = messageId; // [0] MessageId
			
			messageStore = (MessageStore)dbTemplate.executeQuery(
					new DbWorker() {
						
						public Object work(Connection _conn) throws EbmsPersistantException {
							QueryRunner runner = new QueryRunner();
							ResultSetHandler resultSetHandler = new BeanHandler(MessageStore.class);
							
							MessageStore messageStore = null;
							
							try {
								messageStore = (MessageStore)runner.query(_conn, queryStr, parameters, resultSetHandler);
							}
							catch(SQLException e) {
								throw new EbmsPersistantException(0, e.getErrorCode(), e);
							}
							
							return messageStore;
						}
						
					});
			
			return messageStore;
		}
		catch(EbmsPersistantException e) {
			throw e;
		}
	}
}
