package org.kldp.ebmsh.message.processer;

import java.util.Date;

import javax.xml.soap.SOAPException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.kldp.ebmsh.EbmsException;
import org.kldp.ebmsh.application.EbmsApplicationProcesserException;
import org.kldp.ebmsh.message.packaging.EbmsEbxmlMessageException;
import org.kldp.ebmsh.message.packaging.EbxmlMessage;
import org.kldp.ebmsh.message.packaging.ErrorCode;
import org.kldp.ebmsh.message.packaging.impl.ErrorMessage;
import org.kldp.ebmsh.message.packaging.validation.EbmsEbxmlValidationException;
import org.kldp.ebmsh.message.packaging.validation.EbxmlMessageValidator;
import org.kldp.ebmsh.message.persistant.MessageStatusConstraints;
import org.kldp.ebmsh.message.persistant.PersistantStorage;
import org.kldp.ebmsh.message.transport.EbmsTransportException;
import org.kldp.ebmsh.message.transport.rm.AckReceiveNotifier;
import org.kldp.ebmsh.message.util.MessageBuilderUtil;
import org.kldp.ebmsh.message.util.MessageUtil;
import org.kldp.ebmsh.tracker.Tracker;
import org.kldp.util.log.LogUtil;


/**
 * File name   - MessageProcesser.java
 * Create date - May 10, 2007
 * Update time - May 10, 2007
 * Version     - v1.0
 *
 * Author      - Sirius
 */
public abstract class MessageProcesser {
	protected final static Log log = LogFactory.getLog(MessageProcesser.class);
	
	protected PersistantStorage persistantStorage = null;
	
	/**
	 * Constructor
	 * 
	 * PersistantStorage ʱȭմϴ.
	 */
	protected MessageProcesser() {
		persistantStorage = new PersistantStorage();
	}
	
	
	/**
	 * Version     - v1.0
	 * Create time - May 10, 2007
	 * Update time - May 10, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 * @return
	 * @throws EbmsEbxmlMessageException
	 */
	private int ebXMLMessageSort(EbxmlMessage ebxmlMessage) throws EbmsEbxmlMessageException {
		try {
			// Message sorting
			Tracker.tracking(ebxmlMessage.getMessageId(), new Date(), "޽ з մϴ.", "");
			
			// ޽ ¸ 'Normal' ʱȭ
			int statusCode = MessageStatusCode.Normal;
			
			// AckRequested ˻ 
			if( ebxmlMessage.getEbxmlHeader().hasAckRequested() ) 
				statusCode += MessageStatusCode.AckRequested;
			
			// DuplicateElimination ˻ 
			if( ebxmlMessage.getEbxmlHeader().getMessageHeader().hasDuplicateElimination() )
				statusCode += MessageStatusCode.DuplicateElimination;
			
			// MessageOrder ˻ 
			if( ebxmlMessage.getEbxmlHeader().hasMessageOrder() )
				statusCode += MessageStatusCode.MessageOrder;
			
			return statusCode;
		}
		catch(SOAPException e) {
			throw new EbmsEbxmlMessageException(1, 0, e);
		}
	}
	
	
	/**
	 * Version     - v1.0
	 * Create time - May 10, 2007
	 * Update time - May 10, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 * @throws EbmsMessageProcesserException
	 */
	protected void ebXMLMessageProcess(EbxmlMessage ebxmlMessage) throws EbmsMessageProcesserException {
		log.info("Start MessageProcesser.");
		
		String messageId = null;
		
		try {
			// MessageId ɴϴ.
			messageId = ebxmlMessage.getMessageId();
			
			// ޽  Database( Incoming message ) File մϴ.
			log.info(" Save ebXML message to Incoming message store and File system.");
			persistantStorage.saveIncomingMessageDbAndFile(ebxmlMessage);

			// Tracking  
			Tracker.tracking(messageId, new Date(), "޽ μ  մϴ.", "");
			
			// ޽ ¸ Receive մϴ.
			log.info(" Save status( Receive ) of ebXML message.");
			persistantStorage.saveMessageStatus(ebxmlMessage, MessageStatusConstraints.Receive, "");
			
			
			// ebXML ޽ մϴ.
			try {
				log.info(" Start ebXML message validation.");
				
				new EbxmlMessageValidator().validate(ebxmlMessage);
			}
			catch(EbmsEbxmlValidationException e) {
				// ޽  
				log.fatal(" [ Error ] - ebXML message validation fail.");
				log.fatal("  " + LogUtil.stackTraceToString(e));
				
				// Tracking  
				Tracker.tracking(messageId, new Date(), "޽  .", e.getErrorString());
				
				persistantStorage.saveMessageStatus(ebxmlMessage, MessageStatusConstraints.Fail, "");
				
				//  ޽ մϴ.
				Tracker.tracking(messageId, new Date(), " ޽ մϴ", "");
				
				log.fatal("  Creating error ebXML message.");
				EbxmlMessage errorEbxmlMessage = (EbxmlMessage)e.getEbxmlMessage(ebxmlMessage);
				
				// Validation error handler մϴ.
				log.fatal("  Send error ebXML message.");
				
				try {
					this.sendValidationErrorMessage(e, errorEbxmlMessage);
				}
				catch(EbmsTransportException ex) {
					log.fatal("   [ Error ] - Can't send error ebXML message.");
					log.fatal("    " + LogUtil.stackTraceToString(ex));
					
					Tracker.tracking(messageId, new Date(), " ޽  ߽ϴ.", ex.getMessage());
					
					throw ex;
				}

				throw e;
			}

			
			// ޽  Database( MessageStore ) մϴ.
			log.info(" Save ebXML message to MessageStore.");
			persistantStorage.saveMessageStore(ebxmlMessage);
			
			// ebXML ޽ з մϴ.
			log.info(" Sort ebXML message.");
			int statusCode = ebXMLMessageSort(ebxmlMessage);
			
			log.info("  Sort result of ebXML message - " + statusCode);
			
			Tracker.tracking(ebxmlMessage.getMessageId(), new Date(), "StatusCode -> " + statusCode + " μ մϴ.", "");
			
			// ebXML ޽ ¿  ش ó  ޽ մϴ.
			boolean continueProcessapplicationLayer = false;
			
			switch(statusCode) {
				case MessageStatusCode.Normal :
					continueProcessapplicationLayer = Normal(ebxmlMessage);
					break;
			
				case MessageStatusCode.Ar :
					continueProcessapplicationLayer = AckRequested(ebxmlMessage);
					break;
					
				case MessageStatusCode.De :
					continueProcessapplicationLayer = DuplicateElimination(ebxmlMessage);
					break;
					
				case MessageStatusCode.DeAr :
					continueProcessapplicationLayer = DuplicateElimination_AckRequested(ebxmlMessage);
					break;
					
				default :
					throw new EbmsMessageProcesserException(1, 0, "ó   ޽ ŸԴϴ.");
			}
			
			if( continueProcessapplicationLayer ) {
				log.info("Application layer μ մϴ.");
				
				Tracker.tracking(ebxmlMessage.getMessageId(), new Date(), "Application layer μ մϴ.", "");
				
				// ޽ ¸ ApplicationLayerProcess 
				log.info("Save status( ApplicationLayerProcess ) of ebXML message.");
				persistantStorage.saveMessageStatus(ebxmlMessage, MessageStatusConstraints.ApplicationLayerProcess, "");
				
				// ý ޽ Ϲ ޽ ؼ óؾѴ.
				EbxmlMessage responseEbxmlMessage = null;
				
				// ý  action  ޽
				if( MessageUtil.isSystemLevelActionMessage(ebxmlMessage) ) {
					String action = ebxmlMessage.getAction();
					
					log.info("System level action ޽ Դϴ. Action -> " + action);
					
					if( action.equals("Ping") ) {
						log.info("Ping ޽ óմϴ.");
						
						responseEbxmlMessage = this.systemLevelPingMessageProcess(ebxmlMessage);
					}
					
					else if( action.equals("Pong") ) {
						log.info("Pong ޽ óմϴ.");
						
						responseEbxmlMessage = this.systemLevelPongMessageProcess(ebxmlMessage);
					}
					
					else if( action.equals("MessageStatusRequest") ) {
						log.info("MessageStatusRequest ޽ óմϴ.");
						log.error("MessageStatusRequest  ʾҽϴ.");
						
						// responseEbxmlMessage = this.systemLevelMessageStatusRequestMessageProcess(ebxmlMessage);
					}
					
					else if( action.equals("MessageStatusResponse") ) {
						log.info("MessageStatusResponse ޽ óմϴ.");
						
						responseEbxmlMessage = this.systemLevelMessageStatusResponseMessageProcess(ebxmlMessage);
					}
					
					else if( action.equals("Acknowledgment") ) {
						log.info("Acknowledgment ޽ óմϴ.");
						
						responseEbxmlMessage = this.systemLevelAcknowledgmentMessageProcess(ebxmlMessage);
					}
					
					else if( action.equals("MessageError") ) {
						log.info("MessageError ޽ óմϴ.");
						
						responseEbxmlMessage = this.systemLevelMessageErrorMessageProcess(ebxmlMessage);
					}
				}
				// Ϲ action  ޽
				else {
					log.info(ebxmlMessage.getAction() + " ޽ óմϴ.");
					
					responseEbxmlMessage = this.applicationLevelMessageProcess(ebxmlMessage);
				}
				
				//  ޽ 
				if( responseEbxmlMessage != null ) {
					log.info(" 濡  ޽ մϴ.");
					
					this.sendResponseEbxmlMessage(responseEbxmlMessage);
				}
			}
			
			// ޽ ¸ Success մϴ.
			log.info("Save status( Success ) of ebXML message.");
			persistantStorage.saveMessageStatus(ebxmlMessage, MessageStatusConstraints.Success, "");
			
			Tracker.tracking(ebxmlMessage.getMessageId(), new Date(), "޽ ó  մϴ.", "");
		}
		catch(EbmsException e) {
			if( e instanceof EbmsEbxmlValidationException )
				throw e;
			
			EbmsMessageProcesserException exception = new EbmsMessageProcesserException(1, 0, e);
			
			// ޽ μ ó  
			log.fatal(" [ Error ] - ebXML message process fail.");
			log.fatal("  " + LogUtil.stackTraceToString(e));
			
			// Tracking  .
			Tracker.tracking(messageId, new Date(), "޽ ó  ߽ϴ", e.getMessage());
			
			// ޽ ¸ Fail մϴ.
			log.fatal(" Save status( Fail ) of ebXML message.");
			
			persistantStorage.saveMessageStatus(ebxmlMessage, MessageStatusConstraints.Fail, "");
			
			// Processer error handler մϴ.
			Tracker.tracking(messageId, new Date(), " ޽ մϴ", "");
			
			try {
				log.fatal("  Creating error ebXML message.");
				
	    		ErrorMessage errMsg = MessageBuilderUtil.buildMessageErrorMessage(ebxmlMessage);
	
	    		errMsg.setError("urn:oasis:names:tc:ebxml-msg:service:errors", 
	    				ErrorCode.Unknown, 
	    				ErrorCode.Error, 
	    				"Can't process message.",
	    				e.getMessage());
	    		
	    		log.fatal("  Send error ebXML message.");
	    		
	    		this.sendUndefinedErrorMessage(e, errMsg);
			}
			catch(EbmsTransportException ex) {
				log.fatal("   [ Error ] - Can't send error ebXML message.");
				log.fatal("    " + LogUtil.stackTraceToString(ex));
				
				Tracker.tracking(messageId, new Date(), " ޽  ߽ϴ.", "");
				
				exception.initCause(ex);
			}
			catch(EbmsEbxmlMessageException ex) {
				log.fatal("   [ Error ] - Can't create error ebXML message.");
				log.fatal("    " + LogUtil.stackTraceToString(ex));
				
				Tracker.tracking(messageId, new Date(), " ޽   ϴ.", "");
				
				exception.initCause(ex);
			}
			
			throw exception;
		}
	}
	
	/**
	 * Version     - v1.0
	 * Create time - May 11, 2007
	 * Update time - May 11, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 * @return
	 * 
	 * Ping ޽ °
	 * 
	 * - Pong ޽ ؼ Ѵ.
	 */
	protected EbxmlMessage systemLevelPingMessageProcess(EbxmlMessage ebxmlMessage) throws EbmsApplicationProcesserException {
		try {
			log.info("Pong ޽ մϴ.");
			
			EbxmlMessage pongMessage = MessageBuilderUtil.buildPongMessage(ebxmlMessage);
			
			return pongMessage;
		}
		catch(EbmsEbxmlMessageException e) {
			throw new EbmsApplicationProcesserException(1, 0, e);
		}
	}
	
	
	/**
	 * Version     - v1.0
	 * Create time - May 11, 2007
	 * Update time - May 11, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 * @return
	 * 
	 * Pong ޽  
	 */
	protected EbxmlMessage systemLevelPongMessageProcess(EbxmlMessage ebxmlMessage) throws EbmsApplicationProcesserException {
		// ƹ ó  ʴ´.
		
		return null;
	}

	
	/**
	 * Version     - v1.0
	 * Create time - May 11, 2007
	 * Update time - May 11, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 * @return
	 * 
	 * MessageStatusRequest ޽  
	 * 
	 * - MessageStatusResponse ޽ Ѵ.
	 */
	protected EbxmlMessage systemLevelMessageStatusRequestMessageProcess(EbxmlMessage ebxmlMessage) throws EbmsApplicationProcesserException {
		//TODO MessageStatusResponse ؾ.
		
		return null;
	}
	
	
	/**
	 * Version     - v1.0
	 * Create time - May 11, 2007
	 * Update time - May 11, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 * @return
	 * 
	 * MessageStatusResponse ޽  
	 */
	protected EbxmlMessage systemLevelMessageStatusResponseMessageProcess(EbxmlMessage ebxmlMessage) throws EbmsApplicationProcesserException {
		// ƹ ͵  ʴ´.
		
		return null;
	}
	
	
	/**
	 * Version     - v1.0
	 * Create time - May 11, 2007
	 * Update time - May 11, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 * @return
	 */
	protected EbxmlMessage systemLevelAcknowledgmentMessageProcess(EbxmlMessage ebxmlMessage) throws EbmsApplicationProcesserException {
		try {
			String refToMessageId = ebxmlMessage.getEbxmlHeader().getAcknowledgment().getRefToMessageId().getValue();
			
			AckReceiveNotifier.getInstance().receiveAck(refToMessageId);
		}
		catch(SOAPException e) {
			throw new EbmsApplicationProcesserException(1, 0, e);
		}
		
		return null;
	}
	
	
	/**
	 * Version     - v1.0
	 * Create time - May 11, 2007
	 * Update time - May 11, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 * @return
	 */
	protected EbxmlMessage systemLevelMessageErrorMessageProcess(EbxmlMessage ebxmlMessage) throws EbmsApplicationProcesserException {
		// ƹ ͵  ʴ´.
		
		return null;
	}
	
	
	
	/**
	 * Version     - v1.0
	 * Create time - May 11, 2007
	 * Update time - May 11, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 * @return
	 */
	protected abstract EbxmlMessage applicationLevelMessageProcess(EbxmlMessage ebxmlMessage) throws EbmsApplicationProcesserException;

	/**
	 * Version     - v1.0
	 * Create time - May 10, 2007
	 * Update time - May 10, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param e
	 */
	protected abstract void sendUndefinedErrorMessage(Exception e, EbxmlMessage errorEbxmlMessage) throws EbmsTransportException;
	
	
	/**
	 * Version     - v1.0
	 * Create time - May 11, 2007
	 * Update time - May 11, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 */
	protected abstract void sendResponseEbxmlMessage(EbxmlMessage ebxmlMessage) throws EbmsTransportException;
	
	/**
	 * Version     - v1.0
	 * Create time - May 10, 2007
	 * Update time - May 10, 2007
	 *
	 * Author      - Sirius
	 * 
	 *
	 * @param e
	 */
	protected abstract void sendValidationErrorMessage(EbmsEbxmlValidationException e, EbxmlMessage errorEbxmlMessage) throws EbmsTransportException;
	
	
	/**
	 * Version     - v1.0
	 * Create time - May 10, 2007
	 * Update time - May 10, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 * @throws EbmsMessageProcesserException
	 */
	protected abstract boolean Normal(EbxmlMessage ebxmlMessage) throws EbmsMessageProcesserException;
	
	
	/**
	 * Version     - v1.0
	 * Create time - May 10, 2007
	 * Update time - May 10, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 * @throws EbmsMessageProcesserException
	 */
	protected abstract boolean AckRequested(EbxmlMessage ebxmlMessage) throws EbmsMessageProcesserException;
	
	
	
	
	
	
	
	
	
	/**
	 * Version     - v1.0
	 * Create time - May 10, 2007
	 * Update time - May 10, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 * @throws EbmsMessageProcesserException
	 */
	protected abstract boolean DuplicateElimination(EbxmlMessage ebxmlMessage) throws EbmsMessageProcesserException;
	
	
	/**
	 * Version     - v1.0
	 * Create time - May 10, 2007
	 * Update time - May 10, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 * @throws EbmsMessageProcesserException
	 */
	protected abstract boolean DuplicateElimination_AckRequested(EbxmlMessage ebxmlMessage) throws EbmsMessageProcesserException;
	
}