package org.kldp.ebmsh.message.processer.http;

import java.io.IOException;

import javax.servlet.http.HttpServletResponse;

import org.kldp.ebmsh.application.EbmsApplicationProcesserException;
import org.kldp.ebmsh.application.NonBlockingApplicationProcesser;
import org.kldp.ebmsh.message.packaging.EbxmlMessage;
import org.kldp.ebmsh.message.packaging.validation.EbmsEbxmlValidationException;
import org.kldp.ebmsh.message.persistant.MessageStatusConstraints;
import org.kldp.ebmsh.message.processer.EbmsMessageProcesserException;
import org.kldp.ebmsh.message.processer.MessageProcesser;
import org.kldp.ebmsh.message.transport.EbxmlMessageSender;
import org.kldp.ebmsh.message.transport.EbmsTransportException;
import org.kldp.ebmsh.message.util.MessageBuilderUtil;
import org.kldp.ebmsh.message.util.MessageUtil;



/**
 * File name   - ASyncMessageProcesser.java
 * Create date - May 10, 2007
 * Update time - May 10, 2007
 * Version     - v1.0
 *
 * Author      - Sirius
 */
public class ASyncMessageProcesser extends MessageProcesser {
	private HttpServletResponse response;
	
	/**
	 * Constructor
	 */
	public ASyncMessageProcesser() {
	}
	
	/**
	 * Version     - v1.0
	 * Create time - May 10, 2007
	 * Update time - May 10, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 * @param response
	 */
	public void process(EbxmlMessage ebxmlMessage, HttpServletResponse response) {
		this.response = response;
		
		ebXMLMessageProcess(ebxmlMessage);
	}
	
	
	/**
	 * Version     - v1.0
	 * Create time - May 10, 2007
	 * Update time - May 10, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 * 
	 * AckRequested ִ ޽ óմϴ.
	 * 
	 * - Acknowledgment ޽ մϴ.
	 */
	protected boolean AckRequested(EbxmlMessage ebxmlMessage) throws EbmsMessageProcesserException {
		this.flushBuffer();
		
		// Acknoweldgment 
		log.info("Acknowledgment ebXML ޽ մϴ.");
		EbxmlMessage ackMessage = MessageBuilderUtil.buildAcknowledgmentMessage(ebxmlMessage);
		
		// Acknowledgment ޽ 
		log.info("Acknowledgment ebXML ޽ մϴ.");
		EbxmlMessageSender sender = new EbxmlMessageSender();				
		sender.sendMessageWithoutAction(ackMessage);
		
		return true;
	}
	
	
	/**
	 * Version     - v1.0
	 * Create time - May 10, 2007
	 * Update time - May 10, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 * 
	 * DuplicateElimination ִ ޽ óմϴ.
	 * 
	 * - ߺ  ִ  ޽ ó  մϴ.
	 */
	protected boolean DuplicateElimination(EbxmlMessage ebxmlMessage) throws EbmsMessageProcesserException {
		log.info("ߺ ޽ ˻մϴ.");
		
		if( persistantStorage.isDuplicated(ebxmlMessage) ) {
			log.info("ߺ ޽ Դϴ. ޽ ó  մϴ.");
			
			this.flushBuffer();
			
			// ޽ ¸ Success 
			persistantStorage.saveMessageStatus(ebxmlMessage, MessageStatusConstraints.Success, "");
			
			return false;
		}
		else {
			log.info("ߺ ޽ ƴմϴ.");
			
			// ߺ ޽  
			log.info("ߺ ޽  մϴ.");
			persistantStorage.saveDuplicateMessage(ebxmlMessage);
			
			this.flushBuffer();
			
			return true;
		}
	}

	
	/**
	 * Version     - v1.0
	 * Create time - May 10, 2007
	 * Update time - May 10, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 * 
	 * DuplicateElimination, AckRequested ִ ޽ óմϴ.
	 * 
	 * - ߺ  ִ  ޽ ó  մϴ.
	 * - Acknowledgment ޽ մϴ.
	 */
	protected boolean DuplicateElimination_AckRequested(EbxmlMessage ebxmlMessage) throws EbmsMessageProcesserException {
		log.info("ߺ ޽ ˻մϴ.");
		
		if( persistantStorage.isDuplicated(ebxmlMessage) ) {
			log.info("ߺ ޽ Դϴ.");
			
			this.flushBuffer();
			
			// Acknowledgment ޽ մϴ.
			log.info("PersistantStoreκ Acknowledgment ebXML ޽ մϴ.");
			EbxmlMessage ackMessage = persistantStorage.getAckMessage(ebxmlMessage.getMessageId());
			
			// Acknowledgment ޽ 
			log.info("Acknowledgment ebXML ޽ մϴ.");
			EbxmlMessageSender sender = new EbxmlMessageSender();				
			sender.sendMessageWithoutAction(ackMessage);
			
			// ޽ ¸ Success 
			persistantStorage.saveMessageStatus(ebxmlMessage, MessageStatusConstraints.Success, "");
			
			return false;
		}
		else {
			log.info("ߺ ޽ ƴմϴ.");
			
			// ߺ ޽  
			log.info("ߺ ޽  մϴ.");
			persistantStorage.saveDuplicateMessage(ebxmlMessage);
			
			// Acknoweldgment 
			log.info("Acknowledgment ebXML ޽ մϴ.");
			EbxmlMessage ackMessage = MessageBuilderUtil.buildAcknowledgmentMessage(ebxmlMessage);
			
			// Acknowledgment ޽  
			log.info("Acknowledgment ebXML ޽  մϴ.");
			
			persistantStorage.saveAckMessageToFile(ackMessage);
			
			persistantStorage.saveAckMessageInfo(ackMessage);
			
			this.flushBuffer();
			
			// Acknowledgment ޽ 
			log.info("Acknowledgment ebXML ޽ մϴ.");
			EbxmlMessageSender sender = new EbxmlMessageSender();				
			sender.sendMessageWithoutAction(ackMessage);
			
			return true;
		}
	}
	

	/**
	 * Version     - v1.0
	 * Create time - May 10, 2007
	 * Update time - May 10, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 */
	protected boolean Normal(EbxmlMessage ebxmlMessage) throws EbmsMessageProcesserException {
		this.flushBuffer();
		
		return true;
	}

	
	/**
	 * Version     - v1.0
	 * Create time - May 10, 2007
	 * Update time - May 10, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param e
	 * 
	 * Http Async ebXML ޽ ó  ˼   ߻   ޽ ϴ .
	 */
	public void sendUndefinedErrorMessage(Exception e, EbxmlMessage errorEbxmlMessage) throws EbmsTransportException {
		// HttpStatus 200 մϴ.
		flushBuffer();
		
		//  ޽ մϴ.
		EbxmlMessageSender sender = new EbxmlMessageSender();
		
		sender.sendMessageWithoutAction(errorEbxmlMessage);
	}

	/**
	 * Version     - v1.0
	 * Create time - May 10, 2007
	 * Update time - May 10, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param e
	 * 
	 * Http Async ebXML ޽ ó    ߻   ޽ ϴ .
	 * 
	 * - CPA    ׳ Ѵ. ( CPA ߸   ޽  Url   ٰ Ǵ )
	 * -      ޽ ٸ äη Ѵ.
	 */
	protected void sendValidationErrorMessage(EbmsEbxmlValidationException e, EbxmlMessage errorEbxmlMessage) throws EbmsTransportException {
		// HttpStatus 200 մϴ.
		flushBuffer();
		
		// CPA   ׳  մϴ.
		if( e.getLocation().equals("CPA Validation") ) { }			
		//      ޽ ٸ äη  ݴϴ.
		else {
			//  ޽ մϴ.
			EbxmlMessageSender sender = new EbxmlMessageSender();
			
			sender.sendMessageWithoutAction(errorEbxmlMessage);
		}
	}
	
	
	
	
	/**
	 * Version     - v1.0
	 * Create time - May 10, 2007
	 * Update time - May 10, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param response
	 * @throws EbmsTransportException
	 */
	protected void sendResponseEbxmlMessage(EbxmlMessage responseEbxmlMessage) throws EbmsTransportException {
		EbxmlMessageSender sender = new EbxmlMessageSender();
		
		if( MessageUtil.isSystemLevelActionMessage(responseEbxmlMessage) )
			sender.sendMessageWithoutAction(responseEbxmlMessage);
		else
			sender.sendMessage(responseEbxmlMessage);
	}

	
	/**
	 * Version     - v1.0
	 * Create time - May 10, 2007
	 * Update time - May 10, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param response
	 * @throws EbmsTransportException
	 */
	protected EbxmlMessage applicationLevelMessageProcess(EbxmlMessage ebxmlMessage) throws EbmsApplicationProcesserException {
		log.info("Non blocking application processer մϴ.");
		
		new NonBlockingApplicationProcesser().process(ebxmlMessage);
		
		return null;
	}

	/**
	 * Version     - v1.0
	 * Create time - May 10, 2007
	 * Update time - May 10, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param response
	 * @throws EbmsTransportException
	 */
	private void flushBuffer() throws EbmsTransportException {
		try {
			if( response != null && !response.isCommitted() ) {
				response.setStatus(HttpServletResponse.SC_OK);
				response.setContentLength(0);
				
				response.flushBuffer();
			}
		}
		catch(IOException e) {
			throw new EbmsTransportException(1, 0, "FlushBuffer ", e);
		}
	}
}