package org.kldp.ebmsh.message.security;


import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;

import javax.xml.soap.AttachmentPart;
import javax.xml.soap.SOAPException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xml.security.signature.XMLSignatureInput;
import org.apache.xml.security.utils.resolver.ResourceResolverException;
import org.apache.xml.security.utils.resolver.ResourceResolverSpi;
import org.kldp.ebmsh.message.packaging.EbxmlMessage;
import org.kldp.util.log.LogUtil;
import org.w3c.dom.Attr;



public class EbmsResourceResolver extends ResourceResolverSpi {
	private static Log log = LogFactory.getLog(EbmsResourceResolver.class);
	private EbxmlMessage Msg;
	private Iterator attachments;
	
	public 	EbmsResourceResolver(EbxmlMessage msg) {
		Msg = msg;
	}
	
	public boolean engineCanResolve(Attr attr,String baseURI){
		attachments = Msg.getAttachments();
		String href = attr.getNodeValue();
		if ((attachments != null) && (attachments.hasNext()) && (href.trim().startsWith("cid")))
		{
			return true;
		}
		
		return false;
	}
	
	public XMLSignatureInput engineResolve(org.w3c.dom.Attr attr, String uri) throws ResourceResolverException {
		XMLSignatureInput resource = null;
		String href = attr.getNodeValue();
		log.info("A HREF = " + href + "; baseURI = " +uri);
		uri = href;
		
		if(href.trim().equals("#SyncReply")) return null;
		
		attachments = Msg.getAttachments();
		
		try {
			while(attachments.hasNext()){
				AttachmentPart payload = (AttachmentPart)attachments.next();
				log.info(payload.getContentId());
				
				InputStream input = (payload.getDataHandler().getInputStream());
				ByteArrayOutputStream out = new ByteArrayOutputStream();
				byte[] buffer = new byte[4096];
				for (int c=input.read(buffer) ; c!=-1 ; c=input.read(buffer))
					out.write(buffer, 0, c);
				
				resource = new XMLSignatureInput(out.toByteArray());
				
				resource.setSourceURI(payload.getContentId());
				resource.setMIMEType(payload.getContentType());
			}
		}
		catch(IOException e) {
			log.fatal(LogUtil.stackTraceToString(e));
		}
		catch(SOAPException e) {
			log.fatal(LogUtil.stackTraceToString(e));
		}
		
		return resource;
	}
}