package org.kldp.ebmsh.message.security;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.security.InvalidAlgorithmParameterException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertPath;
import java.security.cert.CertPathValidator;
import java.security.cert.CertPathValidatorException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.PKIXParameters;
import java.security.cert.TrustAnchor;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.soap.Name;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xml.security.exceptions.XMLSecurityException;
import org.apache.xml.security.keys.KeyInfo;
import org.apache.xml.security.signature.XMLSignature;
import org.apache.xml.security.signature.XMLSignatureException;
import org.apache.xml.security.transforms.Transforms;
import org.apache.xml.security.transforms.params.XPathContainer;
import org.apache.xml.security.utils.resolver.ResourceResolver;
import org.kldp.ebmsh.config.EbmsConstants;
import org.kldp.ebmsh.cpa.EbmsCpaException;
import org.kldp.ebmsh.cpa.Certificate;
import org.kldp.ebmsh.cpa.Cpa;
import org.kldp.ebmsh.cpa.CpaManagerFactory;
import org.kldp.ebmsh.cpa.DeliveryChannel;
import org.kldp.ebmsh.cpa.NonRepudiation;
import org.kldp.ebmsh.cpa.PartyInfo;
import org.kldp.ebmsh.cpa.ReceiverNonRepudiation;
import org.kldp.ebmsh.cpa.SenderNonRepudiation;
import org.kldp.ebmsh.message.packaging.EbmsEbxmlMessageException;
import org.kldp.ebmsh.message.packaging.EbxmlMessage;
import org.kldp.ebmsh.message.packaging.impl.ReferenceImpl;
import org.kldp.ebmsh.message.util.MessageUtil;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.softmeca.security.keystore.IKeyStoreHandler;
import com.softmeca.security.keystore.JKSKeyStoreHandler;


public class EbmsSignature {
	private final static Log log = LogFactory.getLog(EbmsSignature.class);
	private IKeyStoreHandler keyStoreHandler = null;

	public EbmsSignature() throws EbmsKeyStoreException {
		org.apache.xml.security.Init.init();

		if( EbmsConstants.keyStoreType.equalsIgnoreCase("jks") ) {
			try {
				keyStoreHandler = new JKSKeyStoreHandler(EbmsConstants.keyStorePath, EbmsConstants.keyStorePassword, EbmsConstants.csrPath);
			} 
			catch (NoSuchAlgorithmException e) {
				EbmsKeyStoreException exception = new EbmsKeyStoreException(1, 0, "Key store ʱȭ ", e);

				throw exception;
			} 
			catch (CertificateException e) {
				EbmsKeyStoreException exception = new EbmsKeyStoreException(1, 0, "Key store ʱȭ ", e);

				throw exception;
			} 
			catch (KeyStoreException e) {
				EbmsKeyStoreException exception = new EbmsKeyStoreException(1, 0, "Key store ʱȭ ", e);

				throw exception;
			} 
			catch (IOException e) {
				EbmsKeyStoreException exception = new EbmsKeyStoreException(1, 0, "Key store ʱȭ ", e);

				throw exception;
			}
		}
		else {
			//TODO 켱 PKCS11 ߰ Ѿ߰.
			throw new EbmsKeyStoreException(1, 0, "  ʴ Key store typeԴϴ.");
		}
	}

	/**
	 * @param ebxmlMessage 
	 * @return throws EbxmlSecurityException, EbxmlMessageException
	 */
	public EbxmlMessage signEbxmlMessage(EbxmlMessage ebxmlMessage) throws EbmsSignException {
		EbxmlMessage signedMessage = null;

		try {
			// Get certificate
			Cpa cpaManager = new CpaManagerFactory().createInstance().getCpa(ebxmlMessage.getCPAId());

			PartyInfo partyInfo = cpaManager.getPartyInfo(ebxmlMessage.getFrom());

			SenderNonRepudiation senderNonRepudiation = (SenderNonRepudiation)partyInfo.getNonRepudiationUnderDocExchange(ebxmlMessage.getAction(), "sender");
			String certId = senderNonRepudiation.getCertId();
			Certificate certificate = partyInfo.getCertificate(certId);

			java.security. cert.X509Certificate cert = null;
			String keyName = certificate.getKeyNames()[0];

			if( keyName == null ) {
				log.fatal("CPA KeyName õǾ  ʽϴ.");

				throw new EbmsCpaException(1, 0, "CPA KeyName õǾ  ʽϴ.");
			}

			log.info("CPA õ KeyName -> " + keyName);

			if( certificate.getX509Certificate() == null ) {
				// CPA    -> KeyStore ã´.
				log.info("CPA   ϴ.");

				try {
					cert = (X509Certificate)keyStoreHandler.getCertificate(keyName);
				} 
				catch (KeyStoreException e) {
					throw new EbmsKeyStoreException(1, 0, "  ",e);
				}
			}
			else {
				// CPA  ִ  -> CPA ٷ ´.
				log.info("CPA   ֽϴ.");
				cert = certificate.getX509Certificate()[0];
			}		

			if( cert == null ) {
				log.fatal(" ã  ϴ.");

				EbmsKeyStoreException exception = new EbmsKeyStoreException(1, 0, " ã  ϴ.");

				throw exception;
			}

			// Get privatekey
			PrivateKey privateKey = null;

			try {
				privateKey = (PrivateKey)keyStoreHandler.getPrivateKey(keyName, EbmsConstants.keyPassword);
			}
			catch (UnrecoverableKeyException e) {
				EbmsKeyStoreException exception = new EbmsKeyStoreException(1, 0, "Private key  ", e);

				throw exception;
			}
			catch (NoSuchAlgorithmException e) {
				EbmsKeyStoreException exception = new EbmsKeyStoreException(1, 0, "Private key  ", e);

				throw exception;
			}
			catch (KeyStoreException e) {
				EbmsKeyStoreException exception = new EbmsKeyStoreException(1, 0, e);

				throw exception;
			}

			signedMessage = this.signEbxmlMessage(ebxmlMessage, cert, privateKey);
		}
		catch(EbmsKeyStoreException e) {
			throw new EbmsSignException(0, 0, e);
		}
		catch(EbmsSignException e) {
			throw e;
		}
		catch(EbmsCpaException e) {
			throw new EbmsSignException(0, 0, e);
		}

		return signedMessage;
	}



	private EbxmlMessage signEbxmlMessage(EbxmlMessage ebxmlMessage, X509Certificate certificate, PrivateKey privateKey) throws EbmsSignException {
		EbxmlMessage signedMessage = null;

		SecurityUtils securityUtil = new SecurityUtils();

		try {
			byte[] soapPart = securityUtil.SOAPPartTOBytes(ebxmlMessage);
			ByteArrayInputStream bis = new ByteArrayInputStream(soapPart);
			InputSource is = new InputSource(bis);

			javax.xml.parsers.DocumentBuilderFactory dbf = javax.xml.parsers.DocumentBuilderFactory.newInstance();
			dbf.setNamespaceAware(true);
			dbf.setAttribute("http://xml.org/sax/features/namespaces", Boolean.TRUE);
			javax.xml.parsers.DocumentBuilder db = dbf.newDocumentBuilder();
			db.setErrorHandler(new org.apache.xml.security.utils.IgnoreAllErrorHandler());
			org.w3c.dom.Document doc = db.parse(is);


			Element headerElement = null;
			NodeList nodes = doc.getElementsByTagNameNS
			("http://schemas.xmlsoap.org/soap/envelope/","Header");

			if(nodes.getLength() != 0) {
				log.info("Header  -> " + nodes.getLength());
				headerElement = (Element)nodes.item(0);
			}
			else {
				log.fatal("Header ã ϴ.");

				EbmsEbxmlMessageException exception = new EbmsEbxmlMessageException(1, 0, "Header ã  ϴ.");

				throw exception;
			}


			//get sinature algorithm from cpa.
			Cpa cpaBuilder = new CpaManagerFactory().createInstance().getCpa(ebxmlMessage.getCPAId());            	
			PartyInfo partyInfo = cpaBuilder.getPartyInfo(ebxmlMessage.getFrom());

			NonRepudiation np = partyInfo.getNonRepudiationUnderDocExchange(ebxmlMessage.getAction(),"sender");
			String signAlgorithm[] = np.getSignatureAlgorithms();

			XMLSignature signature = new XMLSignature(doc, "", signAlgorithm[0]);

			log.info("Ebxml message sign start");
			//  Sign
			headerElement.appendChild(signature.getElement());

			// Add Transforms
			Transforms transforms = new Transforms(doc);
			transforms.addTransform(Transforms.TRANSFORM_ENVELOPED_SIGNATURE);

			String s_xpath = "not (ancestor-or-self::node()[@soap-env:actor=\"urn:oasis:names:tc:ebxml-msg:actor:nextMSH\"] | ancestor-or-self::node()[@soap-env:actor=\"http://schemas.xmlsoap.org/soap/actor/next\"])";
			XPathContainer xc = new XPathContainer(doc);
			xc.setXPath(s_xpath);
			transforms.addTransform(Transforms.TRANSFORM_XPATH, xc.getElement());

			transforms.addTransform(Transforms.TRANSFORM_C14N_OMIT_COMMENTS);

			//get Hashing algorithm from cpa 
			String hashFunction = np.getHashFunction();

			signature.addDocument("", transforms, hashFunction, "", "http://www.w3.org/2000/09/xmldsig#Object");
			//			
			log.info("Ebxml message sign end");


			log.info("Ebxml payload sign start");
			/// ÷ι sign
			if(ebxmlMessage.getEbxmlBody().hasManifest()){
				ResourceResolver resolver = new ResourceResolver(new EbmsResourceResolver(ebxmlMessage));
				signature.addResourceResolver(resolver);
				ArrayList it = ebxmlMessage.getEbxmlBody().getManifest().getReferences();

				for(int i=0; i < it.size(); i++){
					ReferenceImpl ref = (ReferenceImpl)it.get(i);

					signature.addDocument(ref.getXlinkRef());
				}
			}
			log.info("Ebxml payload sign end");

			log.info("Key info add...");
			signature.addKeyInfo(certificate);

			log.info("Signature sign...");
			signature.sign(privateKey);

			signedMessage = securityUtil.updateSignedMessage(ebxmlMessage, doc);
		}
		catch(ParserConfigurationException e) {
			throw new EbmsSignException(1, 4001, "Ebxml message Sign -> ebxmlMessage Ľ   newDocumentBuilder()", e);
		}
		catch(IOException e) {
			throw new EbmsSignException(1, 4002, "Ebxml message Sign -> ebxmlMessage Ľ   newDocumentBuilder()", e);
		}
		catch(SAXException e) {
			throw new EbmsSignException(1, 4003, "Ebxml message Sign -> ebxmlMessage Ľ   parse()", e);
		}
		catch(XMLSecurityException e) {
			throw new EbmsSignException(1, 4004, "Ebxml message Sign -> security sign", e);
		}
		catch(SOAPException e) {
			throw new EbmsSignException(1, 4005, "Ebxml message exception", e);
		}
		catch(EbmsEbxmlMessageException e) {
			throw new EbmsSignException(0, 0, e);
		}
		catch(EbmsCpaException e) {
			throw new EbmsSignException(0, 0, e);
		}

		return signedMessage;
	}

	public EbxmlMessage signAckMessage(EbxmlMessage ackMessage, EbxmlMessage receiveMessage) throws EbmsSignException {
		try {
			EbxmlMessage signedAckMessage = null;

			ackMessage = this.addReferenceToAckMessage(ackMessage, receiveMessage);		
			signedAckMessage = this.signAckMessage(ackMessage);

			return signedAckMessage;
		}
		catch(EbmsSignException e) {
			throw e;
		}
		catch(EbmsEbxmlMessageException e) {
			throw new EbmsSignException(0, 0, e);
		}
	}

	private EbxmlMessage signAckMessage(EbxmlMessage ebxmlMessage) throws EbmsSignException {
		try {
			EbxmlMessage signedMessage = null;

			// Get certificate
			Cpa cpaManager = new CpaManagerFactory().createInstance().getCpa(ebxmlMessage.getCPAId());

			// TODO ̷  ù° ͼ ϸ ȵȴ. ( DocExchange - ebXMLSenderBinding - SenderNonRepudiation - SigningCertificateRef ؼ Ѵ. )
			PartyInfo partyInfo = cpaManager.getPartyInfo(ebxmlMessage.getFrom());

			String defaultMshChannelId = partyInfo.getDefaultMshChannelId();

			DeliveryChannel deliveryChannel = partyInfo.getDeliveryChannel(defaultMshChannelId);

			String docExchangeId = deliveryChannel.getDocExchangeId();

			//
			SenderNonRepudiation senderNonRepudiation = (SenderNonRepudiation)partyInfo.getNonRepudiationUnderDocExchangeByDocExchangeId(docExchangeId, "sender");
			String certId = senderNonRepudiation.getCertId();
			Certificate certificate = partyInfo.getCertificate(certId);

			java.security. cert.X509Certificate cert = null;
			String keyName = certificate.getKeyNames()[0];

			if( keyName == null ) {
				log.fatal("CPA KeyName õǾ  ʽϴ.");

				throw new EbmsCpaException(1, 0, "CPA KeyName õǾ  ʽϴ.");
			}

			log.info("CPA õ KeyName -> " + keyName);

			if( certificate.getX509Certificate() == null ) {
				// CPA    -> KeyStore ã´.
				log.info("CPA   ϴ.");

				log.info("KeyStore  ãϴ. Alias -> " + keyName);

				try { 
					cert = (X509Certificate)keyStoreHandler.getCertificate(keyName);
				}
				catch (KeyStoreException e) {
					throw new EbmsKeyStoreException(1, 0, "  ", e);
				}
			}
			else {
				// CPA  ִ  -> CPA ٷ ´.
				log.info("CPA   ֽϴ.");
				cert = certificate.getX509Certificate()[0];
			}		

			if( cert == null ) {
				log.fatal(" ã ϴ.");

				throw new EbmsSignException(1, 4012, " ã ϴ.");
			}

			// Get privatekey
			PrivateKey privateKey = null;
			try {
				privateKey = (PrivateKey)keyStoreHandler.getPrivateKey(keyName, EbmsConstants.keyPassword);
			} 
			catch (UnrecoverableKeyException e) {
				EbmsKeyStoreException exception = new EbmsKeyStoreException(1, 0, "Private key  ", e);

				throw exception;
			}
			catch (NoSuchAlgorithmException e) {
				EbmsKeyStoreException exception = new EbmsKeyStoreException(1, 0, "Private key  ", e);

				throw exception;
			}
			catch (KeyStoreException e) {
				EbmsKeyStoreException exception = new EbmsKeyStoreException(1, 0, e);

				throw exception;
			}

			signedMessage = this.signAckMessage(ebxmlMessage, cert, privateKey);

			return signedMessage;

			/* ڵ
			 SenderNonRepudiation_ senderNonRepudiation = (SenderNonRepudiation_)partyInfo.getNonRepudiationUnderDocExchangeByDocExchangeId(docExchangeId, "sender");
			 String certId = senderNonRepudiation.getCertId();
			 Certificate_ certificate = partyInfo.getCertificate(certId);
			 java.security.cert.X509Certificate cert = certificate.getX509Certificate()[0];

			 signedMessage = this.signAckMessage(ebxmlMessage, cert, privateKey);*/
		}
		catch(EbmsCpaException e) {
			throw new EbmsSignException(0, 0, e);
		}
		catch(EbmsKeyStoreException e) {
			throw new EbmsSignException(0, 0, e);
		}
	}

	private EbxmlMessage signAckMessage(EbxmlMessage ebxmlMessage, X509Certificate certificate, PrivateKey privateKey) throws EbmsSignException {
		try {
			EbxmlMessage signedMessage = null;

			SecurityUtils securityUtil = new SecurityUtils();

			byte[] soapPart = securityUtil.SOAPPartTOBytes(ebxmlMessage);
			ByteArrayInputStream bis = new ByteArrayInputStream(soapPart);
			InputSource is = new InputSource(bis);

			javax.xml.parsers.DocumentBuilderFactory dbf = javax.xml.parsers.DocumentBuilderFactory.newInstance();
			dbf.setNamespaceAware(true);
			dbf.setAttribute("http://xml.org/sax/features/namespaces", Boolean.TRUE);
			javax.xml.parsers.DocumentBuilder db = dbf.newDocumentBuilder();
			db.setErrorHandler(new org.apache.xml.security.utils.IgnoreAllErrorHandler());
			org.w3c.dom.Document doc = db.parse(is);

			Element headerElement = null;
			NodeList nodes = doc.getElementsByTagNameNS
			("http://schemas.xmlsoap.org/soap/envelope/","Header");

			/*NodeList nodes = doc.getElementsByTagNameNS
			 ("http://schemas.xmlsoap.org/soap/envelope/","Envelope");*/

			if(nodes.getLength() != 0) {
				log.info("Header  -> " + nodes.getLength());
				headerElement = (Element)nodes.item(0);
			}
			else {
				log.fatal("Header ã ϴ.");

				throw new EbmsSignException(1, 0, "Header ã  ϴ.");
			}

			//get sinature algorithm from cpa.
			Cpa cpaBuilder = new CpaManagerFactory().createInstance().getCpa(ebxmlMessage.getCPAId());            	
			PartyInfo partyInfo = cpaBuilder.getPartyInfo(ebxmlMessage.getFrom());

			String defaultMshChannelId = partyInfo.getDefaultMshChannelId();			
			DeliveryChannel deliveryChannel = partyInfo.getDeliveryChannel(defaultMshChannelId);			
			String docExchangeId = deliveryChannel.getDocExchangeId();

			NonRepudiation np = partyInfo.getNonRepudiationUnderDocExchangeByDocExchangeId(docExchangeId, "sender");
			String signAlgorithm[] = np.getSignatureAlgorithms();

			XMLSignature signature = new XMLSignature(doc, "", signAlgorithm[0]);

			log.info("Ebxml message sign start");
			///  Sign
			headerElement.appendChild(signature.getElement());

			Transforms transforms = new Transforms(doc);
			transforms.addTransform(Transforms.TRANSFORM_ENVELOPED_SIGNATURE);

			String s_xpath = "not (ancestor-or-self::node()[@soap-env:actor=\"urn:oasis:names:tc:ebxml-msg:actor:nextMSH\"] | ancestor-or-self::node()[@soap-env:actor=\"http://schemas.xmlsoap.org/soap/actor/next\"])";
			XPathContainer xc = new XPathContainer(doc);
			xc.setXPath(s_xpath);
			transforms.addTransform(Transforms.TRANSFORM_XPATH, xc.getElement());			
			transforms.addTransform(Transforms.TRANSFORM_C14N_WITH_COMMENTS);


			//get Hashing algorithm from cpa 
			String hashFunction = np.getHashFunction();

			signature.addDocument("", transforms, hashFunction);
			///			
			log.info("Ebxml message sign end");


			log.info("Ebxml payload sign start");
			/// ÷ι sign
			if(ebxmlMessage.getEbxmlBody().hasManifest()){
				ResourceResolver resolver = new ResourceResolver(new EbmsResourceResolver(ebxmlMessage));
				signature.addResourceResolver(resolver);
				ArrayList it = ebxmlMessage.getEbxmlBody().getManifest().getReferences();

				for(int i=0; i < it.size(); i++){
					ReferenceImpl ref = (ReferenceImpl)it.get(i);

					signature.addDocument(ref.getXlinkRef());
				}
			}
			log.info("Ebxml payload sign end");

			log.info("Key info add...");
			signature.addKeyInfo(certificate);

			log.info("Signature sign...");
			signature.sign(privateKey);

			signedMessage = securityUtil.updateSignedMessage(ebxmlMessage, doc);

			return signedMessage;
		}
		catch(ParserConfigurationException e) {
			throw new EbmsSignException(1, 4006, "Ebxml message Sign -> ebxmlMessage Ľ   newDocumentBuilder()", e);
		}
		catch(IOException e) {
			throw new EbmsSignException(1, 4007, "Ebxml message Sign -> ebxmlMessage Ľ   parse()", e);
		}
		catch(SAXException e) {
			throw new EbmsSignException(1, 4008, "Ebxml message Sign -> ebxmlMessage Ľ   parse()", e);
		}
		catch(XMLSecurityException e) {
			throw new EbmsSignException(1, 4009, "Ebxml message Sign -> security sign", e);
		}
		catch(SOAPException e) {
			throw new EbmsSignException(1, 4010, "Ebxml message exception", e);
		}
		catch(EbmsSignException e) {
			throw e;
		}
		catch(EbmsCpaException e) {
			throw new EbmsSignException(0, 0, e);
		}
	}

	private Iterator generateReference(EbxmlMessage ebxmlMessage) throws EbmsSignException {
		try {
			SecurityUtils securityUtil = new SecurityUtils();

			byte[] soapPart = securityUtil.SOAPPartTOBytes(ebxmlMessage);
			ByteArrayInputStream bis = new ByteArrayInputStream(soapPart);
			InputSource is = new InputSource(bis);

			javax.xml.parsers.DocumentBuilderFactory dbf = javax.xml.parsers.DocumentBuilderFactory.newInstance();
			dbf.setNamespaceAware(true);
			dbf.setAttribute("http://xml.org/sax/features/namespaces", Boolean.TRUE);
			javax.xml.parsers.DocumentBuilder db = dbf.newDocumentBuilder();
			db.setErrorHandler(new org.apache.xml.security.utils.IgnoreAllErrorHandler());
			org.w3c.dom.Document doc = db.parse(is);


			Element headerElement = null;
			NodeList nodes = doc.getElementsByTagNameNS
			("http://schemas.xmlsoap.org/soap/envelope/","Header");

			if(nodes.getLength() != 0) {
				log.info("Header  -> " + nodes.getLength());
				headerElement = (Element)nodes.item(0);
			}
			else {
				log.fatal("Header ã ϴ.");

				return null;
			}

			//get sinature algorithm from cpa.
			Cpa cpaBuilder = new CpaManagerFactory().createInstance().getCpa(ebxmlMessage.getCPAId());            	
			PartyInfo partyInfo = cpaBuilder.getPartyInfo(ebxmlMessage.getFrom());

			NonRepudiation np = partyInfo.getNonRepudiationUnderDocExchange(ebxmlMessage.getAction(),"sender");
			String signAlgorithm[] = np.getSignatureAlgorithms();

			XMLSignature signature = new XMLSignature(doc, "", signAlgorithm[0]);

			log.info("Ebxml message sign start");
			///  Sign
			headerElement.appendChild(signature.getElement());

			Transforms transforms = new Transforms(doc);
			transforms.addTransform(Transforms.TRANSFORM_ENVELOPED_SIGNATURE);

			String s_xpath = "not (ancestor-or-self::node()[@soap-env:actor=\"urn:oasis:names:tc:ebxml-msg:actor:nextMSH\"] | ancestor-or-self::node()[@soap-env:actor=\"http://schemas.xmlsoap.org/soap/actor/next\"])";
			XPathContainer xc = new XPathContainer(doc);
			xc.setXPath(s_xpath);
			transforms.addTransform(Transforms.TRANSFORM_XPATH, xc.getElement());			
			transforms.addTransform(Transforms.TRANSFORM_C14N_WITH_COMMENTS);


			//get Hashing algorithm from cpa 
			String hashFunction = np.getHashFunction();

			signature.addDocument("", transforms, hashFunction);
			log.info("Ebxml message sign end");
			
			


			log.info("Ebxml payload sign start");
			/// ÷ι sign
			if(ebxmlMessage.getEbxmlBody().hasManifest()){
				ResourceResolver resolver = new ResourceResolver(new EbmsResourceResolver(ebxmlMessage));
				signature.addResourceResolver(resolver);
				ArrayList it = ebxmlMessage.getEbxmlBody().getManifest().getReferences();

				for(int i=0; i < it.size(); i++){
					ReferenceImpl ref = (ReferenceImpl)it.get(i);

					signature.addDocument(ref.getXlinkRef());
				}
			}
			log.info("Ebxml payload sign end");

			// generate digest value
			signature.getSignedInfo().generateDigestValues();

			// get Reference element
			String st = new String(signature.getSignedInfo().getCanonicalizedOctetStream());		

			ByteArrayInputStream bis_ = new ByteArrayInputStream(st.getBytes());
			InputSource is_ = new InputSource(bis_);

			javax.xml.parsers.DocumentBuilderFactory documentFactory = javax.xml.parsers.DocumentBuilderFactory.newInstance();
			documentFactory.setNamespaceAware(true);
			documentFactory.setAttribute("http://xml.org/sax/features/namespaces", Boolean.TRUE);
			javax.xml.parsers.DocumentBuilder db_ = dbf.newDocumentBuilder();
			db_.setErrorHandler(new org.apache.xml.security.utils.IgnoreAllErrorHandler());
			org.w3c.dom.Document doc_ = db_.parse(is_);

			org.w3c.dom.Node signedInfoNode = doc_.getFirstChild();
			
			SOAPElement signedInfoElement = MessageUtil.convertDOMToSOAPElement(ebxmlMessage.getSOAPPart().getEnvelope(), signedInfoNode);

			Name referenceName = ebxmlMessage.getSOAPPart().getEnvelope().createName("Reference", "ds", "http://www.w3.org/2000/09/xmldsig#");
			
			
			Iterator iter = signedInfoElement.getChildElements(referenceName);

			if(!iter.hasNext())
				throw new EbmsSignException(1, 0, "Reference element ϴ.");

			return iter;
		}
		catch(ParserConfigurationException e) {
			throw new EbmsSignException(1, 4001, "Ebxml message Sign -> ebxmlMessage Ľ   newDocumentBuilder()", e);
		}
		catch(IOException e) {
			throw new EbmsSignException(1, 4002, "Ebxml message Sign -> ebxmlMessage Ľ   parse()", e);
		}
		catch(SAXException e) {
			throw new EbmsSignException(1, 4003, "Ebxml message Sign -> ebxmlMessage Ľ   parse()", e);
		}
		catch(XMLSecurityException e) {
			throw new EbmsSignException(1, 4004, "Ebxml message Sign -> security sign", e);
		}
		catch(SOAPException e) {
			throw new EbmsSignException(1, 4005, "Ebxml message exception", e);
		}
		catch(EbmsCpaException e) {
			throw new EbmsSignException(0, 0, e);
		}
		catch(EbmsSignException e) {
			throw e;
		}
	}





	private EbxmlMessage addReferenceToAckMessage(EbxmlMessage ackMessage, EbxmlMessage receiveMessage) throws EbmsEbxmlMessageException {
		try {
			EbxmlMessage _ackMessage = ackMessage;
			boolean hasSignature = false;

			Name sinatureName = receiveMessage.getSOAPPart().getEnvelope().createName("Signature", "ds", "http://www.w3.org/2000/09/xmldsig#");
			Iterator iter = receiveMessage.getEbxmlHeader().getChildElements(sinatureName);

			if(iter.hasNext()) {
				log.info("Signature մϴ.");

				hasSignature = true;
			}
			else 
				log.info("Signature ã ϴ.");

			if( hasSignature == true ) {
				iter = getReference(receiveMessage);

				while( iter.hasNext() ) {
					SOAPElement referenceElement = (SOAPElement)iter.next();
					_ackMessage.getEbxmlHeader().getAcknowledgment().addChildElement(referenceElement);
				}
			}
			else {
				System.setProperty("javax.xml.soap.Name", "com.sun.xml.messaging.saaj.soap.dom4j.NameImpl");
				
				iter = generateReference(receiveMessage);

				while( iter.hasNext() ) {
					SOAPElement referenceElement = (SOAPElement)iter.next();
					
					log.info("~~~~~~~~~~~!");
					
					

					_ackMessage.getEbxmlHeader().getAcknowledgment().addChildElement(referenceElement);
				}
			}

			if(_ackMessage != null)
				_ackMessage.saveChanges();

			return _ackMessage;
		}
		catch(SOAPException e) {
			throw new EbmsEbxmlMessageException(1, 4011, "Ebxml message Sign -> ebxmlMessage Ľ   newDocumentBuilder()", e);
		}
		catch(EbmsEbxmlMessageException e) {
			throw e;
		}
	}

	/*private Iterator getReference(EbxmlMessage ebxmlMessage) throws EbxmlMessageException {
	 Iterator iter_ = null;

	 try {
	 Name sinatureName = ebxmlMessage.getSOAPPart().getEnvelope().createName("Signature", "ds", "http://www.w3.org/2000/09/xmldsig#");
	 Iterator iter = ebxmlMessage.getEbxmlHeader().getChildElements(sinatureName);
	 SOAPElement signatureElement = null;

	 signatureElement = (SOAPElement)iter.next();

	 Name signedInfoName = ebxmlMessage.getSOAPPart().getEnvelope().createName("SignedInfo", "ds", "http://www.w3.org/2000/09/xmldsig#");				
	 iter = signatureElement.getChildElements(signedInfoName);				
	 SOAPElement signedInfoElement = (SOAPElement)iter.next();

	 Name referenceName = ebxmlMessage.getSOAPPart().getEnvelope().createName("Reference", "ds", "http://www.w3.org/2000/09/xmldsig#");
	 iter = signedInfoElement.getChildElements(referenceName);

	 if(!iter.hasNext())
	 log.info("Reference element ϴ.");

	 iter_ = iter;
	 }
	 catch(SOAPException e) {
	 throw new EbxmlMessageException(1, 4022, "Ebxml message Sign -> ebxmlMessage Ľ   newDocumentBuilder()", e);
	 }

	 return iter_;
	 }*/


	private Iterator getReference(EbxmlMessage ebxmlMessage) throws EbmsEbxmlMessageException {
		Iterator iter_ = null;

		try {
			Name sinatureName = ebxmlMessage.getSOAPPart().getEnvelope().createName("Signature", "ds", "http://www.w3.org/2000/09/xmldsig#");
			Iterator iter = ebxmlMessage.getEbxmlHeader().getChildElements(sinatureName);
			SOAPElement signatureElement = null;

			signatureElement = (SOAPElement)iter.next();

			Name signedInfoName = ebxmlMessage.getSOAPPart().getEnvelope().createName("SignedInfo", "ds", "http://www.w3.org/2000/09/xmldsig#");				
			iter = signatureElement.getChildElements(signedInfoName);				
			SOAPElement signedInfoElement = (SOAPElement)iter.next();

			Name referenceName = ebxmlMessage.getSOAPPart().getEnvelope().createName("Reference", "ds", "http://www.w3.org/2000/09/xmldsig#");
			iter = signedInfoElement.getChildElements(referenceName);

			if(!iter.hasNext())
				log.info("Reference element ϴ.");

			iter_ = iter;
		}
		catch(SOAPException e) {
			throw new EbmsEbxmlMessageException(1, 4022, "Ebxml message Sign -> ebxmlMessage Ľ   newDocumentBuilder()", e);
		}

		return iter_;
	}








	/**
	 *  2ܰ
	 * 1. ޽   ؼ Root  Ѵ.
	 * 2. ޽ ü  Ѵ.
	 * */
	public boolean validateEbxmlMessage(EbxmlMessage ebxmlMessage) throws EbmsValidateException {
		try {
			boolean result = false;		
			SecurityUtils securityUtil = new SecurityUtils();

			byte[] soapPart = securityUtil.SOAPPartTOBytes(ebxmlMessage);
			ByteArrayInputStream bis = new ByteArrayInputStream(soapPart);
			InputSource is = new InputSource(bis);

			javax.xml.parsers.DocumentBuilderFactory dbf = javax.xml.parsers.DocumentBuilderFactory.newInstance();
			dbf.setNamespaceAware(true);			
			dbf.setAttribute("http://xml.org/sax/features/namespaces", Boolean.TRUE);
			javax.xml.parsers.DocumentBuilder db = dbf.newDocumentBuilder();
			db.setErrorHandler(new org.apache.xml.security.utils.IgnoreAllErrorHandler());
			org.w3c.dom.Document doc = db.parse(is);

			Element signatureElement = null;
			NodeList nodes = doc.getElementsByTagNameNS(org.apache.xml.security.utils.Constants.SignatureSpecNS,"Signature");

			if(nodes.getLength() != 0) {
				log.info("Signature  -> " + nodes.getLength());
				signatureElement = (Element)nodes.item(0);
			}
			else {
				log.fatal("Signature ã ϴ.");

				throw new EbmsValidateException(1, 0, "Signature ã ϴ.");
			}

			XMLSignature signature = new XMLSignature(signatureElement, "");
			signature.getSignedInfo().addResourceResolver(new EbmsResourceResolver(ebxmlMessage));
			KeyInfo keyInfo = signature.getKeyInfo();

			log.info(" ãϴ.");
			java.security.cert.X509Certificate cert = null;

			log.info("1 - ޽  ãϴ.");
			if(keyInfo != null && keyInfo.containsX509Data()) {
				log.info("޽   մϴ.");

				cert = keyInfo.getX509Certificate();
			}
			else {
				log.info("2 - CPA  ãϴ.");
				// Get certificate
				Cpa cpaManager = new CpaManagerFactory().createInstance().getCpa(ebxmlMessage.getCPAId());

				PartyInfo partyInfo = cpaManager.getPartyInfo(ebxmlMessage.getFrom());

				SenderNonRepudiation senderNonRepudiation = (SenderNonRepudiation)partyInfo.getNonRepudiationUnderDocExchange(ebxmlMessage.getAction(), "sender");
				String certId = senderNonRepudiation.getCertId();
				Certificate certificate = partyInfo.getCertificate(certId);

				String keyName = certificate.getKeyNames()[0];

				if( keyName == null ) {
					log.fatal("CPA KeyName õǾ  ʽϴ.");

					throw new EbmsValidateException(1, 0, "CPA KeyName õǾ  ʽϴ.");
				}

				log.info("CPA õ KeyName -> " + keyName);

				if( certificate.getX509Certificate() != null ) {
					// CPA  ִ  -> CPA ٷ ´.
					cert = certificate.getX509Certificate()[0];	    			
				}
				else {
					// CPA    -> KeyStore ã´.
					log.info("3 - KeyStore  ãϴ.");
					try {
						cert = (X509Certificate)keyStoreHandler.getCertificate(keyName);
					} 
					catch (KeyStoreException e) {
						throw new EbmsKeyStoreException(1, 0, " ã  ", e);
					}
				}
			}

			if( cert == null ) {
				log.fatal(" ã ϴ.");

				throw new EbmsValidateException(1, 4016, " ã  ϴ.");
			}

			//TODO 1ܰ. ޽   . // For korbit
			/*log.info("3 Ű ޽   մϴ.");
			try {
				validateCertificateInMessage(ebxmlMessage, cert);

				log.info("  ");
			}
			catch(EbxmlValidateException e) {
				log.fatal("  ");

				throw e;
			}*/


			//TODO 2ܰ. ޽ .
			log.info("޽ մϴ.");          
			result = validateMessage(signature, cert); // For korbit
			//result = true;
			if( !result ) {
				log.fatal("޽  ");

				throw new EbmsValidateException(1, 4016, "޽  ");
			}
			else {
				log.info("޽  ");
			}

			return result;
		}
		catch(ParserConfigurationException e) {
			throw new EbmsValidateException(1, 4013, "Ebxml message Sign -> ebxmlMessage Ľ   newDocumentBuilder()", e);
		}
		catch(IOException e) {
			throw new EbmsValidateException(1, 4014, "Ebxml message Sign -> ebxmlMessage Ľ   parse()", e);
		}
		catch(SAXException e) {
			throw new EbmsValidateException(1, 4015, "Ebxml message Sign -> ebxmlMessage Ľ   parse()", e);
		}
		catch(SOAPException e) {
			throw new EbmsValidateException(1, 4015, "Ebxml message Sign -> ebxmlMessage Ľ   parse()", e);
		}
		catch(XMLSecurityException e) {
			throw new EbmsValidateException(1, 4016, "Ebxml message Sign -> security sign", e);
		}
		catch(EbmsCpaException e) {
			throw new EbmsValidateException(1, 0, e);
		}
		catch(EbmsKeyStoreException e) {
			throw new EbmsValidateException(1, 0, e);
		}
	}

	/**
	 * @param ebxmlMessage EbxmlMessage Certificate 
	 * @param msgCertificate RootCeritifcate  Certificate	 
	 * @return  .
	 * 
	 * ޽ȿ ִ  Root  Ѵ.
	 * */
	private void validateCertificateInMessage(EbxmlMessage ebxmlMessage, X509Certificate msgCertificate) throws EbmsValidateException {
		try {
			Cpa cpaBuilder = new CpaManagerFactory().createInstance().getCpa(ebxmlMessage.getCPAId());            	
			PartyInfo partyInfo = cpaBuilder.getPartyInfo(ebxmlMessage.getTo());
			ReceiverNonRepudiation nonRepudiation = (ReceiverNonRepudiation)partyInfo.getNonRepudiationUnderDocExchange(ebxmlMessage.getAction(), "receiver");
			String securityId = nonRepudiation.getSecurityId();

			String[] trustAnchorIds = partyInfo.getSecurityDetailsTrustAnchorsCertIds(securityId);
			java.security.cert.X509Certificate[] trustAnchors = new java.security.cert.X509Certificate[trustAnchorIds.length];

			log.info("Trust Anchor  -> " + trustAnchorIds.length);

			HashSet hashSet = new HashSet();

			for(int i = 0; i < trustAnchors.length; i++) { 
				Certificate certificate = partyInfo.getCertificate(trustAnchorIds[i]);

				if( certificate.getX509Certificate() == null ) {
					// CPA TrustAnchor Key  KeyStore ã´.	    			
					log.info("CPA TrustAnchor   ϴ.");
					log.info("KeyStore ãϴ.");

					String keyName = partyInfo.getCertificate(trustAnchorIds[i]).getKeyNames()[0];

					try {
						trustAnchors[i] = (X509Certificate)keyStoreHandler.getCertificate(keyName);
					} catch (KeyStoreException e) {
						throw new EbmsKeyStoreException(1, 0, e);
					}
				}
				else {
					log.info("CPA TrustAnchor   ֽϴ.");

					trustAnchors[i] = (java.security.cert.X509Certificate)(partyInfo.getCertificate(trustAnchorIds[i]).getX509Certificate())[0];
				}

				if( trustAnchors[i] == null ) 
					throw new EbmsValidateException(1, 0, "TrustAnchor  ã ϴ.");

				TrustAnchor trustAnchor = new TrustAnchor(trustAnchors[i], null);

				System.out.println(trustAnchors[i].getIssuerDN());

				hashSet.add(trustAnchor);
			}

			PKIXParameters params = new PKIXParameters(hashSet);
			params.setRevocationEnabled(false);

			X509Certificate[] cert = new X509Certificate[] { msgCertificate  };
			CertificateFactory certFact = CertificateFactory.getInstance("X.509");
			CertPath path = certFact.generateCertPath(Arrays.asList(cert));	    	

			CertPathValidator certPathValidator = CertPathValidator.getInstance(CertPathValidator.getDefaultType());
			certPathValidator.validate(path, params);	    	

			//  ҽ
			/*X509Certificate rootCertificate = partyInfo.getRootCertificate();

			 msgCertificate.verify(rootCertificate.getPublicKey());

			 result = true;*/
		}
		catch(InvalidAlgorithmParameterException e) {
			throw new EbmsValidateException(1, 4017, "Invalid Algorithm Parameter exception -> PKIXParameters(hashSet)", e);
		}
		catch(NoSuchAlgorithmException e) {
			throw new EbmsValidateException(1, 4018, " Path  -> CertPathValidator.getInstance()", e);
		}
		catch(CertificateException e) {
			throw new EbmsValidateException(1, 4019, " Path  -> CertificateFactory.getInstance(X.509);", e);
		}
		catch(CertPathValidatorException e) {
			throw new EbmsValidateException(1, 4020, "Messageȿ ִ    -> validate()", e);
		}
		catch(EbmsCpaException e) {
			throw new EbmsValidateException(0, 0, e);
		}
		catch(EbmsKeyStoreException e) {
			throw new EbmsValidateException(0, 0, e);
		}
	}

	/**
	 * @param signature XMLSignature
	 * @param certificate ޽  Certificate. ( Message   ) 
	 * @return  .
	 * 
	 * ޽ ü Ѵ.
	 * */
	private boolean validateMessage(XMLSignature signature, X509Certificate certificate) throws EbmsValidateException {
		try {
			System.out.println(signature);
			System.out.println(certificate);
			
			return signature.checkSignatureValue(certificate);
		}
		catch(XMLSignatureException e) {
			throw new EbmsValidateException(1, 4021, "Message   -> checkSignatureValue()", e);
		}
	}
}