package org.kldp.ebmsh.message.transport;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;

import javax.mail.MessagingException;
import javax.mail.internet.ContentType;
import javax.mail.internet.MimeBodyPart;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.xml.soap.MimeHeader;
import javax.xml.soap.MimeHeaders;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;

import org.apache.commons.httpclient.Header;
import org.apache.commons.httpclient.HostConfiguration;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.MultiThreadedHttpConnectionManager;
import org.apache.commons.httpclient.methods.ByteArrayRequestEntity;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.protocol.DefaultProtocolSocketFactory;
import org.apache.commons.httpclient.protocol.Protocol;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.kldp.ebmsh.message.packaging.EBXMLMimeMultipart;
import org.kldp.ebmsh.message.packaging.EbmsEbxmlMessageException;
import org.kldp.ebmsh.message.packaging.EbxmlMessage;
import org.kldp.ebmsh.message.packaging.EbxmlMessageFactory;
import org.kldp.ebmsh.message.packaging.impl.EbxmlMessageFactoryImpl;
import org.kldp.ebmsh.message.packaging.impl.EbxmlMessageImpl;
import org.kldp.ebmsh.message.util.ByteArrayDataSource;
import org.kldp.ebmsh.message.util.MessageUtil;
import org.kldp.ebmsh.message.util.MultipartDataSource;



/**
 * HttpTransport
 * 
 * Http transport layer  ebxml message ϴ Ŭ
 * 
 * @author Sirius
 */
class HttpTransport {
	private static Log log = LogFactory.getLog(HttpTransport.class);
	
	private final int HTTP_DEFAULT_PORT = 80;
	private final int HTTPS_DEFAULT_PORT = 443;
	
	private static HttpClient httpClient =  null;
	
	/**
	 * 
	 * connManager Ѵ.
	 * HttpClient Ѵ.
	 */
	public HttpTransport() {
		synchronized (HttpTransport.class) {
			if(httpClient == null) {
				MultiThreadedHttpConnectionManager connectionManager = new MultiThreadedHttpConnectionManager();
				
				// ϳ Host ִ Ŀؼ  : 10
				connectionManager.getParams().setDefaultMaxConnectionsPerHost(10);
				// ü ִ Ŀؼ  : 50
				connectionManager.getParams().setMaxTotalConnections(50);
				
				httpClient = new HttpClient(connectionManager);
			}
		}
	}
	
	
	/**
	 * HostConfiguration  մϴ.
	 * 
	 * @param url
	 * @return
	 * @throws EbmsTransportException
	 */
	private HostConfiguration createHostConfiguration(String urlProtocol, String urlHost, int urlPort) throws EbmsTransportException {
		Protocol protocol = null;
		
		if( urlProtocol.toLowerCase().equals("http") ) {
			if( urlPort == -1  )
				urlPort = HTTP_DEFAULT_PORT;
			
			protocol = new Protocol(urlProtocol, new DefaultProtocolSocketFactory(), urlPort);
		}
		else if( urlProtocol.toLowerCase().equals("https") ) {
			if( urlPort == -1  )
				urlPort = HTTPS_DEFAULT_PORT;
			
			protocol = new Protocol(urlProtocol, new EbmsSSLProtocolSocketFactory(), urlPort);
		}
		else {
			throw new EbmsTransportException(1, 0, "    Դϴ. Protocol -> " + urlProtocol);
		}
		
		// Host configuration 
		HostConfiguration configuration = new HostConfiguration();
    	configuration.setHost(urlHost, urlPort, protocol);
    	
		return configuration;
	}
	
	
	/**
	 * @param ebxmlMessage  EbxmlMessage
	 * @param url  EndPoint
	 * @return  ebxmlMessage
	 * 
	 * Message  Function
	 * @throws IOException 
	 */
	public EbxmlMessage sendMessage(EbxmlMessage ebxmlMessage, String url) throws EbmsTransportException {
		log.info(" URL : " + url);
		
		PostMethod postMethod = null;
		
		// Url parsing 
		URL urlParser = null;
		
		try {
			urlParser = new URL(url);
		}
		catch(MalformedURLException e) {
			throw new EbmsTransportException(1, 0, "url parsing   ϴ. url -> " + url, e);
		}
		
		String urlProtocol = urlParser.getProtocol();
		String urlHost = urlParser.getHost();
		int urlPort = urlParser.getPort();		
		String urlPath = urlParser.getPath();
		// Url parsing end
		
		try {
			HostConfiguration hostConfiguration = createHostConfiguration(urlProtocol, urlHost, urlPort);
			
			postMethod = new PostMethod(urlPath);
			
			if (ebxmlMessage.saveRequired())
				ebxmlMessage.saveChanges();
			
			try {
				this.setMimeHeader(ebxmlMessage);
			}
			catch(EbmsEbxmlMessageException e) {
				throw new EbmsTransportException(1, 0, "Set MimeHeader error", e);
			}
			
			MimeHeaders mimeHeaders = ebxmlMessage.getMimeHeaders();				
			Iterator headerIter = mimeHeaders.getAllHeaders();
			
			byte[] ebxmlMessageByteArray = ebxmlMessage.getMsgAsBytes();

			ByteArrayRequestEntity byteArrayRequestEntity = new ByteArrayRequestEntity(ebxmlMessageByteArray);
			postMethod.setRequestEntity(byteArrayRequestEntity);
			
			log.info("----------  ޽   ----------");
			while(headerIter.hasNext()) {
				MimeHeader header = (MimeHeader)headerIter.next();
				
				String headerName = header.getName();
				String headerValue = reMakeHeaderValue(header.getValue());
				
				if( headerName.equals("Content-Length") ) 
					headerValue = String.valueOf(byteArrayRequestEntity.getContentLength());
				
				if( headerName.equals("Content-Type") ) 
					headerValue = headerValue + "; start=\"MessageContentId\"";
				
				postMethod.setRequestHeader(headerName, headerValue);

				log.info(headerName + ": " + headerValue);
			}
			log.info("----------  ޽   ----------");
			
			log.info("----------  ޽ -------------------");
			ByteArrayRequestEntity sre = (ByteArrayRequestEntity)postMethod.getRequestEntity();
			log.info(new String(sre.getContent()));
			log.info("----------  ޽ -------------------");
			
			int responseCode = httpClient.executeMethod(hostConfiguration, postMethod);
			
			log.info("----------  ޽   ----------");			
			MimeHeaders responseMimeheaders = getMimeHeaders(postMethod.getResponseHeaders());							
			Iterator responseHeaderIter = responseMimeheaders.getAllHeaders();
			
			while(responseHeaderIter.hasNext()) {
				MimeHeader header = (MimeHeader)responseHeaderIter.next();
				
				String headerName = header.getName();
				String headerValue = header.getValue();
								
				log.info(headerName + ": " + headerValue);
			}
			log.info("----------  ޽   ----------");
			
			byte[] responseByteArray = inputStreamConvertToByteArray(postMethod.getResponseBodyAsStream());
			
			log.info(new String(responseByteArray));
			if( responseCode >= 300 && responseCode <= 500)
				throw new IOException("  ̻մϴ -> " + responseCode + "\n  -> " + postMethod.getResponseBodyAsString());
			
			if( responseByteArray.length <= 0 )
				return null;
			else {
				EbxmlMessageImpl responseEbxmlMessage = null;
				
				Header[] header = postMethod.getResponseHeaders();
				MimeHeaders headers = getMimeHeaders(header);
				
				EbxmlMessageFactory ebxmlMessageFactory = new EbxmlMessageFactoryImpl();
				ByteArrayInputStream bais = null;
				
				try {
					bais = new ByteArrayInputStream(responseByteArray);
					
					log.info("SOAPMessage մϴ. ");
					SOAPMessage soapMessage = ebxmlMessageFactory.createMessage(headers, bais);
					
					log.info("SOAPMessageκ EbxmlMessage մϴ.");			
					responseEbxmlMessage = new EbxmlMessageImpl(soapMessage, ebxmlMessageFactory);
				}
				finally {
					if( bais != null )
						bais.close();
				}
				
				return responseEbxmlMessage;
			}
		}
		catch(SOAPException e) {
			throw new EbmsTransportException(1, 0, e);
		}
		catch(IOException e) {
			throw new EbmsTransportException(1, 0, "HttpClient error -> sendMessage()" ,e);
		}
		finally {
			if( postMethod != null )
				postMethod.releaseConnection();
		}
	}
	
	
	/**
	 * @param ebxmlMessage  EbxmlMessage
	 * @param url  EndPoint
	 * @return  ebxmlMessage
	 * 
	 * Message  Function
	 * @throws IOException 
	 */
	public EbxmlMessage sendMessageForMultipart(EbxmlMessage ebxmlMessage, String url) throws EbmsTransportException {
		if(MessageUtil.isMimepartEbxmlMessage(ebxmlMessage)) 
			return sendMessage(ebxmlMessage, url);
		
		HttpClient httpClient = new HttpClient();
		
		log.info(" URL : " + url);
		
		PostMethod postMethod = null;
		
		// Url parsing 
		URL urlParser = null;
		
		try {
			urlParser = new URL(url);
		}
		catch(MalformedURLException e) {
			throw new EbmsTransportException(1, 0, "url parsing   ϴ. url -> " + url, e);
		}
		
		String urlProtocol = urlParser.getProtocol();
		String urlHost = urlParser.getHost();
		int urlPort = urlParser.getPort();		
		String urlPath = urlParser.getPath();
		// Url parsing end
		
		try {
			HostConfiguration hostConfiguration = createHostConfiguration(urlProtocol, urlHost, urlPort);
			
			postMethod = new PostMethod(urlPath);
			
			if (ebxmlMessage.saveRequired())
				ebxmlMessage.saveChanges();
			
			try {
				this.setMimeHeader(ebxmlMessage);
			}
			catch(EbmsEbxmlMessageException e) {
				throw new EbmsTransportException(1, 0, "Set MimeHeader error", e);
			}
			
			MimeHeaders mimeHeaders = ebxmlMessage.getMimeHeaders();				
			Iterator headerIter = mimeHeaders.getAllHeaders();
			
			String transformedEbxmlMessageStr = null;
			
			// new section
			MimeHeaders _mimeHeaders = ebxmlMessage.getMimeHeaders();
			
			String _contentType = _mimeHeaders.getHeader("Content-Type")[0];
			EBXMLMimeMultipart mp = null;

			try {
				if( _contentType.startsWith("text/xml") ) {
					ByteArrayDataSource bas = new ByteArrayDataSource(MessageUtil.getSOAPMessage(ebxmlMessage), "text/xml");
					mp = new EBXMLMimeMultipart(bas);
	
					MimeBodyPart mb = mp.getRootPart();
	
					MultipartDataSource ds = new MultipartDataSource(MessageUtil.getSOAPMessage(ebxmlMessage));
					ds.setContentTypeAndId(mb.getContentType(), mb.getContentID());
	
					ContentType ct = mp.getContentType();
					String bnd = ct.getParameter("boundary");
	
					ds.setBoundary(bnd);
	
					transformedEbxmlMessageStr = ds.getString();
				}
				else if( _contentType.startsWith("multipart/related") )
					transformedEbxmlMessageStr = ebxmlMessage.getMsgAsString();
			}
			catch(MessagingException e) {
				throw new EbmsTransportException(1, 0, e);
			}
			// new section end
			
			ByteArrayRequestEntity byteArrayRequestEntity = new ByteArrayRequestEntity(transformedEbxmlMessageStr.getBytes());
			postMethod.setRequestEntity(byteArrayRequestEntity);
			
			log.info("----------  ޽   ----------");
			while(headerIter.hasNext()) {
				MimeHeader header = (MimeHeader)headerIter.next();
				
				String headerName = header.getName();
				String headerValue = reMakeHeaderValue(header.getValue());
								
				if( headerName.equals("Content-Length") ) {
					headerValue = String.valueOf(transformedEbxmlMessageStr.getBytes().length);
				}
				
				if( headerName.equals("Content-Type") ) {
					String contentType = reMakeHeaderValue( mp.getContentType().toString() );
					
					headerValue = contentType;
				}
				
				postMethod.setRequestHeader(headerName, headerValue);

				log.info(headerName + ": " + headerValue);
			}
			log.info("----------  ޽   ----------");
			
			log.info("----------  ޽ -------------------");
			ByteArrayRequestEntity sre = (ByteArrayRequestEntity)postMethod.getRequestEntity();
			log.info(new String(sre.getContent()));
			log.info("----------  ޽ -------------------");
			
			int responseCode = httpClient.executeMethod(hostConfiguration, postMethod);
			
			
			log.info("----------  ޽   ----------");
			MimeHeaders responseMimeheaders = getMimeHeaders(postMethod.getResponseHeaders());
							
			Iterator responseHeaderIter = responseMimeheaders.getAllHeaders();
			
			while(responseHeaderIter.hasNext()) {
				MimeHeader header = (MimeHeader)responseHeaderIter.next();
				
				String headerName = header.getName();
				String headerValue = header.getValue();
								
				log.info(headerName + ": " + headerValue);
			}
			log.info("----------  ޽   ----------");
			
			byte[] responseByteArray = inputStreamConvertToByteArray(postMethod.getResponseBodyAsStream());
			
			log.info(new String(responseByteArray));
			if( responseCode >= 300 && responseCode <= 500)
				throw new IOException("  ̻մϴ -> " + responseCode + "\n  -> " + postMethod.getResponseBodyAsString());
			
			if( responseByteArray.length <= 0 )
				return null;
			else {
				EbxmlMessageImpl responseEbxmlMessage = null;
				
				Header[] header = postMethod.getResponseHeaders();
				MimeHeaders headers = getMimeHeaders(header);
				
				EbxmlMessageFactory ebxmlMessageFactory = new EbxmlMessageFactoryImpl();
				ByteArrayInputStream bais = null;
				
				try {
					bais = new ByteArrayInputStream(responseByteArray);
					
					log.info("SOAPMessage մϴ. ");
					SOAPMessage soapMessage = ebxmlMessageFactory.createMessage(headers, bais);
					
					
					log.info("SOAPMessageκ EbxmlMessage մϴ.");			
					responseEbxmlMessage = new EbxmlMessageImpl(soapMessage, ebxmlMessageFactory);
				}
				finally {
					if( bais != null )
						bais.close();
				}
				
				return responseEbxmlMessage;
			}
		}
		catch(SOAPException e) {
			throw new EbmsTransportException(1, 0, e);
		}
		catch(IOException e) {
			throw new EbmsTransportException(1, 0, "HttpClient error -> sendMessage()" ,e);
		}
		finally {
			if( postMethod != null )
				postMethod.releaseConnection();
		}
	}
	
	
	/**
	 *   Channel  մϴ.
	 * 
	 * @param ebxmlMessage
	 * @param response
	 * @throws EbmsTransportException
	 */		
	public void redirect(EbxmlMessage ebxmlMessage, HttpServletResponse response) throws EbmsTransportException
	{	
		try {
			if (ebxmlMessage.saveRequired())
				ebxmlMessage.saveChanges();
			
			try {
				this.setMimeHeader(ebxmlMessage);
			}
			catch(EbmsEbxmlMessageException e) {
				throw new EbmsTransportException(1, 0, "MimeHeader   ϴ.", e);
			}
			
			byte[] ebxmlMessageByteArray = ebxmlMessage.getMsgAsBytes();
			
			MimeHeaders mimeHeaders = ebxmlMessage.getMimeHeaders();				
			Iterator headerIter = mimeHeaders.getAllHeaders();
			
			log.info("----------  ޽   ----------");
			while(headerIter.hasNext()) {
				MimeHeader header = (MimeHeader)headerIter.next();
				
				String headerName = header.getName();
				String headerValue = reMakeHeaderValue(header.getValue());
				
				if( headerName.equals("Content-Length") ) {
					int contentLength = ebxmlMessageByteArray.length;
					
					headerValue = String.valueOf(contentLength);
					
					continue;
				}
				else if( headerName.equals("Content-Type") ) {
					String reContentType = reMakeHeaderValue(header.getValue());
					
					String contentType = reContentType + "; start=\"MessageContentId\"";
					
					headerValue = contentType;
					
					response.setContentType(contentType);
				}
				
				log.info(headerName + ": " + headerValue);
			}
			log.info("----------  ޽   ----------");
			
			ServletOutputStream sos = response.getOutputStream();
			
			sos.write(ebxmlMessage.toString().getBytes());
			
			log.info("----------  ޽ -------------------");
			log.info(new String(ebxmlMessageByteArray));
			log.info("----------  ޽ -------------------");
			
			sos.flush();
			sos.close();
			
			response.flushBuffer();
		}
		catch(SOAPException e) {
			throw new EbmsTransportException(1, 0, e);
		}
		catch(IOException e) {
			throw new EbmsTransportException(1, 0, "HttpClient error -> redirect()" ,e);
		}
		finally {
			if( !response.isCommitted() ) {
				try {
					response.setStatus(HttpServletResponse.SC_OK);
					response.setContentLength(0);
					ServletOutputStream sos = response.getOutputStream();
					
					sos.flush();
					sos.close();
					
					response.flushBuffer();
				}
				catch(IOException e) {
					throw new EbmsTransportException(1, 0, "HttpServletResponse flush  ϴ.", e);
				}
				catch(Exception e) {
					throw new EbmsTransportException(1, 0, "HttpServletResponse flush  ϴ.", e);
				}
			}
		}
	}
	
	
	/**
	 *   Channel  մϴ.
	 * 
	 * @param ebxmlMessage
	 * @param response
	 * @throws EbmsTransportException
	 */		
	public void redirectForMultipart(EbxmlMessage ebxmlMessage, HttpServletResponse response) throws EbmsTransportException	
	{	
		if(MessageUtil.isMimepartEbxmlMessage(ebxmlMessage)) {
			redirect(ebxmlMessage, response);
		}
		
		try {
			if (ebxmlMessage.saveRequired())
				ebxmlMessage.saveChanges();
			
			try {
				this.setMimeHeader(ebxmlMessage);
			}
			catch(EbmsEbxmlMessageException e) {
				throw new EbmsTransportException(1, 0, "MimeHeader   ϴ.", e);
			}
			
			String ebxmlMessageStr = null;
			
			// new section
			MimeHeaders _mimeHeaders = ebxmlMessage.getMimeHeaders();
			
			String _contentType = _mimeHeaders.getHeader("Content-Type")[0];
			EBXMLMimeMultipart mp = null;

			try {
				if( _contentType.startsWith("text/xml") ) {
					ByteArrayDataSource bas = new ByteArrayDataSource(MessageUtil.getSOAPMessage(ebxmlMessage), "text/xml");
					mp = new EBXMLMimeMultipart(bas);
	
					MimeBodyPart mb = mp.getRootPart();
	
					MultipartDataSource ds = new MultipartDataSource(MessageUtil.getSOAPMessage(ebxmlMessage));
					ds.setContentTypeAndId(mb.getContentType(), mb.getContentID());
	
					ContentType ct = mp.getContentType();
					String bnd = ct.getParameter("boundary");
	
					ds.setBoundary(bnd);
	
					ebxmlMessageStr = ds.getString();
				}
				else if( _contentType.startsWith("multipart/related") )
					ebxmlMessageStr = ebxmlMessage.getMsgAsString();
			}
			catch(MessagingException e) {
				throw new EbmsTransportException(1, 0, e);
			}
			// new section end

			MimeHeaders mimeHeaders = ebxmlMessage.getMimeHeaders();				
			Iterator headerIter = mimeHeaders.getAllHeaders();
			
			log.info("----------  ޽   ----------");
			while(headerIter.hasNext()) {
				MimeHeader header = (MimeHeader)headerIter.next();
			
				String headerName = header.getName();
				String headerValue = header.getValue();
				
				if( headerName.equals("Content-Length") ) {
					int contentLength = ebxmlMessageStr.getBytes().length;
					
					headerValue = String.valueOf(contentLength);
					
					continue;
				}
				else if( headerName.equals("Content-Type") ) {
					//String contentType = "multipart/related; type=\"text/xml\"; boundary=boundary; start=\"MessageContentId\"";
					String contentType = reMakeHeaderValue( mp.getContentType().toString() );
					
					headerValue = contentType;
					
					response.setContentType(headerValue);
				}
				
				log.info(headerName + ": " + headerValue);
			}
			log.info("----------  ޽   ----------");

			ServletOutputStream sos = response.getOutputStream();
			
			sos.write(ebxmlMessageStr.getBytes());
			
			log.info("----------  ޽ -------------------");
			log.info(ebxmlMessageStr);
			log.info("----------  ޽ -------------------");
			
			sos.flush();
			sos.close();
			
			response.flushBuffer();
		}
		catch(SOAPException e) {
			throw new EbmsTransportException(1, 0, e);
		}
		catch(IOException e) {
			throw new EbmsTransportException(1, 0, "HttpClient error -> redirect()" ,e);
		}
		finally {
			if( !response.isCommitted() ) {
				try {
					response.setStatus(HttpServletResponse.SC_OK);
					response.setContentLength(0);
					ServletOutputStream sos = response.getOutputStream();
					
					sos.flush();
					sos.close();
					
					response.flushBuffer();
				}
				catch(IOException e) {
					throw new EbmsTransportException(1, 0, "HttpServletResponse flush  ϴ.", e);
				}
				catch(Exception e) {
					throw new EbmsTransportException(1, 0, "HttpServletResponse flush  ϴ.", e);
				}
			}
		}
	}
	
	
	
	/**
	 * EbxmlMessage mime header HttpConnection mime header settingմϴ.
	 * 
	 * @param ebxmlMessage
	 * @throws EbmsEbxmlMessageException
	 */
	private void setMimeHeader(EbxmlMessage ebxmlMessage) throws EbmsEbxmlMessageException {
		log.info("MimeHeader Setting մϴ.");
		
		// SOAP MimeHeader κ setting
		log.info("SOAP MimeHeader Settingմϴ.");
		ebxmlMessage.getSOAPPart().removeAllMimeHeaders();
		ebxmlMessage.getSOAPPart().addMimeHeader("Content-Type", "text/xml; charset=UTF-8");
		ebxmlMessage.getSOAPPart().setContentId("MessageContentId");
		
		try {
			ebxmlMessage.saveChanges();
		}
		catch(SOAPException e) {
			throw new EbmsEbxmlMessageException(1, 0, "Ebxml message saveChanges()   ϴ.", e);
		}
	}
	
	
	/**
	 * Http header SOAP mime header · ȯ
	 * 
	 * @param httpServletRequest
	 * @return
	 */
	private MimeHeaders getMimeHeaders(Header[] header) throws EbmsEbxmlMessageException {
		MimeHeaders mimeheaders = new MimeHeaders();
		
		for(int i = 0; i < header.length; i++) 
			mimeheaders.addHeader(header[i].getName(), header[i].getValue());
			
		return mimeheaders;
	}
	
	private String reMakeHeaderValue(String headerValue) {
		String[] headerArray = headerValue.split(";");
		
		StringBuffer sb = new StringBuffer();
		
		for(int i = 0; i < headerArray.length; i++) {
			sb.append(headerArray[i].trim());
			
			if( i != headerArray.length -1 )
				sb.append("; ");
		}
		
		return sb.toString();
	}
	
	private byte[] inputStreamConvertToByteArray(InputStream inputStream) throws EbmsEbxmlMessageException {
		try {
			ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
			
			byte[] buffer = new byte[1024];
			int count = 0;
			
			while( (count = inputStream.read(buffer)) != -1 )
				byteArrayOutputStream.write(buffer, 0, count);
			
			byte[] result = byteArrayOutputStream.toByteArray();
			
			byteArrayOutputStream.flush();		
			byteArrayOutputStream.close();		
			inputStream.close();
			
			return result;
		}
		catch(Exception e) {
			throw new EbmsEbxmlMessageException(1, 0, e);
		}
		finally {
			try {
				if( inputStream != null )
					inputStream.close();
			}
			catch(Exception e) {
			}
		}
	}
}