package org.kldp.ebmsh.message.util;

import java.util.Date;
import java.util.Iterator;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.xml.soap.MimeHeaders;
import javax.xml.soap.Name;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.kldp.ebmsh.cpa.EbmsCpaException;
import org.kldp.ebmsh.cpa.Cpa;
import org.kldp.ebmsh.cpa.CpaManagerFactory;
import org.kldp.ebmsh.cpa.DeliveryChannel;
import org.kldp.ebmsh.cpa.PartyInfo;
import org.kldp.ebmsh.message.packaging.EbmsEbxmlMessageException;
import org.kldp.ebmsh.message.packaging.EbxmlConstants;
import org.kldp.ebmsh.message.packaging.EbxmlMessage;
import org.kldp.ebmsh.message.packaging.TimeToLive;
import org.kldp.ebmsh.message.packaging.impl.AcknowledgmentMessage;
import org.kldp.ebmsh.message.packaging.impl.EbxmlMessageFactoryImpl;
import org.kldp.ebmsh.message.packaging.impl.EbxmlMessageImpl;
import org.kldp.ebmsh.message.packaging.impl.ErrorMessage;
import org.kldp.ebmsh.message.security.EbmsKeyStoreException;
import org.kldp.ebmsh.message.security.EbmsSignException;
import org.kldp.ebmsh.message.security.EbmsSignature;



/**
 * File name   - MessageBuilderUtil.java
 * Create date - May 12, 2007
 * Update time - May 12, 2007
 * Version     - v1.0
 *
 * Author      - Sirius
 * 
 * ebXML ޽ Ҷ  Util Ŭ
 */
public class MessageBuilderUtil {
	private final static Log log = LogFactory.getLog(MessageBuilderUtil.class);
	
	
	/**
	 * Version     - v1.0
	 * Create time - May 12, 2007
	 * Update time - May 12, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param fromPartyId
	 * @param toPartyId
	 * @param cpaId
	 * @param conversationId
	 * @param action
	 * @param service
	 * @return
	 * @throws EbmsEbxmlMessageException
	 * 
	 * ebXML ޽ Template մϴ.
	 */
	private static AcknowledgmentMessage createAckEbxmlMessageTemplate(String fromPartyId, String toPartyId, String cpaId, String conversationId, String action, String service, String serviceType) throws EbmsEbxmlMessageException {
		AcknowledgmentMessage em = null;
		
		try {
			EbxmlMessageFactoryImpl mf = new EbxmlMessageFactoryImpl();
			
			SOAPMessage sm = mf.createMessage();
			em = new AcknowledgmentMessage(sm,mf);
			em.initNameSpace();
			em.initMessageHeader();
			
			em.setCPAId(cpaId);			
			em.setFrom(fromPartyId, "urn:oasis:names:tc:ebxml-cppa:partyid-type:duns");
			em.setTo(toPartyId, "urn:oasis:names:tc:ebxml-cppa:partyid-type:duns");
			
			if( serviceType == null || serviceType == "" )
				em.setService(service);
			else
				em.setService(service, serviceType);
			
			em.setConversationId(conversationId);
			em.setAction(action);
			
			String messageId = MessageUtil.generateMessageId(toPartyId, cpaId, service, action);
			em.setMessageId(messageId);
			
			String timestamp = MessageUtil.dateToUTC(new java.util.Date());   	
			em.setTimestamp(timestamp);
			
			//TimeToLive tl = em.getEbxmlHeader().getMessageHeader().getMessageData().addTimeToLive();			
			//tl.setTimeToLive(MessageUtil.dateToUTC( new java.util.Date(System.currentTimeMillis() + 5000) ) );
			
			em.saveChanges();
		}
		catch(SOAPException e) {
			throw new EbmsEbxmlMessageException(1, 0, "޽  ", e);
		}
		
		return em;
	}
	
	private static EbxmlMessage createEbxmlMessageTemplate(String fromPartyId, String toPartyId, String cpaId, String conversationId, String action, String service, String serviceType) throws EbmsEbxmlMessageException {
		EbxmlMessageImpl em = null;
		
		try {
			EbxmlMessageFactoryImpl mf = new EbxmlMessageFactoryImpl();
			
			SOAPMessage sm = mf.createMessage();
			em = new EbxmlMessageImpl(sm,mf);
			em.initNameSpace();
			em.initMessageHeader();
			
			em.setCPAId(cpaId);			
			em.setFrom(fromPartyId, "urn:oasis:names:tc:ebxml-cppa:partyid-type:duns");
			em.setTo(toPartyId, "urn:oasis:names:tc:ebxml-cppa:partyid-type:duns");
			
			if( serviceType == null || serviceType == "" )
				em.setService(service);
			else
				em.setService(service, serviceType);
			
			em.setConversationId(conversationId);
			em.setAction(action);
			
			String messageId = MessageUtil.generateMessageId(toPartyId, cpaId, service, action);
			em.setMessageId(messageId);
			
			String timestamp = MessageUtil.dateToUTC(new java.util.Date());   	
			em.setTimestamp(timestamp);
			
			TimeToLive tl = em.getEbxmlHeader().getMessageHeader().getMessageData().addTimeToLive();			
			tl.setTimeToLive(MessageUtil.dateToUTC( new java.util.Date(System.currentTimeMillis() + 5000) ) );
			
			em.saveChanges();
		}
		catch(SOAPException e) {
			throw new EbmsEbxmlMessageException(1, 0, "޽  ", e);
		}
		
		return em;
	}
	
	
	/**
	 * Version     - v1.0
	 * Create time - May 12, 2007
	 * Update time - May 12, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param fromPartyId
	 * @param toPartyId
	 * @param cpaId
	 * @param conversationId
	 * @param action
	 * @param service
	 * @return
	 * @throws EbmsEbxmlMessageException
	 * 
	 * ÷ι  ebXML ޽ մϴ.
	 */
	public static EbxmlMessage buildCustomActionMessage(String fromPartyId, String toPartyId, String cpaId, String conversationId, String action, String service) throws EbmsEbxmlMessageException {
		EbxmlMessage em = null;
		
		try {
			em = createEbxmlMessageTemplate(fromPartyId, toPartyId, cpaId, conversationId, action, service, "ebXMLBPMS");

			MimeHeaders mimeHeaders = em.getMimeHeaders();
			
			checkSyncReply(em);
			checkAckRequest(em);
			checkDuplicateElimination(em);
			checkEbxmlMessageSign(em);
		}
		catch(EbmsCpaException e) {
			throw new EbmsEbxmlMessageException(1, 0, "޽  ", e);
		}
		
		return em;
	}
	
	
	/**
	 * Version     - v1.0
	 * Create time - May 12, 2007
	 * Update time - May 12, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param fromPartyId
	 * @param toPartyId
	 * @param cpaId
	 * @param conversationId
	 * @param action
	 * @param service
	 * @param payloads
	 * @return
	 * @throws EbmsEbxmlMessageException
	 * 
	 * ÷   ebXML ޽ մϴ.
	 */
	public static EbxmlMessage buildCustomActionMessageWithPayloads(String fromPartyId, String toPartyId, String cpaId, String conversationId, String action, String service, byte[][] payloads) throws EbmsEbxmlMessageException {
		EbxmlMessage em = null;
		
		try{
			em = createEbxmlMessageTemplate(fromPartyId, toPartyId, cpaId, conversationId, action, service, "ebXMLBPMS");
			
			// ÷  ߰ 
			for(int i =0 ; i < payloads.length ;i++) {
				DataSource ds = new StringDataSource(new String(payloads[i]), "text/xml");				
				DataHandler dh = new DataHandler(ds);
				
				em.addPayload(dh, "contentid-" + String.valueOf(i) + "@softmeca.com" );
			}
			
			em.saveChanges();
			
			checkSyncReply(em);
			checkAckRequest(em);
			checkDuplicateElimination(em);
			checkEbxmlMessageSign(em);
		}
		catch(SOAPException e) {
			throw new EbmsEbxmlMessageException(1, 0, "޽  ", e);
		}
		catch(EbmsCpaException e) {
			throw new EbmsEbxmlMessageException(1, 0, "޽  ", e);
		}
		
		return em;
	}
	
	
	/**
	 * Version     - v1.0
	 * Create time - May 12, 2007
	 * Update time - May 12, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param receiveEbxmlMessage
	 * @return
	 * @throws EbmsEbxmlMessageException
	 * 
	 * MessageError  ebXML ޽ ؼ մϴ.
	 */
	public static ErrorMessage buildMessageErrorMessage(EbxmlMessage receiveEbxmlMessage) throws EbmsEbxmlMessageException {
		ErrorMessage em = null;
		
		try {
			String cpaId = receiveEbxmlMessage.getCPAId();
			String toPartyId = receiveEbxmlMessage.getFrom();
			String fromPartyId = receiveEbxmlMessage.getTo();
			String conversationId = receiveEbxmlMessage.getConversationId();
			String action = "MessageError";
			String service = "urn:oasis:names:tc:ebxml-msg:service";
			
			EbxmlMessage tmpEbxmlMessage = createEbxmlMessageTemplate(fromPartyId, toPartyId, cpaId, conversationId, action, service, "");
			
			tmpEbxmlMessage.getEbxmlHeader().getMessageHeader().getMessageData().addRefToMessageId().setValue(receiveEbxmlMessage.getMessageId());
			
			em = new ErrorMessage(tmpEbxmlMessage, null);
		}
		catch(SOAPException e) {
			throw new EbmsEbxmlMessageException(1, 0, e);
		}
		
		return em;
	}
	
	
	/**
	 * Version     - v1.0
	 * Create time - May 12, 2007
	 * Update time - May 12, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param receiveEbxmlMessage
	 * @return
	 * @throws EbmsEbxmlMessageException
	 * 
	 * Acknowledgment ޽ մϴ.
	 */
	public static AcknowledgmentMessage buildAcknowledgmentMessage(EbxmlMessage receiveEbxmlMessage) throws EbmsEbxmlMessageException
	{
		AcknowledgmentMessage em = null;
		
		try {
			String refToMessageId = receiveEbxmlMessage.getMessageId();
			String action = "Acknowledgment";
			String service = "urn:oasis:names:tc:ebxml-msg:service";
			
			em = createAckEbxmlMessageTemplate(receiveEbxmlMessage.getTo(), receiveEbxmlMessage.getFrom(), receiveEbxmlMessage.getCPAId(), receiveEbxmlMessage.getConversationId(), action, service, "");
			
			Date date = new Date();
			String timestamp = MessageUtil.dateToUTC(date);
			em.setAcknowledgment(refToMessageId, receiveEbxmlMessage.getTo(), timestamp);
			
			String s_Actor = null;

			Iterator it = receiveEbxmlMessage.getEbxmlHeader().getAckRequested().getAllAttributes();
			while(it.hasNext()){
				Name t = (Name)it.next();

				if(t.getLocalName().equals("actor"))
					s_Actor = receiveEbxmlMessage.getEbxmlHeader().getAckRequested().getAttributeValue(t);
			}

			if(s_Actor!= null )
				em.getAcknowledgment().setActor(s_Actor);
			else
				em.getAcknowledgment().setActor(EbxmlConstants.partyMsh);

			checkDuplicateElimination(em);
		}
		catch(SOAPException e) {
			throw new EbmsEbxmlMessageException(1, 0, e);
		}
		
		return em;
	}
	
	
	/**
	 * Version     - v1.0
	 * Create time - May 12, 2007
	 * Update time - May 12, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param fromPartyId
	 * @param toPartyId
	 * @param cpaId
	 * @param refToMessageId
	 * @return
	 * @throws EbmsEbxmlMessageException
	 * 
	 * Pong ޽ մϴ.
	 */
	public static EbxmlMessage buildPongMessage(EbxmlMessage receiveEbxmlMessage) throws EbmsEbxmlMessageException 
	{
		EbxmlMessage em = null;
		
		try {
			String fromPartyId = receiveEbxmlMessage.getTo();
			String toPartyId = receiveEbxmlMessage.getFrom();
			String conversationId = receiveEbxmlMessage.getConversationId();
			String cpaId = receiveEbxmlMessage.getCPAId();
			String refToMessageId = receiveEbxmlMessage.getMessageId();
			String action = "Pong";
			String service = "urn:oasis:names:tc:ebxml-msg:service";
			
			em = createEbxmlMessageTemplate(fromPartyId, toPartyId, cpaId, conversationId, action, service, "");
			
			em.getEbxmlHeader().getMessageHeader().getMessageData().addRefToMessageId().setValue(refToMessageId);
			
			checkSyncReply(em);
			checkAckRequest(em);
			checkDuplicateElimination(em);
		}
		catch(SOAPException e) {
			throw new EbmsEbxmlMessageException(1, 0, e);
		}
		
		return em;
	}
	
	
	/**
	 * Version     - v1.0
	 * Create time - May 16, 2007
	 * Update time - May 16, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 * @throws EbmsEbxmlMessageException
	 * @throws EbmsCpaException
	 */
	private static void checkSyncReply(EbxmlMessage ebxmlMessage) throws EbmsEbxmlMessageException, EbmsCpaException {
		log.info("CPA SyncReply ˻մϴ.");

		try {
			Cpa cpa = new CpaManagerFactory().createInstance().getCpa(ebxmlMessage.getCPAId());

			if( cpa == null )
				throw new EbmsCpaException(1, 0, "CPA ã  ϴ.");

			PartyInfo party = cpa.getPartyInfo(ebxmlMessage.getFrom());

			if( party == null )
				throw new EbmsCpaException(1, 0, "Party ã  ϴ.");
			
			String syncReplyMode = null;
			
			if( MessageUtil.isSystemLevelActionMessage(ebxmlMessage)) {
				String defaultMshChannelId = party.getDefaultMshChannelId();
				
				syncReplyMode = party.getDeliveryChannel(defaultMshChannelId).getSyncReplyMode();
			}
			else {
				syncReplyMode = party.getDeliveryChannelByAction(ebxmlMessage.getAction()).getSyncReplyMode();
			}
			
			if( syncReplyMode == null )
				throw new EbmsCpaException(1, 0, "SyncReplyMode ã  ϴ.");

			boolean isSync = false;

			if( syncReplyMode.equals("mshSignalsOnly") || syncReplyMode.equals("signalsOnly") || syncReplyMode.equals("responseOnly") || syncReplyMode.equals("signalsAndResponse") ) 
				isSync = true;

			log.info("Sync => " + syncReplyMode);

			if( isSync ) {
				log.info("SyncReply ߰մϴ.");

				if( !ebxmlMessage.getEbxmlHeader().hasSyncReply() )
					ebxmlMessage.getEbxmlHeader().addSyncReply();
			}

			ebxmlMessage.saveChanges();
		}
		catch(SOAPException e) {
			throw new EbmsEbxmlMessageException(1, 0, "Sync Reply ˻ ", e);
		}
	}


	/**
	 * Version     - v1.0
	 * Create time - May 16, 2007
	 * Update time - May 16, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 * @throws EbmsEbxmlMessageException
	 * @throws EbmsCpaException
	 */
	private static void checkAckRequest(EbxmlMessage ebxmlMessage) throws EbmsEbxmlMessageException, EbmsCpaException {
		log.info("CPA AckRequested ˻մϴ.");

		try {
			Cpa cpa = new CpaManagerFactory().createInstance().getCpa(ebxmlMessage.getCPAId());

			if( cpa == null )
				throw new EbmsCpaException(1, 0, "CPA ã  ϴ.");

			PartyInfo party = cpa.getPartyInfo(ebxmlMessage.getFrom());

			if( party == null )
				throw new EbmsCpaException(1, 0, "Party ã  ϴ.");

			String ackRequested = null;
			String ackRequestedSign = null;
			
			if( MessageUtil.isSystemLevelActionMessage(ebxmlMessage)) {
				String defaultMshChannelId = party.getDefaultMshChannelId();
				
				ackRequested = party.getDeliveryChannel(defaultMshChannelId).getAckRequested();
				ackRequestedSign = party.getDeliveryChannel(defaultMshChannelId).getAckSignatureRequested();
			}
			else {
				ackRequested = party.getDeliveryChannelByAction(ebxmlMessage.getAction()).getAckRequested();
				ackRequestedSign = party.getDeliveryChannelByAction(ebxmlMessage.getAction()).getAckSignatureRequested();
			}

			boolean isAckRequest = false;

			if( ackRequested.equals("always") ) 
				isAckRequest = true;	   

			log.info("AckRequested => " + ackRequested);

			if( isAckRequest ) {
				log.info("AckRequested ߰մϴ.");

				if( !ebxmlMessage.getEbxmlHeader().hasAckRequested() )
					ebxmlMessage.getEbxmlHeader().addAckRequested();

				ebxmlMessage.saveChanges();
			}


			boolean isAckRequestSign = false;

			if( isAckRequest ) {
				if( ackRequestedSign.equals("always") )
					isAckRequestSign = true;

				log.info("AckRequestedSign => " + ackRequestedSign);

				if( isAckRequestSign ) {
					log.info("AckRequested Sign true մϴ.");

					ebxmlMessage.getEbxmlHeader().getAckRequested().setSigned(true);

					ebxmlMessage.saveChanges();
				}
			}   
		}
		catch(SOAPException e) {
			throw new EbmsEbxmlMessageException(1, 0, "AckRequested ˻ ", e);
		}
	}

	
	/**
	 * Version     - v1.0
	 * Create time - May 16, 2007
	 * Update time - May 16, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 * @throws EbmsEbxmlMessageException
	 * @throws EbmsCpaException
	 */
	private static void checkDuplicateElimination(EbxmlMessage ebxmlMessage) throws EbmsEbxmlMessageException, EbmsCpaException {
		log.info("CPA DuplicateElimination ˻մϴ.");

		try {
			Cpa cpa = new CpaManagerFactory().createInstance().getCpa(ebxmlMessage.getCPAId());

			if( cpa == null )
				throw new EbmsCpaException(1, 0, "CPA ã  ϴ.");

			PartyInfo party = cpa.getPartyInfo(ebxmlMessage.getFrom());

			if( party == null )
				throw new EbmsCpaException(1, 0, "Party ã  ϴ.");

			DeliveryChannel deliveryChannel = null;
			
			if( MessageUtil.isSystemLevelActionMessage(ebxmlMessage) ) {
				String defaultMshChannelId = party.getDefaultMshChannelId();
				
				deliveryChannel = party.getDeliveryChannel(defaultMshChannelId);
			}
			else {
				deliveryChannel = party.getDeliveryChannelByAction(ebxmlMessage.getAction());
			}
			
			String duplicateElimination = deliveryChannel.getDuplicateElimination();
			
			log.info("DuplicateElimination => " + duplicateElimination);

			if(duplicateElimination.equals("always"))
			{
				log.info("DuplicateElimination => always̹Ƿ duplicateEliminaion ߰ մϴ.");

				if(!ebxmlMessage.getEbxmlHeader().getMessageHeader().hasDuplicateElimination())
					ebxmlMessage.getEbxmlHeader().getMessageHeader().addDuplicateElimenation();
			}

			ebxmlMessage.saveChanges();
		}
		catch(SOAPException e) {
			throw new EbmsEbxmlMessageException(1, 0, "DuplicateElimination ˻ ", e);
		}
	}

	
	/**
	 * Version     - v1.0
	 * Create time - May 16, 2007
	 * Update time - May 16, 2007
	 *
	 * Author      - Sirius
	 *
	 * @param ebxmlMessage
	 * @throws EbmsCpaException
	 * @throws EbmsEbxmlMessageException
	 * @throws EbmsKeyStoreException
	 * @throws EbmsSignException
	 */
	private static void checkEbxmlMessageSign(EbxmlMessage ebxmlMessage) throws EbmsEbxmlMessageException {
		try {
			log.info("EbxmlMessage Sign ؾϴ ˻մϴ.");
	
			// EbxmlMessage sign check
			Cpa cpa = new CpaManagerFactory().createInstance().getCpa(ebxmlMessage.getCPAId());
	
			if( cpa == null )
				throw new EbmsCpaException(1, 0, "CPA ã  ϴ.");
	
			PartyInfo party = cpa.getPartyInfo(ebxmlMessage.getFrom());
	
			if( party == null )
				throw new EbmsCpaException(1, 0, "Party ã  ϴ.");
	
			boolean isSign = party.getCanSend(ebxmlMessage.getAction()).getNonRepudiation();
	
			log.info("Sign ? " + isSign);
	
			if( isSign ) {
				log.info("EbxmlMessage Sign մϴ.");
				
				ebxmlMessage = new EbmsSignature().signEbxmlMessage(ebxmlMessage);
			}
		}
		catch(EbmsCpaException e) {
			throw new EbmsEbxmlMessageException(1, 0, "CPA  ־ Sign ߽ϴ.", e);
		}
		catch(EbmsSignException e) {
			throw new EbmsEbxmlMessageException(1, 0, "Sign ߽ϴ.", e);
		}
		catch(EbmsKeyStoreException e) {
			throw new EbmsEbxmlMessageException(1, 0, "KeyStore  ־ Sign ߽ϴ.", e);
		}
	}
}