/*
 * Copyright (c) Sybase, Inc. 2002   
 * All rights reserved.                                    
 * 
 * This file contains Original Code and/or Modifications of 
 * Original Code as defined in and that are subject to the  
 * Sybase Public Source License version 1.0 (the           
 * "License").  You may not use this file except in         
 * compliance with the License.  Please obtain a copy of    
 * the License at http://www.sybase.com/developer/opensource 
 * and read it before using this file.                               
 * 
 * The Original Code and all software distributed under the 
 * License are distributed on an 'AS IS' BASIS, WITHOUT     
 * WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND     
 * SYBASE HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING  
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,    
 * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR     
 * NON-INFRINGEMENT.                                        
 * 
 * Please see the License for the specific language.         
 */
package org.kldp.ebmsh.message.util;

/**
 * Constants defined for SOAP.
 *
 * @Cliff Collins
 */
public interface SOAPConstants
{
    // Define the new line character.
    public static final String	CR = System.getProperty("line.separator", "\n");

    // XML Declaration string
    public static final String	XML_DECL = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + CR;

    ///////////////////////////////////////////////////////////////////////////////
    // NAME SPACES DEFINES:
    ///////////////////////////////////////////////////////////////////////////////
    public static final String NS_PRE_SOAP_ENV = "SOAP-ENV";
    public static final String NS_PRE_SOAP_ENC = "SOAP-ENC";
    public static final String NS_PRE_XMLNS = "xmlns";
    public static final String NS_PRE_SCHEMA_XSI = "xsi";
    public static final String NS_PRE_SCHEMA_XSD = "xsd";
    /**
	 * String representing the xml NameSpace URI
	 */
    public static final String NS_PRE_XML = "xml";
    /**
	 * String representing the xlink NameSpace URI
	 */
    public static final String NS_PRE_XLINK = "xlink";

    // Namespace URIs.  
    public static final String NS_URI_SOAP_ENV = "http://schemas.xmlsoap.org/soap/envelope/";
    public static final String NS_URI_SOAP_ENC = "http://schemas.xmlsoap.org/soap/encoding/";
    public static final String NS_URI_XMLNS = "http://www.w3.org/2000/xmlns/";
    public static final String NS_URI_XML = "http://www.w3.org/XML/1998/namespace";
    public static final String NS_URI_XLINK = "http://www.w3.org/1999/xlink";

    public static final String NS_URI_1999_SCHEMA_XSI = "http://www.w3.org/1999/XMLSchema-instance";
    public static final String NS_URI_1999_SCHEMA_XSD = "http://www.w3.org/1999/XMLSchema";
    public static final String NS_URI_2000_SCHEMA_XSI = "http://www.w3.org/2000/10/XMLSchema-instance";
    public static final String NS_URI_2000_SCHEMA_XSD = "http://www.w3.org/2000/10/XMLSchema";
    public static final String NS_URI_2001_SCHEMA_XSI = "http://www.w3.org/2001/XMLSchema-instance";
    public static final String NS_URI_2001_SCHEMA_XSD = "http://www.w3.org/2001/XMLSchema";
    public static final String NS_URI_CURRENT_SCHEMA_XSI = NS_URI_2001_SCHEMA_XSI;
    public static final String NS_URI_CURRENT_SCHEMA_XSD = NS_URI_2001_SCHEMA_XSD;

    /**
	 * Predefine Static Namespaces.
	 */
   
    // placed in front of the schema defined location.
    public static final String SCHEMA_LOCATION = "schemaLocation";

    /** 
	 * Define the schema locations for SOAP Envelope parts.
	 */
    public static final String SCHEMA_SOAP_ENV = "http://www.oasis-open.org/committees/ebxml-msg/schema/envelope.xsd";
    //public static final String SCHEMA_SOAP_ENV = "http://schemas.xmlsoap.org/soap/envelope/";

    /**
     * Predefine the SOAP Attributes.
     */
   
    ///////////////////////////////////////////////////////////////////////////////
    // MIME/HTTP DEFINES:
    ///////////////////////////////////////////////////////////////////////////////
    // HTTP header field names.
    public static final String HEADER_POST = "POST";
    public static final String HEADER_CONTENT_TYPE = "Content-Type";
    public static final String HEADER_CONTENT_LENGTH = "Content-Length";
    public static final String HEADER_CONTENT_LOCATION = "Content-Location";
    public static final String HEADER_CONTENT_ENCODING = "Content-Transfer-Encoding";
    public static final String HEADER_CONTENT_ID = "Content-ID";
    public static final String HEADER_TYPE = "type";
    public static final String HEADER_START = "start";
    public static final String HEADER_BOUNDARY = "boundary";
    public static final String HEADER_CHARSET = "charset";

    public static final String HEADER_SOAP_ACTION = "SOAPAction";

    public static final String FAULT_SOAP_ACTION = "ack";
	public static final String ACK_SOAP_ACTION = "ack";
	
	public static final String MessageContentId = "MessageContentId";

    // HTTP header field values.
    public static final String HEADERVAL_CONTENT_ENCODING_BASE64 = "base64";
    public static final String HEADERVAL_CONTENT_ENCODING_7BIT = "7bit";
    public static final String HEADERVAL_DEFAULT_CHARSET = "iso-8859-1";
    public static final String HEADERVAL_CHARSET_UTF8 = "UTF-8";
    public static final String HEADERVAL_CONTENT_TYPE = "text/xml";
    public static final String HEADERVAL_CHAR_EXTENSION = "; charset=";
    public static final String HEADERVAL_CONTENT_TYPE_UTF8 =
        HEADERVAL_CONTENT_TYPE + HEADERVAL_CHAR_EXTENSION + HEADERVAL_CHARSET_UTF8;
    public static final String HEADERVAL_CONTENT_TYPE_MULTIPART_PRIMARY = "multipart";
    public static final String HEADERVAL_MULTIPART_CONTENT_SUBTYPE = "related";
    public static final String HEADERVAL_CONTENT_TYPE_MULTIPART =
        HEADERVAL_CONTENT_TYPE_MULTIPART_PRIMARY + '/' +
        HEADERVAL_MULTIPART_CONTENT_SUBTYPE;
    public static final String HEADERVAL_SOAP_ENV_ID = "EAServer-ebXML-SOAP-ENV";

    /**
	 * SOAP messages: relative URIs in the SOAP message use the base URI of this. (see RFC 2111)
	 */
    public static final String DEFAULT_BASEURI = "thismessage:/";

    /**
     * List of MIME headers to strip from JavaMail-generated header.
     */
    public static final String[] IGNORE_MIME_HEADERS =
    {
    	// NOTE: Content-Transfer-Encoding is "illegal" in http 1.1 but 
    	// when we send a smime message it fails to decode if we
    	// don't base64 encode it also. So technically, we should add
    	// Content-Transfer-Encoding to the list of ignore headers but we
    	// can't and still work with smime encryption.
        "Message-ID", "Mime-Version"//, "Content-Transfer-Encoding"
    }
    ;

    ///////////////////////////////////////////////////////////////////////////////
    // ELEMENT DEFINES:
    ///////////////////////////////////////////////////////////////////////////////
    // Element names.
    public static final String ELEM_ENVELOPE = "Envelope";
    public static final String ELEM_BODY = "Body";
    public static final String ELEM_HEADER = "Header";
    public static final String ELEM_FAULT = "Fault";
    public static final String ELEM_FAULT_CODE = "faultcode";
    public static final String ELEM_FAULT_STRING = "faultstring";
    public static final String ELEM_FAULT_ACTOR = "faultactor";
    public static final String ELEM_DETAIL = "detail";

    ///////////////////////////////////////////////////////////////////////////////
    // ATTRIBUTE DEFINES:
    ///////////////////////////////////////////////////////////////////////////////
    // SOAP Attribute names.
    public static final String ATTR_ENCODING_STYLE = "encodingStyle";
    public static final String ATTR_MUST_UNDERSTAND = "mustUnderstand";
    public static final String ATTR_ACTOR = "actor";

    // mustUnderstand value must be "1" if the attribute is used.
    public static final String ATTR_MUST_UNDERSTAND_VALUE = "1";
    public static final String ATTR_MUST_UNDERSTAND_OFF_VALUE = "0";

    // according to the SOAP spec the value of the Actor should be:
    public static final String ATTR_SOAP_NEXT_ACTOR_VALUE = "http://schemas.xmlsoap.org/soap/actor/next";
	// this preceeds all id fields in references to the payloads.
	  public static final String CID_FIELD = "cid:";

    ///////////////////////////////////////////////////////////////////////////////
    // SOAP FAULT DEFINES:
    ///////////////////////////////////////////////////////////////////////////////
    // SOAP defined fault codes.
    public static String FAULT_CODE_VERSION_MISMATCH = NS_PRE_SOAP_ENV + ":VersionMismatch";
    public static String FAULT_CODE_MUST_UNDERSTAND = NS_PRE_SOAP_ENV + ":MustUnderstand";
    public static String FAULT_CODE_CLIENT = NS_PRE_SOAP_ENV + ":Client";
    public static String FAULT_CODE_SERVER = NS_PRE_SOAP_ENV + ":Server";
    public static String FAULT_CODE_PROTOCOL = NS_PRE_SOAP_ENV + ":Protocol";
}
