package org.kldp.ebmsh.message.util;


import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

import javax.net.ServerSocketFactory;
import javax.net.SocketFactory;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamSource;

import org.kldp.ebmsh.message.packaging.impl.EbxmlMessageFactoryImpl;


/**TODO replce this utility 
 * WSRM Utility Methods.  Each thread should have its own
 * instance of this utility class for XML binding, parsing,
 * and transformation.  Some static methods are also provided
 * for convenience (e.g., unique ID generation).
 *
 * @author Mark Hansen (mark@javector.com)
 */
public class WSRMUtil {
	
	
	// for for creating XSLT Transformers
	private TransformerFactory transformerFac = null;
	// for transforming XML into and out of a JAXM SOAPMessage
	private Transformer transformer = null;
	// factory for creating instances of SOAPMessage
	private MessageFactory jaxmFac = null;
	// factory for server sockets
	private ServerSocketFactory ssocfac = null;
	// factory for sockets
	private SocketFactory socfac = null;
	// -------------------------------------------- Constructors
	
	/**
	 * Construct the utility class and initialize the instance
	 * variables.
	 */
	public WSRMUtil() {
		
		try {
			
			jaxmFac = MessageFactory.newInstance();
			transformerFac = TransformerFactory.newInstance();
			transformer = transformerFac.newTransformer(); 
			
		} catch (Exception e) {
			e.printStackTrace() ;
		}
		
	}
	
	/**
	 * Create an empty JAXM SOAP message using the default
	 * message factorty within this class.
	 *
	 * @return an <code>javax.xml.soapSOAPMessage</code> 
	 *    instance
	 *
	 * @throws a WSRMException if an error occurs
	 */
	public SOAPMessage createSOAPMessage() 
	throws SOAPException {
		
		try {
			return getMessageFactory().createMessage(); 
		} catch (SOAPException se) {
			throw new SOAPException
			("SOAP error creating JAXM message.", se);
		}
		
	}
	/**
	 * Return the generic MessageFactory.
	 * MessageFactory is not thread safe.
	 *
	 * @return an instance of MessageFactory
	 */
	public MessageFactory getMessageFactory() {
		return jaxmFac; }
	
	
	/**
	 * Construct a JAXM <code>SOAPMessage</code> from the
	 * contents of an XML <code>File</code>.
	 *
	 * @param xmlFile - the <code>File</code> to convert.
	 *
	 * @return a JAXM <code>SOAPMessage</code> object
	 *    constructed from the contents of the <code>File</code>
	 *    provided.
	 */
	public SOAPMessage fileToSOAP(File xmlFile) throws SOAPException 
	{
		
		SOAPMessage jaxm = createSOAPMessage();
		try {
			StreamSource src = 
				new StreamSource(new FileInputStream(xmlFile));
			jaxm.getSOAPPart().setContent(src);
		} catch (FileNotFoundException fe) {
			fe.printStackTrace() ;
		} catch (SOAPException se) {
			se.printStackTrace() ;
		}
		return jaxm;
		
	}
	
	/**
	 * Construct a JAXM <code>SOAPMessage</code> from a
	 * <code>String</code> containing XML.
	 *
	 * @param xmlFile - the <code>String</code> to convert.
	 *
	 * @return a JAXM <code>SOAPMessage</code> object
	 *    constructed from the contents of the
	 *    <code>String</code> provided.
	 */
	public SOAPMessage stringToSOAP(String str) throws SOAPException
	{
		
		SOAPMessage jaxm = createSOAPMessage();
		ByteArrayInputStream in = 
			new ByteArrayInputStream(str.getBytes());
		try {
			StreamSource src = new StreamSource(in);
			jaxm.getSOAPPart().setContent(src);
		} catch (SOAPException se) {
			se.printStackTrace() ;
		}
		return jaxm;
		
	}
	
	public static SOAPMessage stringToSOAP(ByteArrayOutputStream ot) 
	{
		SOAPMessage jaxm=null;
		try {
			EbxmlMessageFactoryImpl f = new EbxmlMessageFactoryImpl();
			jaxm = f.createMessage();
			/*
			 String si = new String(ot.toByteArray());
			 
			 int pos = si.indexOf("soap:Header");
			 String relSoap = si.substring(0,pos + 11) + " xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\"";
			 relSoap += si.substring(pos + 11);
			 System.out.println("\n" + relSoap + "\n");*/
			ByteArrayInputStream in = new ByteArrayInputStream(ot.toByteArray());
			StreamSource src = new StreamSource(in);
			jaxm.getSOAPPart().setContent(src);
		} catch (Exception se) {
			se.printStackTrace();
		}
		return jaxm;
		
	}
	
	
	/**
	 * Return a TransformerFactory. 
	 * Per Sun javadocs, an implementation of the 
	 * TransformerFactory class is NOT guaranteed to be thread
	 * safe.
	 *
	 * @return a instance of TransformerFactory
	 */
	public TransformerFactory getTransformerFactory() { 
		return transformerFac; }
	
	/**
	 * Return the generic Transformer.
	 * Transformer is not thread safe.
	 *
	 * @return a instance of TransformerFactory
	 */
	public Transformer getTransformer() { 
		return transformer;
	}	
}