package org.kldp.ebmsh.queue;


import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.QueueSession;


public class SendMessageConverter implements MessageConverter {

	public Message convertToJMSMessage(QueueSession queueSession, QueueMessage queueMessage) throws EbmsQueueMessageException {
		MapMessage mapMessage = null;
		SendMessage sendMessage = (SendMessage)queueMessage;
		
		try {
			validateMessage(sendMessage);
			
			mapMessage = queueSession.createMapMessage();
			
			mapMessage.setString("cpaId", sendMessage.getCpaId());
			mapMessage.setString("from", sendMessage.getFrom());
			mapMessage.setString("to", sendMessage.getTo());
			mapMessage.setString("action", sendMessage.getAction());
			mapMessage.setString("conversationId", sendMessage.getConversationId());
			mapMessage.setString("service", sendMessage.getService());
			
			if( sendMessage.getPayloadCount() != 0 ) {
				mapMessage.setInt("payloadCount", sendMessage.getPayloadCount());
				
				for(int i = 0; i < sendMessage.getPayloadCount(); i++) 
					mapMessage.setBytes("payload" + i, sendMessage.getPayloads()[i]);
			}
			else 
				mapMessage.setInt("payloadCount", 0);
		}
		catch(JMSException e) {
			throw new EbmsQueueException(1, 0, e);
		}
		
		return mapMessage;
	}

	
	public QueueMessage convertToQueueMessage(Message message) throws EbmsQueueMessageException {
		MapMessage mapMessage= null;
		SendMessage sendMessage = null;
		
		try {
			// message validation
			this.validateMessage(message);
			
			// ޽ casting
			mapMessage = (MapMessage)message;
			sendMessage = new SendMessage();
			
			sendMessage.setCpaId(mapMessage.getString("cpaId"));
			sendMessage.setFrom(mapMessage.getString("from"));
			sendMessage.setTo(mapMessage.getString("to"));
			sendMessage.setAction(mapMessage.getString("action"));
			sendMessage.setConversationId(mapMessage.getString("conversationId"));
			sendMessage.setService(mapMessage.getString("service"));
			
			int payloadCount = mapMessage.getInt("payloadCount");
			
			if( payloadCount != 0  ) {
				byte[][] payloads = new byte[payloadCount][0];
				
				for(int i = 0; i < payloadCount; i++) 
					payloads[i] = mapMessage.getBytes("payload" + i);
				
				sendMessage.setPayloadCount(payloadCount);
				sendMessage.setPayloads(payloads);
			}
			else 
				sendMessage.setPayloadCount(0);
			
		}
		catch(JMSException e) {
			throw new EbmsQueueException(1, 0, e);
		}
		
		return sendMessage;
	}

	public void validateMessage(Message message) throws EbmsQueueMessageException {
		try {
			if( message == null )
				throw new IllegalArgumentException("Message nullԴϴ.");
			
			if( !(message instanceof MapMessage) )
				throw new IllegalArgumentException("Message Ÿ MapMessage ƴմϴ.");
			
			MapMessage _mapMessage = (MapMessage)message;
			
			if( !_mapMessage.itemExists("cpaId") )
				throw new IllegalArgumentException("CpaId nullԴϴ.");
			
			if( !_mapMessage.itemExists("from") )
				throw new IllegalArgumentException("From nullԴϴ.");
			
			if( !_mapMessage.itemExists("to") ) 
				throw new IllegalArgumentException("To nullԴϴ.");
			
			if( !_mapMessage.itemExists("service") )
				throw new IllegalArgumentException("Service nullԴϴ.");
			
			if( !_mapMessage.itemExists("conversationId") )
				throw new IllegalArgumentException("ConversationId nullԴϴ.");
			
			if( !_mapMessage.itemExists("action") )
				throw new IllegalArgumentException("Action nullԴϴ.");
			
			if( !_mapMessage.itemExists("payloadCount") )
				throw new IllegalArgumentException("PayloadCount nullԴϴ.");
				
			int payloadCount = _mapMessage.getInt("payloadCount");
			
			if( payloadCount != 0 ) {
				byte[] tmpPayload = _mapMessage.getBytes("payload" + (payloadCount - 1));
				
				if( tmpPayload == null ) {
					throw new IllegalArgumentException("÷ι  ġ ʽϴ.");
				}
			}
		}
		catch(IllegalArgumentException e) {
			throw new EbmsQueueException(1, 0, e);
		}
		catch(JMSException e) {
			throw new EbmsQueueException(1, 0, e);
		}
	}

	public void validateMessage(QueueMessage queueMessage) throws EbmsQueueMessageException {
		try {
			if( queueMessage == null )
				throw new IllegalArgumentException("SendMessage nullԴϴ.");
		}
		catch(IllegalArgumentException e) {
			throw new EbmsQueueException(1, 0, e);
		}
	}
	
}