package org.kldp.ebmsh.queue;

import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.QueueSession;


public class ToApplicationMessageConverter implements MessageConverter {
	public Message convertToJMSMessage(QueueSession queueSession, QueueMessage queueMessage) throws EbmsQueueMessageException {
		MapMessage message = null;
		ToApplicationMessage _applicationMessage = (ToApplicationMessage)queueMessage;

		try {
			validateMessage(_applicationMessage);

			message = queueSession.createMapMessage();

			message.setLong("uid", _applicationMessage.getUid());
			message.setString("messageId", _applicationMessage.getMessageId());
			message.setString("cpaId", _applicationMessage.getCpaId());
			message.setString("from", _applicationMessage.getFrom());
			message.setString("to", _applicationMessage.getTo());
			message.setString("action", _applicationMessage.getAction());
			message.setString("conversationId", _applicationMessage.getConversationId());
			message.setString("service", _applicationMessage.getService());
			message.setString("timestamp", _applicationMessage.getTimestamp());
			message.setBoolean("block", _applicationMessage.getBlock());

			if( _applicationMessage.getPayloadCount() != 0 ) {
				message.setInt("payloadCount", _applicationMessage.getPayloadCount());

				for(int i = 0; i < _applicationMessage.getPayloadCount(); i++) 
					message.setBytes("payload" + i, _applicationMessage.getPayloads()[i]);
			}
			else 
				message.setInt("payloadCount", 0);
		}
		catch(JMSException e) {
			throw new EbmsQueueMessageException(0, 0, e);
		}

		return message;
	}

	public QueueMessage convertToQueueMessage(Message message) throws EbmsQueueMessageException {
		ToApplicationMessage applicationMessage = null;
		MapMessage mapMessage = null;

		try {
			validateMessage(message);

			applicationMessage = new ToApplicationMessage();

			mapMessage = (MapMessage)message;

			applicationMessage.setUid(mapMessage.getLong("uid"));
			applicationMessage.setMessageId(mapMessage.getString("messageId"));
			applicationMessage.setCpaId(mapMessage.getString("cpaId"));
			applicationMessage.setFrom(mapMessage.getString("from"));
			applicationMessage.setTo(mapMessage.getString("to"));
			applicationMessage.setAction(mapMessage.getString("action"));
			applicationMessage.setConversationId(mapMessage.getString("conversationId"));
			applicationMessage.setService(mapMessage.getString("service"));
			applicationMessage.setTimestamp(mapMessage.getString("timestamp"));
			applicationMessage.setBlock(mapMessage.getBoolean("block"));

			int payloadCount = mapMessage.getInt("payloadCount");

			if (payloadCount != 0) {
				byte[][] payloads = new byte[payloadCount][0];

				for (int i = 0; i < payloadCount; i++)
					payloads[i] = mapMessage.getBytes("payload" + i);

				applicationMessage.setPayloads(payloads);
			} else
				applicationMessage.setPayloadCount(0);

		}
		catch(JMSException e) {
			throw new EbmsQueueMessageException(0, 0, e);
		}

		return applicationMessage;
	}

	public void validateMessage(Message message) throws EbmsQueueMessageException {
		try {
			if( message == null )
				throw new IllegalArgumentException("Message nullԴϴ.");

			if( !(message instanceof MapMessage) )
				throw new IllegalArgumentException("Message Ÿ MapMessage ƴմϴ.");

			MapMessage _mapMessage = (MapMessage)message;

			if (!_mapMessage.itemExists("uid"))
				throw new IllegalArgumentException("UID nullԴϴ.");

			if (!_mapMessage.itemExists("cpaId"))
				throw new IllegalArgumentException("CpaId nullԴϴ.");

			if (!_mapMessage.itemExists("from"))
				throw new IllegalArgumentException("From nullԴϴ.");

			if (!_mapMessage.itemExists("to"))
				throw new IllegalArgumentException("To nullԴϴ.");

			if (!_mapMessage.itemExists("service"))
				throw new IllegalArgumentException("Service nullԴϴ.");

			if (!_mapMessage.itemExists("conversationId"))
				throw new IllegalArgumentException("ConversationId nullԴϴ.");

			if (!_mapMessage.itemExists("action"))
				throw new IllegalArgumentException("Action nullԴϴ.");

			if (!_mapMessage.itemExists("messageId"))
				throw new IllegalArgumentException("MessageId nullԴϴ.");

			if (!_mapMessage.itemExists("timestamp"))
				throw new IllegalArgumentException("Timestamp nullԴϴ.");

			if (!_mapMessage.itemExists("payloadCount"))
				throw new IllegalArgumentException("PayloadCount nullԴϴ.");

			if (!_mapMessage.itemExists("block"))
				throw new IllegalArgumentException("Block nullԴϴ.");

			int payloadCount = _mapMessage.getInt("payloadCount");

			if (payloadCount != 0) {
				byte[] tmpPayload = _mapMessage.getBytes("payload" + (payloadCount - 1));

				if (tmpPayload == null) {
					throw new IllegalArgumentException("÷ι  ġ ʽϴ.");
				}
			}
		}
		catch(IllegalArgumentException e) {
			throw new EbmsQueueMessageException(1, 0, e);
		}
		catch(JMSException e) {
			throw new EbmsQueueMessageException(1, 0, e);
		}
	}

	public void validateMessage(QueueMessage queueMessage) throws EbmsQueueMessageException {
		try {
			if( queueMessage == null )
				throw new IllegalArgumentException("ToApplicationMessage nullԴϴ.");
		}
		catch(IllegalArgumentException e) {
			throw new EbmsQueueException(1, 0, e);
		}
	}

}
