package org.kldp.ebmsh.queue;

import java.util.Date;

import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.QueueSession;

/**
 * TrackerMessage Ʈ ִ Ŭ
 * 
 * @author Sirius
 */
public class TrackerMessageConverter implements MessageConverter {
	public QueueMessage convertToQueueMessage(Message message) throws EbmsQueueMessageException {
		TrackerMessage trackerMessage = null;

		try {
			validateMessage(message);

			trackerMessage = new TrackerMessage();

			MapMessage mapMessage = (MapMessage)message;

			trackerMessage.setMessageId(mapMessage.getString("messageId"));

			Date date = new Date(mapMessage.getLong("timestamp"));
			trackerMessage.setTimestamp(date);

			trackerMessage.setState(mapMessage.getString("state"));

			trackerMessage.setInformation(mapMessage.getString("information"));
		}
		catch(JMSException e) {
			throw new EbmsQueueMessageException(0, 0, e);
		}

		return trackerMessage;
	}

	public Message convertToJMSMessage(QueueSession queueSession, QueueMessage queueMessage) throws EbmsQueueMessageException {
		MapMessage mapMessage = null;
		TrackerMessage trackerMessage = (TrackerMessage)queueMessage;

		try {
			validateMessage(trackerMessage);

			mapMessage = queueSession.createMapMessage();

			mapMessage.setString("messageId", trackerMessage.getMessageId());

			mapMessage.setLong("timestamp", trackerMessage.getTimestamp().getTime());

			mapMessage.setString("state", trackerMessage.getState());

			mapMessage.setString("information", trackerMessage.getInformation());
		}
		catch(JMSException e) {
			throw new EbmsQueueMessageException(0, 0, e);
		}

		return mapMessage;
	}

	public void validateMessage(Message message) throws EbmsQueueMessageException {
		try {
			if( message == null )
				throw new IllegalArgumentException("Message nullԴϴ.");

			if( !(message instanceof MapMessage) )
				throw new IllegalArgumentException("Message Ÿ MapMessage ƴմϴ.");

			MapMessage _mapMessage = (MapMessage)message;

			if( !_mapMessage.itemExists("messageId") )
				throw new IllegalArgumentException("MapMessage MessageId  ϴ.");

			if( !_mapMessage.itemExists("timestamp") )
				throw new IllegalArgumentException("MapMessage Timestamp  ϴ.");

			if( !_mapMessage.itemExists("state") )
				throw new IllegalArgumentException("MapMessage State  ϴ.");

			if( !_mapMessage.itemExists("information") )
				throw new IllegalArgumentException("MapMessage Information  ϴ.");
		}
		catch(IllegalArgumentException e) {
			throw new EbmsQueueMessageException(1, 0, e);
		}
		catch(JMSException e) {
			throw new EbmsQueueMessageException(0, 0, e);
		}
	}

	public void validateMessage(QueueMessage queueMessage) throws EbmsQueueMessageException {
		try {
			if( queueMessage == null )
				throw new IllegalArgumentException("TrackerMessage nullԴϴ.");
			
			TrackerMessage trackerMessage = (TrackerMessage)queueMessage;
			
			if( trackerMessage.getMessageId() == null )
				throw new IllegalArgumentException("MessageId nullԴϴ.");

			if( trackerMessage.getTimestamp() == null )
				throw new IllegalArgumentException("Timestamp nullԴϴ.");

			if( trackerMessage.getState() == null )
				throw new IllegalArgumentException("State nullԴϴ.");

			if( trackerMessage.getInformation() == null )
				throw new IllegalArgumentException("Information nullԴϴ.");
		}
		catch(IllegalArgumentException e) {
			throw new EbmsQueueException(1, 0, e);
		}
	}
}