package org.kldp.util.xml;

import java.io.*;
import javax.xml.parsers.*;

import org.w3c.dom.*;
import org.xml.sax.*;


/**
 * Dom parser ϱ  util
 * 
 * @author	antasis9
 * @version	1.0
 * @since	2006.5.10
 */
public class XmlParserUtil {
	private DocumentBuilderFactory documentBuilderFactory = null;
	private DocumentBuilder documentBuilder = null;
	private Document document = null;
	private Element rootElement = null;
	
	/**
	 * ⺻ 
	 *  
	 */
	private XmlParserUtil() { }
	
	
	/**
	 * 
	 * 
	 * @param _xmlFile				Parsing  Ǵ xml File object
	 * @throws XmlParserException	_xmlFile Parsing   ߻		
	 */
	public XmlParserUtil(File _xmlFile) throws XmlParserException {
		try {
			documentBuilderFactory = DocumentBuilderFactory.newInstance();
			documentBuilderFactory.setNamespaceAware(true);
			documentBuilderFactory.setIgnoringComments(true);
			documentBuilderFactory.setIgnoringElementContentWhitespace(true);
			
			documentBuilder = documentBuilderFactory.newDocumentBuilder();		
			document = documentBuilder.parse(_xmlFile);
			rootElement = document.getDocumentElement();
		}
		catch(SAXException e) {
			throw new XmlParserException("Parsing ");
		}
		catch(ParserConfigurationException e) {
			throw new XmlParserException("Parsing ");
		}
		catch(IOException e) {
			throw new XmlParserException("Parsing ");
		}
	}
	
	
	/**
	 * 
	 * 
	 * @param _xmlStream			Parsing  Ǵ xml input stream
	 * @throws XmlParserException	_xmlFile Parsing   ߻		
	 */
	public XmlParserUtil(InputStream _xmlStream) throws XmlParserException {
		try {
			documentBuilderFactory = DocumentBuilderFactory.newInstance();
			documentBuilderFactory.setNamespaceAware(true);
			documentBuilderFactory.setIgnoringComments(true);
			documentBuilderFactory.setIgnoringElementContentWhitespace(true);
			
			documentBuilder = documentBuilderFactory.newDocumentBuilder();		
			document = documentBuilder.parse(_xmlStream);
			rootElement = document.getDocumentElement();
		}
		catch(SAXException e) {
			throw new XmlParserException("Parsing ");
		}
		catch(ParserConfigurationException e) {
			throw new XmlParserException("Parsing ");
		}
		catch(IOException e) {
			throw new XmlParserException("Parsing ");
		}
	}
	
	
	/**
	 * Xml Ʈ ã´.
	 * 
	 * @param _elementName			Element ̸
	 * @return						_elementName ġϴ element
	 * @throws XmlParserException	_elementName ġϴ element ã   ߻
	 */
	public Element getElement(String _elementName) throws XmlParserException {
		Node element = null;
		
		NodeList nodeList = rootElement.getElementsByTagName(_elementName);
		
		element = nodeList.item(0);
		
		if( element == null )
			throw new XmlParserException(_elementName + " Ʈ ã  ϴ.");
		
		return (Element)element;
	}
	
	
	/**
	 * Xml Ʈ ã´.
	 * 
	 * @param _elementName			Element ̸
	 * @param _namespace			Element namespace
	 * @return						_elementName ġϴ element
	 * @throws XmlParserException	_elementName ġϴ element ã   ߻
	 */
	public Element getElement(String _elementName, String _namespace) throws XmlParserException {
		Node element = null;
		
		NodeList nodeList = rootElement.getElementsByTagNameNS(_elementName, _namespace);
		
		element = nodeList.item(0);
		
		if( element == null )
			throw new XmlParserException(_elementName + " Ʈ ã  ϴ.");
		
		return (Element)element;
	}
	
	
	/**
	 * Xml Ʈ  
	 * 
	 * @param _elementName			Element ̸
	 * @return						Element value
	 * @throws XmlParserException	Ʈ ã   ߻
	 */
	public String getElementValue(String _elementName) throws XmlParserException {
		String value = null;
		
		try {
			Node node = this.getElement(_elementName);
			Node firstNode = node.getFirstChild();
			
			value = firstNode.getNodeValue();
		}
		catch(NullPointerException e) {
			throw new XmlParserException(_elementName + " Ʈ ã  ϴ.");
		}
		
		return value;
	}
	
	
	/**
	 * Xml Ʈ  
	 * 
	 * @param _elementName			Element ̸
	 * @param _namespace			Element namespace
	 * @return						Element value
	 * @throws XmlParserException	Ʈ ã   ߻
	 */
	public String getElementValue(String _elementName, String _namespace) throws XmlParserException {
		String value = null;
		
		try {
			Node node = this.getElement(_elementName, _namespace);
			Node firstNode = node.getFirstChild();
			
			value = firstNode.getNodeValue();
		}
		catch(NullPointerException e) {
			throw new XmlParserException(_elementName + " Ʈ ã  ϴ.");
		}
		
		return value;
	}
	
	
	/**
	 * Xml Ʈ attribute   
	 * 
	 * @param _elementName			Element ̸
	 * @param _attributeName		Attribute ̸
	 * @return						Attribute 
	 * @throws XmlParserException	Element Attribute ã    ߻
	 */
	public String getAttributeValue(String _elementName, String _attributeName) throws XmlParserException {
		String value = null;
		
		Element element = this.getElement(_elementName);
		
		value = element.getAttribute(_attributeName);
		
		if( value == null || value.equals("") )
			throw new XmlParserException(_attributeName + " Ӽ ã  ϴ.");
		
		return value;
	}
	
	
	/**
	 * Xml Ʈ attribute   
	 * 
	 * @param _elementName			Element ̸
	 * @param _elementNamespace		Element namespace
	 * @param _attributeName		Attribute ̸
	 * @param _attributeNamespace	Attribute namespace
	 * @return						Attribute 
	 * @throws XmlParserException	Element Attribute ã    ߻
	 */
	public String getAttributeValue(String _elementName, String _elementNamespace, String _attributeName, String _attributeNamespace) throws XmlParserException {
		String value = null;
		
		Element element = this.getElement(_elementName, _elementNamespace);
		
		value = element.getAttributeNS(_attributeName, _attributeNamespace);
		
		if( value == null || value.equals("") )
			throw new XmlParserException(_attributeName + " Ӽ ã  ϴ.");
		
		return value;
	}
}