package org.kldp.ebmsh.moniter.failHandler;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.kldp.ebmsh.application.NonBlockingApplicationProcesser;
import org.kldp.ebmsh.config.EbmsConstants;
import org.kldp.ebmsh.message.packaging.EbxmlMessage;
import org.kldp.ebmsh.message.persistant.MessageStore;
import org.kldp.ebmsh.message.transport.EbxmlMessageSender;
import org.kldp.ebmsh.message.util.Serializer;
import org.kldp.ebmsh.moniter.persistant.BsiMessageStatus;
import org.kldp.ebmsh.moniter.persistant.EbmsMoniterManager;
import org.kldp.ebmsh.moniter.persistant.InComingMessage;
import org.kldp.ebmsh.moniter.persistant.MoniterPersistantException;
import org.kldp.util.io.FileIoUtil;
import org.kldp.util.log.LogUtil;

public class FailMessageManager {
	
	public List<BiFailMessageInfo> getBusinessLogicFailMessage() throws MoniterPersistantException {
		EbmsMoniterManager moniterManager = new EbmsMoniterManager();
		
		List<BiFailMessageInfo> biFailMessageInfoList = new ArrayList<BiFailMessageInfo>();
		
		List failBsiMessages = moniterManager.searchFailBsiMessages();
		
		for(int i = 0; i < failBsiMessages.size(); i++) {
			BsiMessageStatus bsiMessageStatus = (BsiMessageStatus)failBsiMessages.get(i);
			
			InComingMessage incomingMessage = moniterManager.getInComingMessage(bsiMessageStatus.getMessageId());
			
			BiFailMessageInfo biFailMessageInfo = new BiFailMessageInfo();
			
			biFailMessageInfo.setMessagePath(incomingMessage.getMSGFilePath());
			biFailMessageInfo.setMessageId(bsiMessageStatus.getMessageId());
			biFailMessageInfo.setStatus(bsiMessageStatus.getStatus());
			
			biFailMessageInfoList.add(biFailMessageInfo);
		}
		
		return biFailMessageInfoList;
	}
	
	public List<EngineFailMessageInfo> getEbxmlEngineFaultMessage() {
		List<EngineFailMessageInfo> faultMessageInfoList = new ArrayList<EngineFailMessageInfo>();
		
		String failMessagePath = EbmsConstants.messagePath + FileIoUtil.fileSeparatorAddOrNone("fail");
		
		File failMessageDirectory = new File(failMessagePath);
		
		File[] failMessageArray = failMessageDirectory.listFiles();
		
		for(int i = 0; i < failMessageArray.length; i++) {
			File failMessage = failMessageArray[i];
			
			String fileName = failMessage.getName();
			Date fileMakeTime = new Date(failMessage.lastModified());
			
			EngineFailMessageInfo faultMessageInfo = new EngineFailMessageInfo();
			
			faultMessageInfo.setFileName(fileName);
			faultMessageInfo.setFileMakeTime(fileMakeTime);
			
			faultMessageInfoList.add(faultMessageInfo);
		}
		
		return faultMessageInfoList;
	}
	
	public BiFailMessageProcessResult reProcessBiFailMessage(String messageId, String messagePath) {
		String failMessagePath = EbmsConstants.messagePath + messagePath;
		
		BiFailMessageProcessResult biFailMessageProcessResult = new BiFailMessageProcessResult();
		biFailMessageProcessResult.setMessageId(messageId);
		
		try {
			if( !FileIoUtil.hasFile(failMessagePath) ) {
				biFailMessageProcessResult.setSuccess(false);
				biFailMessageProcessResult.setMessage(" ã  ϴ.");
				
				return biFailMessageProcessResult;
			}
			
			InputStream is = null;
			
			EbxmlMessage ebxmlMessage = null;
			
			try {
				is = new FileInputStream(failMessagePath);
				
				ebxmlMessage = Serializer.deserialize(is);
			}
			catch(Throwable e) {
				biFailMessageProcessResult.setSuccess(false);
				biFailMessageProcessResult.setMessage("޽ üȭ   ϴ.");
				biFailMessageProcessResult.setDetailMessage(LogUtil.stackTraceToString(e));
				
				return biFailMessageProcessResult;
			}
			finally {
				if( is != null )
					is.close();
			}
			
			try {	
				new NonBlockingApplicationProcesser().process(ebxmlMessage);
			}
			catch(Throwable e) {
				biFailMessageProcessResult.setSuccess(false);
				biFailMessageProcessResult.setMessage("޽  ߽ϴ.");
				biFailMessageProcessResult.setDetailMessage(LogUtil.stackTraceToString(e));
				
				return biFailMessageProcessResult;
			}
			
			biFailMessageProcessResult.setSuccess(true);
			biFailMessageProcessResult.setMessage("ó û ");
			
			return biFailMessageProcessResult;
		}
		catch(Throwable t) {
			biFailMessageProcessResult.setSuccess(false);
			biFailMessageProcessResult.setMessage("˼   ߻߽ϴ.");
			biFailMessageProcessResult.setDetailMessage(LogUtil.stackTraceToString(t));
			
			return biFailMessageProcessResult;
		}
	}
	
	public EngineFailMessageProcessResult reProcessFailMessage(String failMessageFileName, String url) {
		String failMessageDirectory = EbmsConstants.messagePath + FileIoUtil.fileSeparatorAddOrNone("fail");
		String failMessagePath = failMessageDirectory + failMessageFileName;
		
		EngineFailMessageProcessResult failMessageProcessResult = new EngineFailMessageProcessResult();
		failMessageProcessResult.setFileName(failMessageFileName);
		
		byte[] ebxmlMessageByte = null;
		
		try {
			if( !FileIoUtil.hasFile(failMessagePath) ) {
				failMessageProcessResult.setSuccess(false);
				failMessageProcessResult.setMessage(" ã  ϴ.");
				
				return failMessageProcessResult;
			}
			
			// ޽ Ͽ о
			try {
				ebxmlMessageByte = FileIoUtil.readDataFromFile(failMessagePath);
			}
			catch(Throwable e) {
				failMessageProcessResult.setSuccess(false);
				failMessageProcessResult.setMessage("   ϴ.");
				failMessageProcessResult.setDetailMessage(LogUtil.stackTraceToString(e));
				
				return failMessageProcessResult;
			}
			
			InputStream is = null;
			
			EbxmlMessage ebxmlMessage = null;
			
			try {
				is = new FileInputStream(failMessagePath);
				
				ebxmlMessage = Serializer.deserialize(is);
			}
			catch(Throwable e) {
				failMessageProcessResult.setSuccess(false);
				failMessageProcessResult.setMessage("޽ üȭ   ϴ.");
				failMessageProcessResult.setDetailMessage(LogUtil.stackTraceToString(e));
				
				return failMessageProcessResult;
			}
			finally {
				if( is != null )
					is.close();
			}
			
			// ޽ 
			FileIoUtil.delete(failMessagePath);
			
			EbxmlMessage resultEbxmlMessage = null;
			
			try {
				EbxmlMessageSender sender = new EbxmlMessageSender();
				
				resultEbxmlMessage = sender.sendMessageForFaultHandler(ebxmlMessage, url);
			}
			catch(Throwable e) {
				failMessageProcessResult.setSuccess(false);
				failMessageProcessResult.setMessage("޽  ߽ϴ.");
				failMessageProcessResult.setDetailMessage(LogUtil.stackTraceToString(e));
				
				return failMessageProcessResult;
			}
			
			if( resultEbxmlMessage == null ) {
				failMessageProcessResult.setSuccess(false);
				failMessageProcessResult.setMessage(" ޽ ϴ.");
				
				return failMessageProcessResult;
			}
			
			failMessageProcessResult.setSuccess(true);
			failMessageProcessResult.setMessage("ó û ");
			
			return failMessageProcessResult;
		}
		catch(Throwable t) {
			failMessageProcessResult.setSuccess(false);
			failMessageProcessResult.setMessage("˼   ߻߽ϴ.");
			failMessageProcessResult.setDetailMessage(LogUtil.stackTraceToString(t));
			
			return failMessageProcessResult;
		}
		finally {
			if( !failMessageProcessResult.isSuccess() ) {
				//   ٽ Ͽ 
				// ̹ Ͽ ϵ 쿡  ʴ´.  BizboxServlet   ٽ  ޽ ̹ ϱ 
				if( !FileIoUtil.hasFile(failMessageDirectory + failMessageFileName) ) {
					try {
						FileIoUtil.saveDataToFile(failMessageDirectory, failMessageFileName, ebxmlMessageByte);
					}
					catch(Throwable t) {
						failMessageProcessResult.setSuccess(false);
						failMessageProcessResult.setMessage("Ebxml message Ͽ   ϴ.");
						failMessageProcessResult.setDetailMessage(LogUtil.stackTraceToString(t));
						
						return failMessageProcessResult;
					}
				}
			}
		}
	}
}
