package org.kldp.ebmsh.moniter.persistant;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.dbutils.QueryRunner;
import org.apache.commons.dbutils.ResultSetHandler;
import org.apache.commons.dbutils.handlers.BeanListHandler;
import org.kldp.ebmsh.config.EbmsConstants;


public class EbmsMoniterManager {
	//  ޽ ˻ Section
	/**
	 * DocumentId  ޽ ˻մϴ
	 * 
	 * @param documentId
	 * @return
	 * @throws MoniterPersistantException
	 */
	public List searchReceiveMessagesByDocumentId(String documentId, int pageCount) throws MoniterPersistantException {
		try {
			List resultList = new ArrayList();
			
			MessageIdAndDocumentIdDAO messageIdAndDocumentIdDAO = new MessageIdAndDocumentIdDAO();
			IncomingMessageDAO incomingMessageDAO = new IncomingMessageDAO();
			MessageStatusDAO messageStatusDAO = new MessageStatusDAO();
			BsiMessageStatusDAO bsiMessageStatusDAO = new BsiMessageStatusDAO();
			
			List messageIdList = messageIdAndDocumentIdDAO.getMessageIdsByDocumentId(documentId, pageCount);
			
			int messageIdSize = messageIdList.size();
			
			for(int i = 0; i < messageIdSize; i++) {
				String messageId = (String)messageIdList.get(i);
				
				InComingMessage inComingMessage = incomingMessageDAO.getInComingMessage(messageId);
				
				// MessageId شϴ  incomingMessage  
				if( inComingMessage == null )
					continue ;
				
				MessageStatus messageStatus = messageStatusDAO.getMessageStatus(messageId);
				BsiMessageStatus bsiMessageStatus = bsiMessageStatusDAO.getBsiMessageStatus(messageId);
				
				Timestamp receiveTime = inComingMessage.getReceiveTime();
				int ebxmlMessageStatusValue = MessageStatusConstraints.Unknown;
				int bsiMessssageStatusValue = BsiMessageStatusConstraints.Unknown;
				
				if( messageStatus != null )
					ebxmlMessageStatusValue = messageStatus.getStatus();
				
				if( bsiMessageStatus != null )
					bsiMessssageStatusValue = bsiMessageStatus.getStatus();
				
				SearchResult searchResult = new SearchResult();
				searchResult.setDocumentId(documentId);
				searchResult.setMessageId(messageId);
				searchResult.setReceiveTime(receiveTime);
				searchResult.setEbxmlMessageStatus(ebxmlMessageStatusValue);
				searchResult.setBsiMessageStatus(bsiMessssageStatusValue);
				
				resultList.add(searchResult);
			}
			
			return resultList;
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	/**
	 * MessageId  ޽ ˻մϴ
	 * 
	 * @param documentId
	 * @return
	 * @throws MoniterPersistantException
	 */
	public SearchResult searchReceiveMessageByMessageId(String messageId) throws MoniterPersistantException {
		try {
			MessageIdAndDocumentIdDAO messageIdAndDocumentIdDAO = new MessageIdAndDocumentIdDAO();
			IncomingMessageDAO incomingMessageDAO = new IncomingMessageDAO();
			MessageStatusDAO messageStatusDAO = new MessageStatusDAO();
			BsiMessageStatusDAO bsiMessageStatusDAO = new BsiMessageStatusDAO();
			
			InComingMessage inComingMessage = incomingMessageDAO.getInComingMessage(messageId);
			
			// MessageId شϴ  incomingMessage  
			if( inComingMessage == null )
				return null;
			
			String documentId = messageIdAndDocumentIdDAO.getDocumentIdByMessageId(messageId);						
			MessageStatus messageStatus = messageStatusDAO.getMessageStatus(messageId);
			BsiMessageStatus bsiMessageStatus = bsiMessageStatusDAO.getBsiMessageStatus(messageId);	
			
			int ebxmlMessageStatusValue = MessageStatusConstraints.Unknown;
			int bsiMessssageStatusValue = BsiMessageStatusConstraints.Unknown;
			
			if( messageStatus != null )
				ebxmlMessageStatusValue = messageStatus.getStatus();
			
			if( bsiMessageStatus != null )
				bsiMessssageStatusValue = bsiMessageStatus.getStatus();
			
			Timestamp receiveTime = inComingMessage.getReceiveTime();
			
			SearchResult searchResult = new SearchResult();
			
			searchResult.setDocumentId(documentId);
			searchResult.setMessageId(messageId);
			searchResult.setReceiveTime(receiveTime);
			searchResult.setEbxmlMessageStatus(ebxmlMessageStatusValue);
			searchResult.setBsiMessageStatus(bsiMessssageStatusValue);
			
			return searchResult;
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	
	/**
	 * Ⱓ ޽ ˻մϴ.
	 * 
	 * @param startTime
	 * @param endTime
	 * @param pageCount
	 * @return
	 * @throws MoniterPersistantException
	 */
	public List searchReceiveMessagesByPeriod(long startTime, long endTime, int pageCount) throws MoniterPersistantException {
		try {
			List searchResultList = new ArrayList();
			
			MessageIdAndDocumentIdDAO messageIdAndDocumentIdDAO = new MessageIdAndDocumentIdDAO();
			IncomingMessageDAO incomingMessageDAO = new IncomingMessageDAO();
			MessageStatusDAO messageStatusDAO = new MessageStatusDAO();
			BsiMessageStatusDAO bsiMessageStatusDAO = new BsiMessageStatusDAO();
			
			List inComingMessageInfoList = incomingMessageDAO.getInComingMessagesByPeriod(startTime, endTime, pageCount);
			
			int inComingMessageInfoSize = inComingMessageInfoList.size();
			
			for(int i = 0; i < inComingMessageInfoSize; i++) {
				InComingMessage inComingMessage = (InComingMessage)inComingMessageInfoList.get(i);
				
				if( inComingMessage == null )
					continue;
				
				String messageId = inComingMessage.getMessageId();
				String documentId = messageIdAndDocumentIdDAO.getDocumentIdByMessageId(messageId);
				MessageStatus messageStatus = messageStatusDAO.getMessageStatus(messageId);
				BsiMessageStatus bsiMessageStatus = bsiMessageStatusDAO.getBsiMessageStatus(messageId);
				
				int ebxmlMessageStatusValue = MessageStatusConstraints.Unknown;
				int bsiMessssageStatusValue = BsiMessageStatusConstraints.Unknown;
				
				if( messageStatus != null )
					ebxmlMessageStatusValue = messageStatus.getStatus();
				
				if( bsiMessageStatus != null )
					bsiMessssageStatusValue = bsiMessageStatus.getStatus();
				
				Timestamp receiveTime = inComingMessage.getReceiveTime();
				
				SearchResult searchResult = new SearchResult();
				searchResult.setDocumentId(documentId);
				searchResult.setMessageId(messageId);
				searchResult.setReceiveTime(receiveTime);
				searchResult.setEbxmlMessageStatus(ebxmlMessageStatusValue);
				searchResult.setBsiMessageStatus(bsiMessssageStatusValue);
				
				searchResultList.add(searchResult);
			}
			
			return searchResultList;
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	/**
	 * Ⱓ ˻ ޽   ޽ մϴ.
	 * 
	 * @param startTime
	 * @param endTime
	 * @param pageCount
	 * @return
	 * @throws MoniterPersistantException
	 */
	public List searchFailReceiveMessagesByPeriod(long startTime, long endTime, int pageCount) throws MoniterPersistantException {
		try {
			List searchResultList = new ArrayList();
			
			MessageIdAndDocumentIdDAO messageIdAndDocumentIdDAO = new MessageIdAndDocumentIdDAO();
			IncomingMessageDAO incomingMessageDAO = new IncomingMessageDAO();
			MessageStatusDAO messageStatusDAO = new MessageStatusDAO();
			BsiMessageStatusDAO bsiMessageStatusDAO = new BsiMessageStatusDAO();
			
			List inComingMessageInfoList = incomingMessageDAO.getInComingMessagesByPeriod(startTime, endTime, pageCount);
			
			int inComingMessageInfoSize = inComingMessageInfoList.size();
			
			for(int i = 0; i < inComingMessageInfoSize; i++) {
				InComingMessage inComingMessage = (InComingMessage)inComingMessageInfoList.get(i);
				
				if( inComingMessage == null )
					continue;
				
				String messageId = inComingMessage.getMessageId();				
				String documentId = messageIdAndDocumentIdDAO.getDocumentIdByMessageId(messageId);
				MessageStatus messageStatus = messageStatusDAO.getMessageStatus(messageId);				
				BsiMessageStatus bsiMessageStatus = bsiMessageStatusDAO.getBsiMessageStatus(messageId);
				
				Timestamp receiveTime = inComingMessage.getReceiveTime();
				
				int ebxmlMessageStatusValue = MessageStatusConstraints.Unknown;
				int bsiMessageStatusValue = BsiMessageStatusConstraints.Unknown;
				
				if( messageStatus != null )
					ebxmlMessageStatusValue = messageStatus.getStatus();
				
				if( bsiMessageStatus != null )
					bsiMessageStatusValue = bsiMessageStatus.getStatus();
				
				if( ebxmlMessageStatusValue == MessageStatusConstraints.Unknown || ebxmlMessageStatusValue == MessageStatusConstraints.Fail || bsiMessageStatusValue == BsiMessageStatusConstraints.Unknown || bsiMessageStatusValue == BsiMessageStatusConstraints.Fail ) {
					SearchResult searchResult = new SearchResult();
					searchResult.setDocumentId(documentId);
					searchResult.setMessageId(messageId);
					searchResult.setReceiveTime(receiveTime);
					searchResult.setEbxmlMessageStatus(ebxmlMessageStatusValue);
					searchResult.setBsiMessageStatus(bsiMessageStatusValue);
					
					searchResultList.add(searchResult);
				}
			}
			
			return searchResultList;
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	
	/**
	 * Ⱓ ˻ ޽ ˻  մϴ.
	 * 
	 * @param startTime
	 * @param endTime
	 * @return
	 * @throws MoniterPersistantException
	 */
	public long searchReceiveMessageCountByPeriod(long startTime, long endTime) throws MoniterPersistantException {
		try {
			IncomingMessageDAO incomingMessageDAO = new IncomingMessageDAO();
		
			return incomingMessageDAO.getInComingMessageCountByPeriod(startTime, endTime);
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	public int searchReceiveMessageCountByDocumentId(String documentId) throws MoniterPersistantException {
		try {
			MessageIdAndDocumentIdDAO messageIdAndDocumentIdDAO = new MessageIdAndDocumentIdDAO();
			
			return messageIdAndDocumentIdDAO.getMessageIdCountByDocumentId(documentId);
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	//  ޽ ˻ Section end
	
	
	//  ޽ ˻ Section
	/**
	 * MessageId ޽ ˻մϴ.
	 */
	public List searchSendMessageByMessageId(String messageId) throws MoniterPersistantException {
		try {
			List sendMessages = new ArrayList();
			List outgoingMessages = new OutgoingMessageDAO().getOutgoingMessagesByMessageId(messageId);
			
			for(int i = 0; i < outgoingMessages.size(); i++) {
				SendSearchResult sendSearchResult = new SendSearchResult();
				
				OutgoingMessage outgoingMessage = (OutgoingMessage)outgoingMessages.get(i);
				
				sendSearchResult.setMessageId(outgoingMessage.getMessageId());
				sendSearchResult.setRefToMessageId(outgoingMessage.getRefToMessageId());
				sendSearchResult.setSendTime(outgoingMessage.getSendTime());
				sendSearchResult.setStatus(outgoingMessage.getStatus());
				sendSearchResult.setMSGFilePath(outgoingMessage.getMSGFilePath());
				
				sendMessages.add(sendSearchResult);
			}
			
			return sendMessages;
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	/**
	 * Ⱓ ˻ ޽ ˻  մϴ.
	 * 
	 * @param startTime
	 * @param endTime
	 * @return
	 * @throws MoniterPersistantException
	 */
	public long searchSendMessageCountByMessageId(String messageId) throws MoniterPersistantException {
		try {
			OutgoingMessageDAO outgoingMessageDAO = new OutgoingMessageDAO();
		
			return outgoingMessageDAO.getOutgoingMessageCountByRefToMessageId(messageId);
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	/**
	 * RefToMessageId ޽ ˻մϴ.
	 * 
	 * @param refToMessageId
	 * @return
	 * @throws MoniterPersistantException
	 */
	public List searchSendMessageByRefToMessageId(String refToMessageId) throws MoniterPersistantException {
		try {
			List sendMessages = new ArrayList();
			List outgoingMessages = new OutgoingMessageDAO().getOutgoingMessagesByRefToMessageId(refToMessageId);
			
			for(int i = 0; i < outgoingMessages.size(); i++) {
				SendSearchResult sendSearchResult = new SendSearchResult();
				
				OutgoingMessage outgoingMessage = (OutgoingMessage)outgoingMessages.get(i);
				
				sendSearchResult.setMessageId(outgoingMessage.getMessageId());
				sendSearchResult.setRefToMessageId(outgoingMessage.getRefToMessageId());
				sendSearchResult.setSendTime(outgoingMessage.getSendTime());
				sendSearchResult.setStatus(outgoingMessage.getStatus());
				sendSearchResult.setMSGFilePath(outgoingMessage.getMSGFilePath());
				
				sendMessages.add(sendSearchResult);
			}
			
			return sendMessages;
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	/**
	 * Ⱓ ˻ ޽ ˻  մϴ.
	 * 
	 * @param startTime
	 * @param endTime
	 * @return
	 * @throws MoniterPersistantException
	 */
	public long searchSendMessageCountByRefToMessageId(String refToMessageId) throws MoniterPersistantException {
		try {
			OutgoingMessageDAO outgoingMessageDAO = new OutgoingMessageDAO();
		
			return outgoingMessageDAO.getOutgoingMessageCountByRefToMessageId(refToMessageId);
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}

	
	/**
	 * Ⱓ ޽ ˻մϴ.
	 * 
	 * @param startTime
	 * @param endTime
	 * @return
	 * @throws MoniterPersistantException
	 */
	public List searchSendMessageByPeriod(long startTime, long endTime, int pageCount) throws MoniterPersistantException {
		try {
			List sendMessages = new ArrayList();
			List outgoingMessages = new OutgoingMessageDAO().getOutgoingMessagesByPeriod(startTime, endTime, pageCount);
			
			for(int i = 0; i < outgoingMessages.size(); i++) {
				SendSearchResult sendSearchResult = new SendSearchResult();
				
				OutgoingMessage outgoingMessage = (OutgoingMessage)outgoingMessages.get(i);
				
				sendSearchResult.setMessageId(outgoingMessage.getMessageId());
				sendSearchResult.setRefToMessageId(outgoingMessage.getRefToMessageId());
				sendSearchResult.setSendTime(outgoingMessage.getSendTime());
				sendSearchResult.setStatus(outgoingMessage.getStatus());
				sendSearchResult.setMSGFilePath(outgoingMessage.getMSGFilePath());
				
				sendMessages.add(sendSearchResult);
			}
			
			return sendMessages;
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	/**
	 * Ⱓ ˻ ޽ ˻  մϴ.
	 * 
	 * @param startTime
	 * @param endTime
	 * @return
	 * @throws MoniterPersistantException
	 */
	public long searchSendMessageCountByPeriod(long startTime, long endTime) throws MoniterPersistantException {
		try {
			OutgoingMessageDAO outgoingMessageDAO = new OutgoingMessageDAO();
		
			return outgoingMessageDAO.getOutgoingMessageCountByPeriod(startTime, endTime);
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	//  ޽ ˻ Section end
	
	public boolean authorizeAdmin(String id, String password) throws MoniterPersistantException {
		try {
			AdminDAO adminDAO = new AdminDAO();
			
			return adminDAO.authorizeAdmin(id, password);
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	public void updateAdminPassword(String id, String password) throws MoniterPersistantException {
		try {
			AdminDAO adminDAO = new AdminDAO();
			
			adminDAO.updateAdminPassword(id, password);
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	public void deleteDocumentIdInfo(String action) throws MoniterPersistantException {
		try {
			DocumentIdInfoDAO documentIdInfoDAO = new DocumentIdInfoDAO();
			
			documentIdInfoDAO.deleteDocumentIdInfo(action);
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	public void insertDocumentIdInfo(String action, int payloadIndex, String xPath) throws MoniterPersistantException {
		try {
			DocumentIdInfoDAO documentIdInfoDAO = new DocumentIdInfoDAO();
			
			documentIdInfoDAO.insertDocumentIdInfo(action, payloadIndex, xPath);
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	public void updateDocumentIdInfo(String action, int payloadIndex, String xPath) throws MoniterPersistantException {
		try {
			DocumentIdInfoDAO documentIdInfoDAO = new DocumentIdInfoDAO();
			
			documentIdInfoDAO.updateDocumentIdInfo(action, payloadIndex, xPath);
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	public List getTrackers(String messageId) throws MoniterPersistantException {
		try {
			TrackerDAO trackerDAO = new TrackerDAO();
			
			return trackerDAO.getTrackers(messageId);
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	public MessageStore getMessageStoreMessage(String messageId) throws MoniterPersistantException {
		try {
			MessageStoreDAO messageStoreDAO = new MessageStoreDAO();
			
			return messageStoreDAO.getMessageStoreMessage(messageId);
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	public DocumentIdInfo getDocumentIdInfoByAction(String action) throws MoniterPersistantException {
		try {
			DocumentIdInfoDAO documentIdInfoDAO = new DocumentIdInfoDAO();
			
			return documentIdInfoDAO.getDocumentIdInfoByAction(action);
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	public List getDocumentIdInfos() throws MoniterPersistantException {
		try {
			DocumentIdInfoDAO documentIdInfoDAO = new DocumentIdInfoDAO();
			
			return documentIdInfoDAO.getDocumentIdInfos();
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	public InComingMessage getInComingMessage(String messageId) throws MoniterPersistantException {
		try {
			IncomingMessageDAO incomingMessageDAO = new IncomingMessageDAO();
			
			return incomingMessageDAO.getInComingMessage(messageId);
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	public MessageStatus getMessageStatus(String messageId) throws MoniterPersistantException {
		try {
			return new MessageStatusDAO().getMessageStatus(messageId);
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	/**
	 * MessageId  ޽ ˻մϴ
	 * 
	 * @param documentId
	 * @return
	 * @throws MoniterPersistantException
	 */
	public List searchFailBsiMessages() throws MoniterPersistantException {
		try {
			BsiMessageStatusDAO bsiMessageStatusDAO = new BsiMessageStatusDAO();
			
			return bsiMessageStatusDAO.getBsiMessagesByStatusCode(BsiMessageStatusConstraints.Fail);
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
}
