package org.kldp.ebmsh.moniter.persistant;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.dbutils.QueryRunner;
import org.apache.commons.dbutils.ResultSetHandler;
import org.apache.commons.dbutils.handlers.BeanHandler;
import org.apache.commons.dbutils.handlers.BeanListHandler;
import org.kldp.ebmsh.config.EbmsConstants;


public class MessageIdAndDocumentIdDAO {
	/**
	 * DocumentId شϴ MessageId ˻մϴ.
	 * 
	 * @param trackerKey
	 * @return
	 * @throws EbmsPersistantException
	 */
	List getMessageIdsByDocumentId(String documentId, int pageCount) throws MoniterPersistantException {
		List messageIdAndDocumentIds = null;
		List messageIds = new ArrayList();
		DbTemplate dbTemplate = new DbTemplate();
		
		int perPageContentCount = 10;
		
		int startCount = pageCount * perPageContentCount - 9;
		int endCount = startCount + 10;
		
		final String queryStr = 
			"select * from (" +
				   "select /*+ index_desc(" + EbmsConstants.bizboxDatabaseHeaderName + "MessageIdAndDocumentId idx_" + EbmsConstants.bizboxDatabaseHeaderName + "DocumentId) */ rownum as numrow, " + EbmsConstants.bizboxDatabaseHeaderName + "MessageIdAndDocumentId.* from " + EbmsConstants.bizboxDatabaseHeaderName + "MessageIdAndDocumentId where " + EbmsConstants.bizboxDatabaseHeaderName + "MessageIdAndDocumentId.DocumentId = ?" +
			") where numrow >= ? and numrow <= ?";
		
		final Object[] parameters = new Object[] { documentId, new Integer(startCount), new Integer(endCount) };
		
		messageIdAndDocumentIds = (List)dbTemplate.executeQuery(
				new DbWorker() {
					
					public Object work(Connection _conn) throws MoniterPersistantException {
						QueryRunner runner = new QueryRunner();
						ResultSetHandler resultSetHandler = new BeanListHandler(MessageIdAndDocumentId.class);
						
						List resultList = null;
						
						try {
							resultList = (List)runner.query(_conn, queryStr, parameters, resultSetHandler);
						}
						catch(SQLException e) {
							throw new MoniterPersistantException(0, e.getErrorCode(), e);
						}
						
						return resultList;
					}
					
				});
		
		for(int i = 0; i < messageIdAndDocumentIds.size(); i++) {
			MessageIdAndDocumentId messageIdAndDocumentId = (MessageIdAndDocumentId)messageIdAndDocumentIds.get(i);
			
			messageIds.add(messageIdAndDocumentId.getMessageId());
		}
		
		return messageIds;
	}
	
	
	/**
	 * MessageId شϴ DocumentId ˻մϴ.
	 * 
	 * @param messageId
	 * @return
	 * @throws EbmsPersistantException
	 */
	String getDocumentIdByMessageId(String messageId) throws MoniterPersistantException {
		String trackerKey = null;
		MessageIdAndDocumentId tracker = null;
		
		DbTemplate dbTemplate = new DbTemplate();
		
		final String queryStr = "select * from " + EbmsConstants.bizboxDatabaseHeaderName + "MessageIdAndDocumentId where MessageId=?";
		
		final Object[] parameters = new Object[1];
		parameters[0] = messageId; // [0] MessageId
		
		tracker = (MessageIdAndDocumentId)dbTemplate.executeQuery(
				new DbWorker() {
					
					public Object work(Connection _conn) throws MoniterPersistantException {
						QueryRunner runner = new QueryRunner();
						ResultSetHandler resultSetHandler = new BeanHandler(MessageIdAndDocumentId.class);
						
						MessageIdAndDocumentId tracker = null;
						
						try {
							tracker = (MessageIdAndDocumentId)runner.query(_conn, queryStr, parameters, resultSetHandler);
						}
						catch(SQLException e) {
							throw new MoniterPersistantException(0, e.getErrorCode(), e);
						}
						
						return tracker;
					}
					
				});
		
		if( tracker != null )
			trackerKey = tracker.getDocumentId();
		
		return trackerKey;
	}
	
	
	/**
	 * DocumentId شϴ MessageId ˻մϴ.
	 * 
	 * @param trackerKey
	 * @return
	 * @throws EbmsPersistantException
	 */
	int getMessageIdCountByDocumentId(String documentId) throws MoniterPersistantException {
		List messageIdAndDocumentIds = null;
		DbTemplate dbTemplate = new DbTemplate();
		
		final String queryStr = 
			"select * from " + EbmsConstants.bizboxDatabaseHeaderName + "MessageIdAndDocumentId where DocumentId = ?";
		
		final Object[] parameters = new Object[] { documentId };
		
		messageIdAndDocumentIds = (List)dbTemplate.executeQuery(
				new DbWorker() {
					
					public Object work(Connection _conn) throws MoniterPersistantException {
						QueryRunner runner = new QueryRunner();
						ResultSetHandler resultSetHandler = new BeanListHandler(MessageIdAndDocumentId.class);
						
						List resultList = null;
						
						try {
							resultList = (List)runner.query(_conn, queryStr, parameters, resultSetHandler);
						}
						catch(SQLException e) {
							throw new MoniterPersistantException(0, e.getErrorCode(), e);
						}
						
						return resultList;
					}
					
				});
		
		return messageIdAndDocumentIds.size();
	}
}
