package org.kldp.ebmsh.moniter.persistant;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.List;

import org.apache.commons.dbutils.QueryRunner;
import org.apache.commons.dbutils.ResultSetHandler;
import org.apache.commons.dbutils.handlers.BeanListHandler;
import org.apache.commons.dbutils.handlers.ScalarHandler;
import org.kldp.ebmsh.config.EbmsConstants;


public class OutgoingMessageDAO {
	/**
	 * MessageId شϴ OutgoingConmingMessage 
	 * 
	 * @param startTime
	 * @param endTime
	 * @return
	 * @throws EbmsPersistantException
	 */
	List getOutgoingMessagesByMessageId(String messageId) throws MoniterPersistantException {
		try {
			List outgoingMessages = null;
			
			DbTemplate dbTemplate = new DbTemplate();
			
			final String queryStr = "select * from " + EbmsConstants.bizboxDatabaseHeaderName + "OutgoingMessage where MessageId=?";
			
			final Object[] params = new Object[] { messageId };
			
			outgoingMessages = (List)dbTemplate.executeQuery(
					new DbWorker() {
						public Object work(Connection _conn) throws MoniterPersistantException {
							QueryRunner runner = new QueryRunner();
							ResultSetHandler resultSetHandler = new BeanListHandler(OutgoingMessage.class);
							
							List queryResult = null;
							
							try {
								queryResult = (List)runner.query(_conn, queryStr, params, resultSetHandler);
							}
							catch(SQLException e) {
								throw new MoniterPersistantException(0, e.getErrorCode(), e);
							}
							
							return queryResult;
						}
						
					});
			
			return outgoingMessages;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	/**
	 * RefToMessageId شϴ OutgoingConmingMessage 
	 * 
	 * @param startTime
	 * @param endTime
	 * @return
	 * @throws EbmsPersistantException
	 */
	long getOutgoingMessageCountByMessageId(String messageId) throws MoniterPersistantException {
		try {
			long outgoingMessageCount = 0;
			BigDecimal outgoingMessageCountBigDecimal = null;
			
			DbTemplate dbTemplate = new DbTemplate();
			
			final Object[] params = new Object[] { messageId };
			final String queryStr = 
				"select count(*) from " + EbmsConstants.bizboxDatabaseHeaderName + "OutgoingMessage where MessageId = ?";
			
			outgoingMessageCountBigDecimal = (BigDecimal)dbTemplate.executeQuery(
					new DbWorker() {
						
						public Object work(Connection _conn) throws MoniterPersistantException {
							QueryRunner runner = new QueryRunner();
							ResultSetHandler resultSetHandler = new ScalarHandler();
							
							BigDecimal queryResult = null;
							
							try {
								queryResult = (BigDecimal)runner.query(_conn, queryStr, params, resultSetHandler);
							}
							catch(SQLException e) {
								throw new MoniterPersistantException(0, e.getErrorCode(), e);
							}
							
							return queryResult;
						}
						
					});
			
			if( outgoingMessageCountBigDecimal != null )
				outgoingMessageCount = outgoingMessageCountBigDecimal.longValue();
			
			return outgoingMessageCount;
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	/**
	 * RefToMessageId شϴ OutgoingConmingMessage 
	 * 
	 * @param startTime
	 * @param endTime
	 * @return
	 * @throws EbmsPersistantException
	 */
	List getOutgoingMessagesByRefToMessageId(String refToMessageId) throws MoniterPersistantException {
		try {
			List outgoingMessages = null;
			
			DbTemplate dbTemplate = new DbTemplate();
			
			final String queryStr = "select * from " + EbmsConstants.bizboxDatabaseHeaderName + "OutgoingMessage where RefToMessageId=?";
			
			final Object[] params = new Object[] { refToMessageId };
			
			outgoingMessages = (List)dbTemplate.executeQuery(
					new DbWorker() {
						
						public Object work(Connection _conn) throws MoniterPersistantException {
							QueryRunner runner = new QueryRunner();
							ResultSetHandler resultSetHandler = new BeanListHandler(OutgoingMessage.class);
							
							List queryResult = null;
							
							try {
								queryResult = (List)runner.query(_conn, queryStr, params, resultSetHandler);
							}
							catch(SQLException e) {
								throw new MoniterPersistantException(0, e.getErrorCode(), e);
							}
							
							return queryResult;
						}
						
					});
			
			return outgoingMessages;
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	/**
	 * RefToMessageId شϴ OutgoingConmingMessage 
	 * 
	 * @param startTime
	 * @param endTime
	 * @return
	 * @throws EbmsPersistantException
	 */
	long getOutgoingMessageCountByRefToMessageId(String refToMessageId) throws MoniterPersistantException {
		try {
			long outgoingMessageCount = 0;
			BigDecimal outgoingMessageCountBigDecimal = null;
			
			DbTemplate dbTemplate = new DbTemplate();
			
			final Object[] params = new Object[] { refToMessageId };
			final String queryStr = 
				"select count(*) from " + EbmsConstants.bizboxDatabaseHeaderName + "OutgoingMessage where RefToMessageId = ?";
			
			outgoingMessageCountBigDecimal = (BigDecimal)dbTemplate.executeQuery(
					new DbWorker() {
						
						public Object work(Connection _conn) throws MoniterPersistantException {
							QueryRunner runner = new QueryRunner();
							ResultSetHandler resultSetHandler = new ScalarHandler();
							
							BigDecimal queryResult = null;
							
							try {
								queryResult = (BigDecimal)runner.query(_conn, queryStr, params, resultSetHandler);
							}
							catch(SQLException e) {
								throw new MoniterPersistantException(0, e.getErrorCode(), e);
							}
							
							return queryResult;
						}
						
					});
			
			if( outgoingMessageCountBigDecimal != null )
				outgoingMessageCount = outgoingMessageCountBigDecimal.longValue();
			
			return outgoingMessageCount;
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	
	/**
	 * Ⱓ شϴ OutgoingConmingMessage 
	 * 
	 * @param startTime
	 * @param endTime
	 * @return
	 * @throws EbmsPersistantException
	 */
	List getOutgoingMessagesByPeriod(long startTime, long endTime, int pageCount) throws MoniterPersistantException {
		try {
			int perPageContentCount = 10;
			List outgoingMessageList = null;
			
			DbTemplate dbTemplate = new DbTemplate();
			
			int startCount = pageCount * perPageContentCount - 9;
			int endCount = startCount + 10;
			
			final Object[] params = new Object[] { new Timestamp(startTime), new Timestamp(endTime), new Integer(startCount), new Integer(endCount) };
			
			final String queryStr = 
				"select * from (" +
					   "select /*+ index_desc(" + EbmsConstants.bizboxDatabaseHeaderName + "OutgoingMessage idx_" + EbmsConstants.bizboxDatabaseHeaderName + "OutgoingMessage) */ rownum as numrow, " + EbmsConstants.bizboxDatabaseHeaderName + "OutgoingMessage.* from " + EbmsConstants.bizboxDatabaseHeaderName + "OutgoingMessage where " + EbmsConstants.bizboxDatabaseHeaderName + "OutgoingMessage.SendTime >= ? and " + EbmsConstants.bizboxDatabaseHeaderName + "OutgoingMessage.SendTime <= ?" +
				") where numrow >= ? and numrow < ?";
			
			outgoingMessageList = (List)dbTemplate.executeQuery(
					new DbWorker() {
						
						public Object work(Connection _conn) throws MoniterPersistantException {
							QueryRunner runner = new QueryRunner();
							ResultSetHandler resultSetHandler = new BeanListHandler(OutgoingMessage.class);
							
							List queryResult = null;
							
							try {
								queryResult = (List)runner.query(_conn, queryStr, params, resultSetHandler);
							}
							catch(SQLException e) {
								throw new MoniterPersistantException(0, e.getErrorCode(), e);
							}
							
							return queryResult;
						}
						
					});
			
			return outgoingMessageList;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
	
	
	/**
	 * ð   ðȿ  ޽  ˻մϴ.( Oracle )
	 * 
	 * @param startTime
	 * @param endTime
	 * @param pageCount
	 * @return
	 * @throws EbmsPersistantException
	 */
	long getOutgoingMessageCountByPeriod(long startTime, long endTime) throws MoniterPersistantException {
		try {
			long outgoingMessageCount = 0;
			BigDecimal outgoingMessageCountBigDecimal = null;
			
			DbTemplate dbTemplate = new DbTemplate();
			
			final Object[] params = new Object[] { new Timestamp(startTime), new Timestamp(endTime)};
			final String queryStr = 
				"select count(*) from " + EbmsConstants.bizboxDatabaseHeaderName + "OutgoingMessage where SendTime >= ? and SendTime <= ?";
			
			outgoingMessageCountBigDecimal = (BigDecimal)dbTemplate.executeQuery(
					new DbWorker() {
						
						public Object work(Connection _conn) throws MoniterPersistantException {
							QueryRunner runner = new QueryRunner();
							ResultSetHandler resultSetHandler = new ScalarHandler();
							
							BigDecimal queryResult = null;
							
							try {
								queryResult = (BigDecimal)runner.query(_conn, queryStr, params, resultSetHandler);
							}
							catch(SQLException e) {
								throw new MoniterPersistantException(0, e.getErrorCode(), e);
							}
							
							return queryResult;
						}
						
					});
			
			if( outgoingMessageCountBigDecimal != null )
				outgoingMessageCount = outgoingMessageCountBigDecimal.longValue();
			
			return outgoingMessageCount;
		}
		catch(MoniterPersistantException e) {
			throw e;
		}
		catch(Exception e) {
			throw new MoniterPersistantException(0, 0, e);
		}
	}
}
