package org.kldp.ebmsh.webservice;

import java.util.ArrayList;
import java.util.List;

import org.kldp.ebmsh.cpa.Cpa;
import org.kldp.ebmsh.cpa.CpaManagerFactory;
import org.kldp.ebmsh.cpa.EbmsCpaException;
import org.kldp.ebmsh.cpa.PartyInfo;
import org.kldp.ebmsh.cpa.ejb.CpaManager;


/**
 * File name   - CpaServiceImpl.java
 * Create date - 2007. 06. 12
 * Update time - 2007. 06. 12
 * Version     - v1.0
 *
 * Author      - Administrator
 */
public class CpaServiceImpl implements ICpaService {
	/**
	 * Version     - v1.0
	 * Create time - 2007. 06. 12
	 * Update time - 2007. 06. 12
	 *
	 * Author      - Administrator
	 */
	public void addAction(
			String cpaId, 
			String partyName, String bindingId, String action, String packageId, String[] channelId, 
			String otherPartyName, String otherBindingId, String otherAction, String[] otherChannelId) throws EbmsCpaException {
		

		Cpa cpa = new CpaManagerFactory().createInstance().getCpa(cpaId);

		if( cpa == null )
			throw new EbmsCpaException(0, 0, "Cpa ã  ϴ. CPA ID: " + cpaId);
		
		PartyInfo partyInfo = cpa.getPartyInfo(partyName);
		
		if( partyInfo == null )
			throw new EbmsCpaException(0, 0, "Party ã  ϴ. Party name: " + partyName);
		
		partyInfo.addCanSend(bindingId, action, packageId, channelId, otherBindingId);
		
		PartyInfo otherPartyInfo = cpa.getPartyInfo(otherPartyName);
		
		if( otherPartyInfo == null )
			throw new EbmsCpaException(0, 0, "Other party ã  ϴ. Party name: " + otherPartyName);
		
		otherPartyInfo.addCanReceive(otherBindingId, otherAction, packageId, otherChannelId, bindingId);
		
		cpa.save();
	}
	
	
	/**
	 * Version     - v1.0
	 * Create time - 2007. 06. 22
	 * Update time - 2007. 06. 22
	 *
	 * Author      - Administrator
	 *
	 * @param cpaId
	 * @param packagingId
	 * @param compositeId
	 * @param constituentMessageHeaderIdRef
	 * @param constituentPayloadIdRef
	 * @throws EbmsCpaException
	 * 
	 * Packaging ߰ϴ ޼
	 */
	public void addPackaging(
			String cpaId, 
			String packagingId, String compositeId, String constituentMessageHeaderIdRef, String constituentPayloadIdRef
			) throws EbmsCpaException {
		Cpa cpa = new CpaManagerFactory().createInstance().getCpa(cpaId);
		
		if( cpa == null )
			throw new EbmsCpaException(0, 0, "Cpa ã  ϴ. CPA ID: " + cpaId);
		
		cpa.addPackagingwithComposite(
				packagingId, 
				compositeId, 
				constituentMessageHeaderIdRef, 
				constituentPayloadIdRef);
		
		cpa.save();
	}

	
	/**
	 * Version     - v1.0
	 * Create time - 2007. 06. 12
	 * Update time - 2007. 06. 12
	 *
	 * Author      - Administrator
	 */
	public void createNewCpa(String orgName, String cpaName) {
		
	}

	
	/**
	 * Version     - v1.0
	 * Create time - 2007. 06. 12
	 * Update time - 2007. 06. 12
	 *
	 * Author      - Administrator
	 */
	public List<CanReceive> getCanReceives(String cpaId, String partyName) throws EbmsCpaException {
		List<CanReceive> result = new ArrayList<CanReceive>();
		
		Cpa cpa = new CpaManagerFactory().createInstance().getCpa(cpaId);
		
		if( cpa == null )
			throw new EbmsCpaException(0, 0, "Cpa ã  ϴ. CPA ID: " + cpaId);
		
		PartyInfo partyInfo = cpa.getPartyInfo(partyName);
		
		if( partyInfo == null )
			throw new EbmsCpaException(0, 0, "Party ã  ϴ. Party name: " + partyName);
		
		org.kldp.ebmsh.cpa.CanReceive[] canReceives = partyInfo.getCanReceives();
		
		for(int i = 0; i < canReceives.length; i++) {
			CanReceive canReceive = new CanReceive();
			
			canReceive.setAction(canReceives[i].getAction());
			canReceive.setBindingId(canReceives[i].getBindingId());
			canReceive.setChannelIds(canReceives[i].getChannelIds());
			canReceive.setNonRepudiation(canReceives[i].getNonRepudiation());
			canReceive.setNonRepudiationReceipt(canReceives[i].getNonRepudiationReceipt());
			canReceive.setOtherPartyActionBinding(canReceives[i].getOtherPartyActionBinding());
			canReceive.setPackageId(canReceives[i].getPackageId());
			
			result.add(canReceive);
		}
		
		return result;
	}

	/**
	 * Version     - v1.0
	 * Create time - 2007. 06. 12
	 * Update time - 2007. 06. 12
	 *
	 * Author      - Administrator
	 */
	public List<CanSend> getCanSends(String cpaId, String partyName) throws EbmsCpaException {
		List<CanSend> result = new ArrayList<CanSend>();
		
		Cpa cpa = new CpaManagerFactory().createInstance().getCpa(cpaId);
		
		if( cpa == null )
			throw new EbmsCpaException(0, 0, "Cpa ã  ϴ. CPA ID: " + cpaId);
		
		PartyInfo partyInfo = cpa.getPartyInfo(partyName);
		
		if( partyInfo == null )
			throw new EbmsCpaException(0, 0, "Party ã  ϴ. Party name: " + partyName);
		
		org.kldp.ebmsh.cpa.CanSend[] canSends = partyInfo.getCanSends();
		
		for(int i = 0; i < canSends.length; i++) {
			CanSend canSend = new CanSend();
			
			canSend.setAction(canSends[i].getAction());
			canSend.setBindingId(canSends[i].getBindingId());
			canSend.setChannelIds(canSends[i].getChannelIds());
			canSend.setNonRepudiation(canSends[i].getNonRepudiation());
			canSend.setNonRepudiationReceipt(canSends[i].getNonRepudiationReceipt());
			canSend.setOtherPartyActionBinding(canSends[i].getOtherPartyActionBinding());
			canSend.setPackageId(canSends[i].getPackageId());
			
			result.add(canSend);
		}
		
		return result;
	}

	
	/**
	 * Version     - v1.0
	 * Create time - 2007. 06. 12
	 * Update time - 2007. 06. 12
	 *
	 * Author      - Administrator
	 */
	public List<Certificate> getCertificates(String cpaId, String partyName) throws EbmsCpaException {
		List<Certificate> result = new ArrayList<Certificate>();
		Cpa cpa = new CpaManagerFactory().createInstance().getCpa(cpaId);
		
		if( cpa == null )
			throw new EbmsCpaException(0, 0, "Cpa ã  ϴ. CPA ID: " + cpaId);
		
		PartyInfo partyInfo = cpa.getPartyInfo(partyName);
		
		if( partyInfo == null )
			throw new EbmsCpaException(0, 0, "Party ã  ϴ. Party name: " + partyName);
		
		org.kldp.ebmsh.cpa.Certificate[] certificates = partyInfo.getCertificates();
		
		for(int i = 0; i < certificates.length; i++) {
			Certificate certificate = new Certificate();
			
			certificate.setCertId( certificates[i].getCertId() );
			certificate.setKeyNames( certificates[i].getKeyNames() );
			
			result.add(certificate);
		}
		
		return result;
	}

	
	/**
	 * Version     - v1.0
	 * Create time - 2007. 06. 12
	 * Update time - 2007. 06. 12
	 *
	 * Author      - Administrator
	 */
	public String[] getDeliveryChannelIds(String cpaId, String partyName) throws EbmsCpaException {
		Cpa cpa = new CpaManagerFactory().createInstance().getCpa(cpaId);
		
		if( cpa == null )
			throw new EbmsCpaException(0, 0, "Cpa ã  ϴ. CPA ID: " + cpaId);
		
		PartyInfo partyInfo = cpa.getPartyInfo(partyName);
		
		if( partyInfo == null )
			throw new EbmsCpaException(0, 0, "Party ã  ϴ. Party name: " + partyName);
		
		return partyInfo.getDeliveryChannelIds();
	}

	/**
	 * Version     - v1.0
	 * Create time - 2007. 06. 12
	 * Update time - 2007. 06. 12
	 *
	 * Author      - Administrator
	 */
	public List<String> getCpaIds() throws EbmsCpaException {
		return new CpaManagerFactory().createInstance().getCpaIds();
	}

	
	/**
	 * Version     - v1.0
	 * Create time - 2007. 06. 12
	 * Update time - 2007. 06. 12
	 *
	 * Author      - Administrator
	 */
	public String[] getDefaultEndpoints(String cpaId, String partyName) throws EbmsCpaException {
		Cpa cpa = new CpaManagerFactory().createInstance().getCpa(cpaId);
		
		if( cpa == null )
			throw new EbmsCpaException(0, 0, "Cpa ã  ϴ. CPA ID: " + cpaId);
		
		PartyInfo partyInfo = cpa.getPartyInfo(partyName);
		
		if( partyInfo == null )
			throw new EbmsCpaException(0, 0, "Party ã  ϴ. Party name: " + partyName);
		
		return partyInfo.getEndpointsWithDefaultChannelUnderTransport();
	}

	
	/**
	 * Version     - v1.0
	 * Create time - 2007. 06. 12
	 * Update time - 2007. 06. 12
	 *
	 * Author      - Administrator
	 */
	public String[] getPackageIds(String cpaId) throws EbmsCpaException {		
		Cpa cpa = new CpaManagerFactory().createInstance().getCpa(cpaId);
		
		return cpa.getPackagingIds();
	}

	/**
	 * Version     - v1.0
	 * Create time - 2007. 06. 12
	 * Update time - 2007. 06. 12
	 *
	 * Author      - Administrator
	 * 
	 * CpaId ϴ Party̸  ɴϴ.
	 */
	public List<String> getPartyNames(String cpaId) throws EbmsCpaException {
		List<String> partyInfoList = new ArrayList<String>();
		
		Cpa cpa = new CpaManagerFactory().createInstance().getCpa(cpaId);
		
		if( cpa == null )
			throw new EbmsCpaException(0, 0, "CPA ã  ϴ. CPA ID: " + cpaId);
		
		PartyInfo[] partyInfoArray = cpa.getPartyInfos();
		
		for(int i = 0; i < partyInfoArray.length; i++) {
			PartyInfo partyInfo = partyInfoArray[i];
			
			partyInfoList.add(partyInfo.getPartyName());
		}
		
		return partyInfoList;
	}

	
	/**
	 * Version     - v1.0
	 * Create time - 2007. 06. 12
	 * Update time - 2007. 06. 12
	 *
	 * Author      - Administrator
	 */
	public void setCertificate(String cpaid, String partyName,
			String _oldCertId, String _newCertId, int _keySeq, String _keyName,
			int _certSeq, String _x509Certificate) {
		

	}

	public void setCpaId(String orgCpaId, String newCpaId) {
		/**
		 * Version     - v1.0
		 * Create time - 2007. 06. 12
		 * Update time - 2007. 06. 12
		 *
		 * Author      - Administrator
		 */

	}

	public void setEndpoints(String cpaid, String partyName, String endpoint,
			String _deliveryChannelId) {
		/**
		 * Version     - v1.0
		 * Create time - 2007. 06. 12
		 * Update time - 2007. 06. 12
		 *
		 * Author      - Administrator
		 */

	}

	public void setPartyName(String cpaid, String orgName, String newName) {
		/**
		 * Version     - v1.0
		 * Create time - 2007. 06. 12
		 * Update time - 2007. 06. 12
		 *
		 * Author      - Administrator
		 */

	}

}
